
CREATE DATABASE /*!32312 IF NOT EXISTS*/ metricstore /*!40100 DEFAULT CHARACTER SET latin1 */;

USE metricstore; 

CREATE  TABLE IF NOT EXISTS `metricstatus` (
  `id` INT NOT NULL ,
  `description` VARCHAR(45) NULL ,
  `description_nagios` VARCHAR(45) NULL ,
  `comment` VARCHAR(45) NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdetails` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `detail` TEXT NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `metricstatus_id` INT NOT NULL ,
  `metricdetail_id` INT NULL ,
  `metric_id` INT,
  `service_id` INT,
  `vo_id` INT,
  `summarydata` VARCHAR(256) NULL ,
  `check_time` INT NOT NULL ,
  `insert_time` INT NOT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_metricdata_sam_metrics` (`metric_id` ASC) ,
  INDEX `fk_metricdata_sam_nodes` (`service_id` ASC) ,
  INDEX `fk_metricdata_sam_vos` (`vo_id` ASC) ,
  INDEX `fk_metricresults_metricstatus` (`metricstatus_id` ASC) ,
  INDEX `fk_metricdata_metricdetails` (`metricdetail_id` ASC) ,
  INDEX `fk_metricdata_check_time` (`check_time` ASC) ,

  CONSTRAINT `fk_metricdata_sam_metrics`
    FOREIGN KEY (`metric_id` )
    REFERENCES `mddb`.`metric` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `atp`.`vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


CREATE  TABLE IF NOT EXISTS `statuschange_service` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `service_id` INT NULL ,
  `metricstatus_id` INT NOT NULL DEFAULT 0 ,
  `detail` VARCHAR(256) NOT NULL ,
  `timestamp` INT NOT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_statuschange_servicetype_metricstatus` (`metricstatus_id` ASC) ,
  INDEX `fk_statuschange_servicetype_services` (`service_id` ASC) ,
  CONSTRAINT `fk_statuschange_servicetype_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_servicetype_services`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `statuschange_service_profile` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `service_id` INT NULL ,
  `profile_id` INT NULL ,
  `metricstatus_id` INT NOT NULL DEFAULT 0 ,
  `detail` VARCHAR(256) NOT NULL ,
  `timestamp` INT NOT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_statuschange_service_p_metricstatus` (`metricstatus_id` ASC) ,
  INDEX `fk_statuschange_service_p_services` (`service_id` ASC) ,
  INDEX `fk_statuschange_service_p_profiles` (`profile_id` ASC) ,

  CONSTRAINT `fk_statuschange_service_p_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_services`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_profiles`
    FOREIGN KEY (`profile_id` )
    REFERENCES `mddb`.`profile` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)

ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata_latest` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `metricstatus_id` INT NOT NULL ,
  `metricdetail_id` INT NULL ,
  `metric_id` INT NULL ,
  `service_id` INT NULL ,
  `vo_id` INT NULL ,
  `summarydata` VARCHAR(256) NULL ,
  `check_time` INT NOT NULL ,
  `insert_time` INT NOT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_metricdata_latest_metrics` (`metric_id` ASC) ,
  INDEX `fk_metricdata_latest_nodes` (`service_id` ASC) ,
  INDEX `fk_metricdata_latest_vos` (`vo_id` ASC) ,
  INDEX `fk_metricresults_metricstatus_latest` (`metricstatus_id` ASC) ,
  INDEX `fk_metricdata_metricdetails_latest` (`metricdetail_id` ASC) ,
  INDEX `fk_metricdata_check_time_latest` (`check_time` ASC) ,

  CONSTRAINT `fk_metricdata_latest_metrics`
    FOREIGN KEY (`metric_id` )
    REFERENCES `mddb`.`metric` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `atp`.`vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus_latest`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails_latest`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;



CREATE  TABLE IF NOT EXISTS `metricdata_rejected` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `metricstatus` VARCHAR(25),
  `metric` VARCHAR(256),
  `service` VARCHAR(256),
  `service_flavour` VARCHAR(256),
  `vo` VARCHAR(256),
  `summarydata` TEXT NULL,
  `check_time` INT NOT NULL ,
  `insert_time` INT NOT NULL ,
  `reason` VARCHAR(256),
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

INSERT INTO metricstatus(id, description,
description_nagios, comment) VALUES (1, 'OK','OK',NULL),(2,
'WARNING','WARNING',NULL),(3, 'CRITICAL','CRITICAL',NULL),(4,
'UNKNOWN','UNKNOWN',NULL), (5,'MISSING', '', 'Automatically calculate by Metric Store');

