DELIMITER //

USE `metricstore`;

DROP PROCEDURE IF EXISTS loadmetricdata;

CREATE PROCEDURE loadmetricdata(in_serviceflavour VARCHAR(100), in_metricname VARCHAR(100), in_metricstatus VARCHAR(10), 
                 in_performancedata VARCHAR(255), in_summarydata VARCHAR(255), in_detailsdata TEXT, in_voname VARCHAR(100), 
                 in_hostname VARCHAR(255), in_timestamp DATETIME)

BEGIN


  DECLARE v_serviceflavour_name VARCHAR(100);
  DECLARE v_serviceflavour_id INT;
  DECLARE v_metricstatus_id INT;
  DECLARE v_metric_id INT;
  DECLARE v_vo_id INT;
  DECLARE v_service_id INT;
  DECLARE v_metricdetail_id INT;
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(100);
  DECLARE done INT DEFAULT 0;
  DECLARE flavourCurs CURSOR FOR select name from tmpServiceFlavourList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpServiceFlavourList;
  CREATE TEMPORARY TABLE tmpServiceFlavourList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;



  SET intFoundPos = INSTR(in_serviceflavour,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_serviceflavour, 1, intFoundPos-1);
    SET in_serviceflavour = REPLACE(in_serviceflavour, CONCAT(strElement,','), '');
    INSERT INTO tmpServiceFlavourList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_serviceflavour,',');
  END WHILE;

  IF in_serviceflavour <> '' THEN
    INSERT INTO tmpServiceFlavourList (`name`) VALUES (in_serviceflavour);
  END IF; 


  OPEN flavourCurs;

  REPEAT 
  
    FETCH flavourCurs INTO v_serviceflavour_name;

    IF NOT done THEN

      select id into v_serviceflavour_id from atp.service_type_flavour where lower(flavourname) = lower(v_serviceflavour_name);
      select id into v_metric_id from mddb.metric where lower(name) = lower(in_metricname);
      select id into v_metricstatus_id from metricstatus where lower(description) = lower(in_metricstatus);
      select id into v_vo_id from atp.vo where lower(voname) = lower(in_voname);
      select id into v_service_id from atp.service where lower(hostname) = lower(in_hostname) and flavour_id = v_serviceflavour_id;
    
    
    
      IF v_service_id is not NULL and v_metric_id is not NULL THEN
        if in_detailsdata is not null then
          insert into metricdetails(detail)
          values (in_detailsdata);
      
          select last_insert_id() into v_metricdetail_id;
      
        end if;
    
    
        DELETE FROM metricdata_latest WHERE vo_id <=> v_vo_id and metric_id <=> v_metric_id and service_id <=> v_service_id;
    
        insert into metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, vo_id, summarydata, check_time, insert_time)
        VALUES (v_metricstatus_id, v_metricdetail_id, v_metric_id, v_service_id, v_vo_id, in_summarydata, UNIX_TIMESTAMP(in_timestamp), UNIX_TIMESTAMP());
     
      ELSEIF (v_service_id is NULL and v_metric_id is NULL) THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, summarydata, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_summarydata, UNIX_TIMESTAMP(in_timestamp), UNIX_TIMESTAMP(), 'service_id and metric_id NULL');
      ELSEIF v_service_id is NULL THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, summarydata, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_summarydata, UNIX_TIMESTAMP(in_timestamp), UNIX_TIMESTAMP(), 'service_id NULL');
      ELSEIF v_metric_id is NULL THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, summarydata, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_summarydata, UNIX_TIMESTAMP(in_timestamp), UNIX_TIMESTAMP(), 'metric_id NULL');

      END IF;

   END IF;

  UNTIL done END REPEAT;

  CLOSE flavourCurs;


  commit;
END //

DROP PROCEDURE IF EXISTS calculateStatusChange;

CREATE PROCEDURE calculateStatusChange(serviceId INT, metricId INT, metricstatusId INT, metricdataId INT, check_time INT, voId INT)
BEGIN

  DECLARE v_profileId INT;

  DECLARE done INT DEFAULT 0;
  DECLARE profileCurs CURSOR FOR select id from mddb.profile;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;


  OPEN profileCurs;

  REPEAT

    FETCH profileCurs INTO v_profileId;

    IF NOT done THEN
      IF checkIfCalculationNeeded(v_profileId, serviceId, metricId, voId) THEN
        call calculateStatusChangeProfile(v_profileId, serviceId, metricId, metricstatusId, metricdataId, check_time, voId);
        insert into tested_services (service_id, profile_id, last_insert_time)
        values(serviceId, v_profileId, unix_timestamp())
        on duplicate key update last_insert_time = unix_timestamp();

      END IF;
    END IF;
      
  UNTIL done END REPEAT;
 

  CLOSE profileCurs;

END //


DROP PROCEDURE IF EXISTS calculateStatusChangeProfile;

CREATE PROCEDURE calculateStatusChangeProfile(profileId INT, serviceId INT, metricId INT, metricstatusId INT, metricdataId INT, check_time INT, voId INT)

BEGIN
  DECLARE old_metricstatus_id INTEGER; 
  DECLARE old_timestamp INTEGER; 
  DECLARE num_row INTEGER;
  DECLARE curr_servicestatus_id INTEGER;

  SET num_row = 0;

  select count(*) INTO num_row
  from statuschange_service_profile 
  where service_id = serviceId and profile_id = profileId;

  IF num_row > 0 THEN
    SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_service_profile 
    WHERE service_id = serviceId AND profile_id = profileId AND timestamp =
      (SELECT max(timestamp) FROM statuschange_service_profile
      WHERE service_id = serviceId AND profile_id = profileId and timestamp <= check_time);

    IF (metricstatusId < old_metricstatus_id) THEN
        select max(metricstatus_id) INTO curr_servicestatus_id from metricdata_latest where metric_id in (
          SELECT m.metric_id FROM mddb.profile_metric_map m
            where m.profile_id = profileId 
              and (test_vo_id = voId or voId is null)
              and m.service_type_flavour_id = (select flavour_id from atp.service l where l.id = serviceId)
          ) 
          and service_id = serviceId;
    END IF;


    IF (metricstatusId > old_metricstatus_id) THEN
      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_service_profile(service_id, profile_id, metricstatus_id, detail, timestamp)
        VALUES(serviceId, profileId, metricstatusId, concat('Status changed by metricdata(1): ', '', metricdataId, ''), check_time);
      ELSE
         UPDATE statuschange_service_profile SET metricstatus_id = metricstatusId, detail = concat('Status changed by metricdata(2): ', '', metricdataId, '') WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
      END IF;

    ELSEIF ((metricstatusId < old_metricstatus_id) AND (curr_servicestatus_id < old_metricstatus_id)) THEN

      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_service_profile(service_id, profile_id, metricstatus_id, detail, timestamp)
        VALUES(serviceId, profileId, metricstatusId, concat('Status changed by metricdata(3): ', '', metricdataId, ''), check_time);
      ELSE
         UPDATE statuschange_service_profile SET metricstatus_id = curr_servicestatus_id, detail = concat('Status changed by metricdata(4): ', '', metricdataId, '') WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
      END IF;

    END IF;
  ELSE 
    INSERT INTO statuschange_service_profile(service_id, profile_id, metricstatus_id, detail, timestamp)
    VALUES(serviceId, profileId, metricstatusId, concat('Status changed by metricdata  : ', '', metricdataId, ''), check_time); 
  END IF;

END //

DROP FUNCTION IF EXISTS checkIfCalculationNeeded;

CREATE FUNCTION checkIfCalculationNeeded(profileId INT, serviceId INT, metricId INT, voId INT)
RETURNS INT
BEGIN

  DECLARE v_counter INT;

  DECLARE metricCurs CURSOR FOR SELECT count(1) FROM mddb.profile_metric_map 
    where profile_id = profileId and (test_vo_id = voId or voId is null) and metric_id = metricId and service_type_flavour_id =
    (select flavour_id from atp.service where id = serviceId);

  
  OPEN metricCurs;
  FETCH metricCurs INTO v_counter;
  CLOSE metricCurs;

  IF (v_counter > 0) THEN
    RETURN 1;
  ELSE 
    RETURN 0;
  END IF;

END //

DROP PROCEDURE IF EXISTS insertMissingMetrics;

CREATE PROCEDURE insertMissingMetrics(maxAgeInHours INT)
BEGIN

  DECLARE v_id INT;
  DECLARE v_metric_id INT;
  DECLARE v_service_id INT;
  DECLARE v_vo_id INT;

  DECLARE done INT DEFAULT 0;
  DECLARE oldMetricsCurs CURSOR FOR select id, metric_id, service_id, vo_id from metricdata_latest
    where check_time <=  UNIX_TIMESTAMP(now() - interval maxAgeInHours hour)
    and service_id in (select id from atp.service where isdeleted = 'N');

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  OPEN oldMetricsCurs;

  REPEAT
    FETCH oldMetricsCurs INTO v_id, v_metric_id, v_service_id, v_vo_id;

    IF NOT done THEN
      DELETE FROM metricdata_latest WHERE id = v_id;
      insert into metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, vo_id, summarydata, check_time, insert_time)
        VALUES (5, NULL, v_metric_id, v_service_id, v_vo_id, NULL, unix_timestamp(), unix_timestamp());

    END IF;
  UNTIL done END REPEAT;

  CLOSE oldMetricsCurs;

  commit;

END //

DROP PROCEDURE IF EXISTS purgeMetricStore;

CREATE PROCEDURE purgeMetricStore(maxAgeInDays INT)
BEGIN

  DELETE FROM statuschange_service_profile
    WHERE timestamp <  UNIX_TIMESTAMP(now() - interval maxAgeInDays day);

  DELETE FROM metricdata WHERE insert_time < UNIX_TIMESTAMP(now() - interval maxAgeInDays day);

  /*Deleting data from metricdata_latest isn't neeeded since insertMissingMetrics will create "MISSING" metrics. 
    If we switch off insertMissingMetrics, we would have to delete entries from tested_services;
  */ 
  DELETE FROM metricdata_latest WHERE insert_time < UNIX_TIMESTAMP(now() - interval maxAgeInDays day);

  COMMIT;

END //

DROP PROCEDURE IF EXISTS addSupportedVos;

CREATE PROCEDURE addSupportedVos(in_VoList VARCHAR(255))
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_VoName VARCHAR(255);
  DECLARE v_vo_id INT;

  DECLARE done INT DEFAULT 0;
  DECLARE voCurs CURSOR FOR select name from tmpVoList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpVoList;
  CREATE TEMPORARY TABLE tmpVoList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_VoList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_VoList, 1, intFoundPos-1);
    SET in_VoList = REPLACE(in_VoList, CONCAT(strElement,','), '');
    INSERT INTO tmpVoList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_VoList,',');
  END WHILE;

  IF in_VoList <> '' THEN
    INSERT INTO tmpVoList (`name`) VALUES (in_VoList);
  END IF; 

  OPEN voCurs;

  REPEAT 
  
    FETCH voCurs INTO v_VoName;

    select id into v_vo_id from atp.vo where lower(voname) = lower(v_VoName);

    IF NOT done THEN
      insert into supportedVos (vo_id, insert_time)
      values(v_vo_id, unix_timestamp())
       on duplicate key update vo_id = v_vo_id;

    END IF;

  UNTIL done END REPEAT;

  CLOSE voCurs;

  COMMIT;

END //


DELIMITER ;
