USE mrs; 

create table if not exists dataloader_lastchecktime
(
  check_time int not null,
  constraint uk_dataloader_lastchecktime unique (check_time)
) ENGINE = InnoDB;

create table if not exists gatheredat
(
  id int auto_increment not null,
  name varchar(256),
  primary key(id)  
) ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS fqan (
  id INT NOT NULL AUTO_INCREMENT ,
  name VARCHAR(256) NOT NULL ,
  PRIMARY KEY (id) )
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricstatus` (
  `id` INT NOT NULL,
  `weight` INT NOT NULL,
  `description` VARCHAR(45) NULL,
  `description_nagios` VARCHAR(45) NULL,
  `comments` VARCHAR(45) NULL,
  PRIMARY KEY (`id`),
  CONSTRAINT uk_metricstatus_weight UNIQUE(weight))
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdetails` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `detail` TEXT NOT NULL,
  timestamp INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX metricdetails_timestamp (timestamp ASC))
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan_id` INT,
  `gatheredat_id` INT,
  `metricstatus_id` INT NOT NULL,
  `metricdetail_id` INT NULL,
  `metric_id` INT,
  `service_id` INT,
  `vo_id` INT,
  `summarydata` VARCHAR(256) NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  poem_sync_metric_id INT,
  PRIMARY KEY (`id`),
  INDEX `fk_metricdata_fqan_id` (`fqan_id` ASC),
  INDEX `fk_metricdata_gatheredat_id` (`gatheredat_id` ASC),
  INDEX `fk_metricdata_sam_metrics` (`metric_id` ASC),
  INDEX `fk_metricdata_sam_nodes` (`service_id` ASC),
  INDEX `fk_metricdata_sam_vos` (`vo_id` ASC),
  INDEX `fk_metricresults_metricstatus` (`metricstatus_id` ASC),
  INDEX `fk_metricdata_metricdetails` (`metricdetail_id` ASC),
  INDEX `fk_metricdata_check_time` (`check_time` ASC),
  INDEX metricdata_p_s_metric_id_idx (poem_sync_metric_id ASC),
  CONSTRAINT `fk_metricdata_fqan_id`
    FOREIGN KEY (`fqan_id`)
    REFERENCES `fqan` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_gatheredat_id`
    FOREIGN KEY (`gatheredat_id`)
    REFERENCES `gatheredat` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_metrics`
    FOREIGN KEY (`metric_id` )
    REFERENCES `metric` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id))
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS metricdata_spool (
  id INT NOT NULL AUTO_INCREMENT,
  fqan_id INT,
  gatheredat_id INT,
  metricstatus_id INT NOT NULL,
  detailsdata text,
  metric_id INT NOT NULL,
  service_id INT NOT NULL,
  vo_id INT,
  summarydata VARCHAR(256) NULL,
  check_time INT NOT NULL,
  insert_time INT NOT NULL,
  poem_sync_metric_id INT,
  PRIMARY KEY (id),
  INDEX fk_metricdata_spool_fqan_id (fqan_id ASC),
  INDEX fk_metricdata_sp_gatheredat_id (gatheredat_id ASC),
  INDEX fk_metricdata_sp_metrics (metric_id ASC),
  INDEX fk_metricdata_sp_nodes (service_id ASC),
  INDEX fk_metricdata_sp_vos (vo_id ASC),
  INDEX fk_metricidata_sp_metricstatus (metricstatus_id ASC),
  INDEX fk_metricdata_sp_check_time (check_time ASC),
  CONSTRAINT fk_metricdata_spool_fqan_id
    FOREIGN KEY (fqan_id)
    REFERENCES fqan (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_gatheredat_id 
    FOREIGN KEY (gatheredat_id)
    REFERENCES gatheredat (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_metrics
    FOREIGN KEY (metric_id )
    REFERENCES metric (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_nodes
    FOREIGN KEY (service_id)
    REFERENCES service (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_vos
    FOREIGN KEY (vo_id)
    REFERENCES vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_metricstatus
    FOREIGN KEY (metricstatus_id)
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_s_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id)
) ENGINE = InnoDB ;

CREATE TABLE IF NOT EXISTS `statuschange_service_profile` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan_id` INT NULL,
  `service_id` INT NULL,
  `profile_id` INT NULL,
  `metricstatus_id` INT NOT NULL DEFAULT 0,
  `detail` VARCHAR(256) NULL,
  `timestamp` INT NOT NULL,
  `insert_time` INT NOT NULL,
  is_considered_ace enum('N', 'Y', 'C') default 'N' NOT NULL,
  poem_sync_profile_id INT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_stch_serv_profile_fqan_id` (`fqan_id` ASC),
  INDEX `fk_statuschange_service_p_metricstatus` (`metricstatus_id` ASC),
  INDEX `fk_statuschange_service_p_services` (`service_id` ASC),
  INDEX `fk_statuschange_service_p_profiles` (`profile_id` ASC),
  INDEX statuschange_service_prof_b (is_considered_ace ASC),  
  INDEX statuschg_s_p_poem_s_prof_idx (poem_sync_profile_id ASC),
  CONSTRAINT `fk_stch_serv_profile_fqan_id`
    FOREIGN KEY (`fqan_id`)
    REFERENCES `fqan` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `uk_statuschange_service_profile`
    UNIQUE KEY (`service_id`, `profile_id`, `timestamp`),
  CONSTRAINT `fk_statuschange_service_p_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_services`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_profiles`
    FOREIGN KEY (`profile_id` )
    REFERENCES `profile` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_stchg_service_p_poem_prof
    FOREIGN KEY (poem_sync_profile_id)
    REFERENCES poem_sync_profile (id))

ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata_latest` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan_id` INT,
  `gatheredat_id` INT,
  `metricstatus_id` INT NOT NULL,
  `metricdetail_id` INT NULL,
  `metric_id` INT NULL,
  `service_id` INT NULL,
  `vo_id` INT NULL,
  `summarydata` VARCHAR(256) NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  poem_sync_metric_id INT,
  PRIMARY KEY (`id`),
  INDEX `fk_metricdata_latest_fqan_id` (`fqan_id` ASC),
  INDEX `fk_metricdata_latest_gatheredat_id` (`gatheredat_id` ASC),
  INDEX `fk_metricdata_latest_metrics` (`metric_id` ASC),
  INDEX `fk_metricdata_latest_nodes` (`service_id` ASC),
  INDEX `fk_metricdata_latest_vos` (`vo_id` ASC),
  INDEX `fk_metricresults_metricstatus_latest` (`metricstatus_id` ASC),
  INDEX `fk_metricdata_metricdetails_latest` (`metricdetail_id` ASC),
  INDEX `fk_metricdata_check_time_latest` (`check_time` ASC),
  INDEX metricdata_l_p_s_metric_id_idx (poem_sync_metric_id ASC),
  CONSTRAINT `fk_metricdata_latest_fqan_id`
    FOREIGN KEY (`fqan_id`)
    REFERENCES `fqan` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_gatheredat_id` 
    FOREIGN KEY  (`gatheredat_id`)
    REFERENCES `gatheredat` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_metrics`
    FOREIGN KEY (`metric_id` )
    REFERENCES `metric` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus_latest`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails_latest`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_l_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id))
ENGINE = InnoDB;



CREATE  TABLE IF NOT EXISTS `metricdata_rejected` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan` VARCHAR(256),
  `gatheredat` VARCHAR(255),
  `metricstatus` VARCHAR(25),
  `metric` VARCHAR(256),
  `service` VARCHAR(256),
  `service_flavour` VARCHAR(256),
  `vo` VARCHAR(256),
  `summarydata` TEXT NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  `reason` VARCHAR(256),
  PRIMARY KEY (`id`),
  INDEX `mdata_rej_checktime_idx` (`check_time` ASC))
ENGINE = InnoDB;

INSERT INTO metricstatus(id, weight, description, description_nagios, comments) VALUES 
(1, 1, 'OK', 'OK', NULL),
(2, 2, 'WARNING', 'WARNING', NULL),
(3, 3, 'UNKNOWN', 'UNKNOWN', NULL), 
(4, 4, 'MISSING', '', 'Automatically calculate by Metric Store'),
(5, 5, 'CRITICAL', 'CRITICAL', NULL),
(6, 6, 'REMOVED', '', 'Service removed');

CREATE  TABLE IF NOT EXISTS `tested_services` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `service_id` INT NOT NULL,
  `profile_id` INT NOT NULL,
  `last_insert_time` INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_tested_services_service` (`service_id` ASC),
  INDEX `fk_tested_services_profile` (`profile_id` ASC),
  INDEX `tested_services_time` (`last_insert_time` ASC),

  CONSTRAINT `uk_tested_services`
    UNIQUE KEY (`service_id`, `profile_id`),
  CONSTRAINT `fk_tested_services_service`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_tested_services_profile`
    FOREIGN KEY (`profile_id` )
    REFERENCES `profile` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)

ENGINE = InnoDB;

CREATE TABLE  IF NOT EXISTS schema_details (
     id INT AUTO_INCREMENT NOT NULL,
     ver VARCHAR(20) NOT NULL,
     creation_date DATETIME NOT NULL,
     remarks VARCHAR(1000) BINARY NULL,
     PRIMARY KEY (id)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_vos (
  id INT AUTO_INCREMENT NOT NULL,
  name VARCHAR(256) NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  CONSTRAINT `uk_supported_vos`
    UNIQUE KEY (`name`)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_profiles (
  id INT AUTO_INCREMENT NOT NULL,
  name VARCHAR(256) NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  CONSTRAINT `uk_supported_profiles`
    UNIQUE KEY (`name`)
)
ENGINE=INNODB;


CREATE  TABLE IF NOT EXISTS metrics_in_supported_services (
  id INT NOT NULL AUTO_INCREMENT ,
  service_id INT NOT NULL ,
  metric_id INT NOT NULL ,
  vo_id INT NULL ,
  fqan_id INT NULL ,
  profile_id INT NOT NULL ,
  frequency INT,
  timestamp INT NULL,
  updated tinyint(1) NULL,
  PRIMARY KEY (id),
  UNIQUE INDEX uk_supported_services (service_id ASC, metric_id ASC, vo_id ASC, fqan_id ASC, profile_id ASC),
  INDEX fk_metrics_in_supp_service (service_id ASC),
  INDEX fk_metrics_in_supp_metric (metric_id ASC),
  INDEX fk_metrics_in_supp_vo (vo_id ASC),
  INDEX fk_metrics_in_supp_fqan (fqan_id ASC),
  INDEX fk_metrics_in_supp_profile (profile_id ASC),
  CONSTRAINT fk_metrics_in_supp_service
    FOREIGN KEY (service_id )
    REFERENCES service (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_metric
    FOREIGN KEY (metric_id )
    REFERENCES metric (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_vo
    FOREIGN KEY (vo_id )
    REFERENCES vo (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_fqan
    FOREIGN KEY (fqan_id )
    REFERENCES fqan (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_profile
    FOREIGN KEY (profile_id )
    REFERENCES profile (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
 )
ENGINE = InnoDB;
CREATE TABLE IF NOT EXISTS metrics_in_supp_serv_history (
  id INT NOT NULL AUTO_INCREMENT,
  service_id INT NOT NULL,
  metric_id INT NOT NULL,
  vo_id INT NULL,
  fqan_id INT NULL,
  profile_id INT NOT NULL,
  operation ENUM ('insert', 'delete') NOT NULL,
  timestamp INT NULL,
  PRIMARY KEY (id),
  UNIQUE INDEX uk_supported_services_hist (service_id ASC, metric_id ASC, vo_id ASC, fqan_id ASC, profile_id ASC, operation ASC, timestamp ASC),
  INDEX fk_metrics_in_supp_h_service (service_id ASC),
  INDEX fk_metrics_in_supp_h_metric (metric_id ASC),
  INDEX fk_metrics_in_supp_h_vo (vo_id ASC),
  INDEX fk_metrics_in_supp_h_fqan (fqan_id ASC),
  INDEX fk_metrics_in_supp_h_profile (profile_id ASC),
  CONSTRAINT fk_metrics_in_supp_h_service
    FOREIGN KEY (service_id )
    REFERENCES service (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_h_metric
    FOREIGN KEY (metric_id )
    REFERENCES metric (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_h_vo
    FOREIGN KEY (vo_id )
    REFERENCES vo (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_h_fqan
    FOREIGN KEY (fqan_id )
    REFERENCES fqan (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metrics_in_supp_h_profile
    FOREIGN KEY (profile_id )
    REFERENCES profile (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
 )
ENGINE = InnoDB;
CREATE  TABLE IF NOT EXISTS metrics_in_supp_serv_rejected (
  id INT NOT NULL AUTO_INCREMENT ,
  service_type_flavour VARCHAR(256) NULL ,
  hostname VARCHAR(256) NULL ,
  metric_name VARCHAR(256) NULL ,
  vo VARCHAR(256) NULL ,
  fqan VARCHAR(256) NULL ,
  profile VARCHAR(256) NULL ,
  frequency INT,
  timestamp INT NULL,
  reason VARCHAR(256) NULL,
  PRIMARY KEY (id)
 )
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS statuschange_metric_service (
  id INT NOT NULL AUTO_INCREMENT ,
  metric_id INT NOT NULL ,
  service_id INT NOT NULL ,
  vo_id INT NULL ,
  metricstatus_id INT NOT NULL ,
  timestamp INT NOT NULL ,
  insert_time INT NOT NULL,
  poem_sync_metric_id INT NULL,
  PRIMARY KEY (id) ,
  UNIQUE INDEX uk_statuschange_metric_service (service_id ASC, metric_id ASC, vo_id ASC, timestamp ASC) ,
  INDEX fk_statuschange_metric_s_metrics (metric_id ASC),
  INDEX fk_statuschange_metric_s_metricstatus (metricstatus_id ASC),
  INDEX fk_statuschange_metric_s_services (service_id ASC),
  INDEX fk_statuschange_metric_s_vos (vo_id ASC),
  CONSTRAINT fk_statuschange_metric_s_metrics
    FOREIGN KEY (metric_id )
    REFERENCES metric (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_statuschange_metric_s_metricstatus
    FOREIGN KEY (metricstatus_id )
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_statuschange_metric_s_services
    FOREIGN KEY (service_id )
    REFERENCES service (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_stchg_metric_s_poem_serv
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric (id),
  CONSTRAINT fk_statuschange_metric_s_vos
    FOREIGN KEY (vo_id)
    REFERENCES vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS fqans_in_supported_services (
  id INT NOT NULL AUTO_INCREMENT,
  profile_id INT NOT NULL,
  service_id INT NOT NULL,
  fqan_id INT NOT NULL,
  timestamp INT NOT NULL,
  updated TINYINT(1) NULL ,
  PRIMARY KEY (id) ,
  UNIQUE INDEX uk_fqan_in_supp_serv(profile_id ASC, service_id ASC, fqan_id ASC),
  CONSTRAINT fk_fqans_in_supp_serv_fqan
    FOREIGN KEY (fqan_id)
    REFERENCES fqan(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_fqans_in_supp_serv_profile
    FOREIGN KEY (profile_id)
    REFERENCES profile(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_fqans_in_supp_serv_service
    FOREIGN KEY (service_id)
    REFERENCES service(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)

ENGINE = InnoDB;

create or replace view fqans_in_regions as
  select distinct get_roc_name(service_id) as roc_name, fqan_id from fqans_in_supported_services;

CREATE TABLE IF NOT EXISTS fqans_in_supp_serv_history(
  id INT NOT NULL AUTO_INCREMENT,
  profile_id INT NOT NULL,
  service_id INT NOT NULL,
  fqan_id INT NOT NULL,
  operation ENUM ('insert', 'delete') NOT NULL,
  timestamp INT NOT NULL,
  PRIMARY KEY (id) ,
  UNIQUE INDEX uk_fqan_in_supp_serv_h(profile_id ASC, service_id ASC, fqan_id ASC, operation ASC, timestamp ASC),
  CONSTRAINT fk_fqans_in_supp_serv_h_fqan
    FOREIGN KEY (fqan_id)
    REFERENCES fqan(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_fqans_in_supp_serv_h_profile
    FOREIGN KEY (profile_id)
    REFERENCES profile(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_fqans_in_supp_serv_h_service
    FOREIGN KEY (service_id)
    REFERENCES service(id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS fqans_in_supp_serv_rejected (
  id INT NOT NULL AUTO_INCREMENT ,
  service_type_flavour VARCHAR(256) NULL ,
  hostname VARCHAR(256) NULL ,
  fqan VARCHAR(256) NULL ,
  profile VARCHAR(256) NULL ,
  timestamp INT NULL,
  reason VARCHAR(256) NULL,
  PRIMARY KEY (id)
 )
ENGINE = InnoDB;

create or replace view metric_history_in_profile as
SELECT a.name profile_name, d.voname defined_by_vo, l.hostname, f.serviceabbr service_type,
  e.flavourname service_flavour, h.check_time metric_exec_time,
  c.name metric_name, j.name fqan, g.voname tested_by_vo,
  i.description metric_status, h.summarydata metric_output_summary,
  k.name gathered_at, h.insert_time metric_result_insert_time
FROM profile a, profile_metric_map b, metric c, vo d, service_type_flavour e,
  service_type f, vo g, metricdata h, metricstatus i, fqan j, gatheredat k, service l
WHERE b.profile_id=a.id AND b.metric_id=c.id AND a.defining_vo_id=d.id
  AND b.metric_id=h.metric_id AND h.metricstatus_id=i.id AND h.service_id=l.id
  AND l.flavour_id=e.id AND b.service_type_flavour_id=l.flavour_id
  AND e.service_type_id=f.id AND b.test_vo_id=g.id AND h.vo_id=g.id
  AND h.fqan_id=j.id AND h.gatheredat_id=k.id;

CREATE OR REPLACE VIEW statuschange_service_profile_l_maxtimestamp AS
  SELECT i.service_id, i.profile_id, i.poem_sync_profile_id, MAX(i.timestamp) as max_timestamp
  FROM statuschange_service_profile i
  GROUP BY i.service_id, i.profile_id, i.poem_sync_profile_id;

CREATE OR REPLACE VIEW statuschange_service_profile_l AS
  SELECT s.id, s.service_id, s.profile_id, s.poem_sync_profile_id, s.metricstatus_id, s.detail, s.timestamp
  FROM statuschange_service_profile s, statuschange_service_profile_l_maxtimestamp r
  WHERE s.service_id = r.service_id AND s.profile_id = r.profile_id 
  	AND s.timestamp = r.max_timestamp and s.poem_sync_profile_id = r.poem_sync_profile_id;

CREATE TABLE mon_frequency_statistics (
  roc VARCHAR(200),
  flavour_id INT NOT NULL,
  metric_id INT NOT NULL,
  freq_mean INT NOT NULL,
  freq_std INT NOT NULL,
  UNIQUE INDEX uk_mon_frequency_statistics(roc ASC, flavour_id ASC, metric_id ASC)
) ENGINE = InnoDB;


CREATE TABLE metricstore_curr_load_h
(
	id INT
		NOT NULL 
		AUTO_INCREMENT,
	rec_date VARCHAR(20)
		NOT NULL,
    hour NUMERIC(2,0)
    	NOT NULL,
    timestamp TIMESTAMP
    	NOT NULL,
    number_of_records INT
    	NOT NULL,
	CONSTRAINT metricstore_curr_load_h_pk
    	PRIMARY KEY (id),
    CONSTRAINT metricstore_curr_load_h_chk1
    	CHECK (hour >= 0 AND hour <= 23
  			AND number_of_records >= 0),
    CONSTRAINT metricstore_curr_load_h_uk1
    	UNIQUE (rec_date, hour)
) ENGINE=InnoDB;


CREATE TABLE metricstore_curr_load_m
(
	id INT
		NOT NULL
		AUTO_INCREMENT,
    timestamp TIMESTAMP
    	NOT NULL,
    hour NUMERIC(2,0) 
    	NOT NULL,
    minute NUMERIC(2,0)  
    	NOT NULL,
    number_of_records INT 
    	NOT NULL,
    CONSTRAINT metricstore_curr_load_m_pk
    	PRIMARY KEY (id),
    CONSTRAINT metricstore_curr_load_m_uk1
    UNIQUE (hour, minute),
    CONSTRAINT metricstore_curr_load_m_chk1
    	CHECK (hour >= 0 AND hour <= 23 
    		AND minute >= 0 AND minute <= 59 
    		AND minute >= 0)
) ENGINE=InnoDB;


CREATE TABLE metricstore_curr_load_h_det
  (
    id INT
    	NOT NULL
		AUTO_INCREMENT,
    rec_date VARCHAR(20)
    	NOT NULL,
    hour NUMERIC(2,0) 
    	NOT NULL,
    timestamp TIMESTAMP
    	NOT NULL,
    ngi_name VARCHAR(20),
    profile_id INT,
    poem_sync_profile_id INT,
    flavour_id INT 
    	NOT NULL,
    number_of_records INT,
    CONSTRAINT metricstore_curr_load_h_d_chk1 
    	CHECK (hour >= 0 AND hour <= 23
  			AND number_of_records >= 0),
    CONSTRAINT metricstore_curr_load_h_d_pk 
    	PRIMARY KEY (id),
    CONSTRAINT metricstore_curr_load_h_d_uk1 
    	UNIQUE (rec_date, hour, ngi_name, profile_id, flavour_id, poem_sync_profile_id),
    INDEX metricstore_curr_load_h_d_fk1 (profile_id ASC),
    CONSTRAINT metricstore_curr_load_h_d_fk1
    	FOREIGN KEY (profile_id) 
    	REFERENCES profile (id)
    	ON DELETE NO ACTION
    	ON UPDATE NO ACTION,
    INDEX metricstore_curr_load_h_d_fk2 (flavour_id ASC),
    CONSTRAINT metricstore_curr_load_h_d_fk2
    	FOREIGN KEY (flavour_id) 
    	REFERENCES service_type_flavour (id)
    	ON DELETE NO ACTION
    	ON UPDATE NO ACTION,
    INDEX metricstore_curr_load_h_d_fk3 (poem_sync_profile_id ASC),
    CONSTRAINT metricstore_curr_load_h_d_fk3
    	FOREIGN KEY (poem_sync_profile_id) 
    	REFERENCES poem_sync_profile (id)
    	ON DELETE NO ACTION
    	ON UPDATE NO ACTION
) ENGINE=InnoDB;
