DELIMITER //

USE `mrs`;

DROP TRIGGER IF EXISTS load_metricdata_fer;

CREATE TRIGGER load_metricdata_fer

AFTER INSERT ON metricdata FOR EACH ROW

BEGIN
  DECLARE v_check_time INT;
  DECLARE done INT DEFAULT 0;
  DECLARE metric_curs CURSOR FOR SELECT check_time FROM metricdata_latest 
    WHERE fqan_id <=> NEW.fqan_id AND vo_id <=> NEW.vo_id AND metric_id <=> NEW.metric_id 
      AND service_id <=> NEW.service_id;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  OPEN metric_curs;
  FETCH metric_curs INTO v_check_time;
  CLOSE metric_curs;

  IF ((v_check_time IS NOT NULL) AND (v_check_time <= NEW.check_time)) THEN

    DELETE FROM metricdata_latest WHERE fqan_id <=> NEW.fqan_id AND vo_id <=> NEW.vo_id AND metric_id <=> NEW.metric_id AND service_id <=> NEW.service_id;
    INSERT INTO metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (NEW.metricstatus_id, NEW.metricdetail_id, NEW.metric_id, NEW.service_id, NEW.fqan_id, NEW.vo_id, NEW.summarydata, NEW.gatheredat_id, NEW.check_time, NEW.insert_time);
  ELSEIF (v_check_time IS NULL) THEN 
    INSERT INTO metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (NEW.metricstatus_id, NEW.metricdetail_id, NEW.metric_id, NEW.service_id, NEW.fqan_id, NEW.vo_id, NEW.summarydata, NEW.gatheredat_id, NEW.check_time, NEW.insert_time);
  END IF;

END //

DROP TRIGGER IF EXISTS ins_metrics_in_sup_services_fer;

CREATE TRIGGER ins_metrics_in_sup_services_fer

AFTER INSERT ON metrics_in_supported_services FOR EACH ROW

BEGIN
  insert into metrics_in_supp_serv_history (service_id, metric_id, vo_id, fqan_id, profile_id, timestamp, operation)
  values(NEW.service_id, NEW.metric_id, NEW.vo_id, NEW.fqan_id, NEW.profile_id, unix_timestamp(), 'insert');
END //


DROP TRIGGER IF EXISTS del_metrics_in_sup_services_fer;

CREATE TRIGGER del_metrics_in_sup_services_fer

AFTER DELETE ON metrics_in_supported_services FOR EACH ROW

BEGIN

  insert into metrics_in_supp_serv_history (service_id, metric_id, vo_id, fqan_id, profile_id, timestamp, operation)
  values(OLD.service_id, OLD.metric_id, OLD.vo_id, OLD.fqan_id, OLD.profile_id, unix_timestamp(), 'delete');

END //

DROP TRIGGER IF EXISTS ins_fqans_in_sup_services_fer;

CREATE TRIGGER ins_fqans_in_sup_services_fer

AFTER INSERT ON fqans_in_supported_services FOR EACH ROW

BEGIN
  insert into fqans_in_supp_serv_history (profile_id, service_id, fqan_id, timestamp, operation)
  values(NEW.profile_id, NEW.service_id, NEW.fqan_id, unix_timestamp(), 'insert');
END //

DROP TRIGGER IF EXISTS del_fqans_in_sup_services_fer;

CREATE TRIGGER del_fqans_in_sup_services_fer

AFTER DELETE ON fqans_in_supported_services FOR EACH ROW

BEGIN

  insert into fqans_in_supp_serv_history (profile_id, service_id, fqan_id, timestamp, operation)
  values(OLD.profile_id, OLD.service_id, OLD.fqan_id, unix_timestamp(), 'delete');

END //


DROP TRIGGER IF EXISTS ins_metricdetails_fer;

CREATE TRIGGER ins_metricdetails_fer 

BEFORE INSERT ON metricdetails FOR EACH ROW

BEGIN

  SET NEW.timestamp = unix_timestamp();

END //


DROP TRIGGER IF EXISTS metricdata_spool_insert;

CREATE TRIGGER metricdata_spool_insert
AFTER INSERT ON metricdata_spool
FOR EACH ROW 
  
trig: BEGIN
	DECLARE current_hour NUMERIC DEFAULT EXTRACT(hour FROM utc_timestamp);
	DECLARE current_minute NUMERIC DEFAULT EXTRACT(minute FROM utc_timestamp);
	DECLARE this_flavour_id INT;
	DECLARE this_profile_id INT;
	DECLARE no_more_profiles BOOLEAN DEFAULT FALSE;
	DECLARE has_profile BOOLEAN DEFAULT FALSE;
	DECLARE profile_ids CURSOR FOR SELECT id FROM profile;
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET no_more_profiles = TRUE;

    IF NEW.metricstatus_id = 5 OR NEW.metricstatus_id = 6 THEN
   		LEAVE trig;
    END IF;
    
    -- updating metricstore_curr_load_m
    DELETE FROM metricstore_curr_load_m
      WHERE DATEDIFF(utc_timestamp, timestamp) > 0;
	
	 INSERT INTO metricstore_curr_load_m (id, timestamp, hour, minute, number_of_records)
      VALUES (NULL, utc_timestamp, EXTRACT(hour FROM utc_timestamp), EXTRACT(minute FROM utc_timestamp), 1)
      ON DUPLICATE KEY UPDATE number_of_records = number_of_records + 1;
      
      
    -- updating metricstore_curr_load_h
    DELETE FROM metricstore_curr_load_h
   	  WHERE DATEDIFF(utc_timestamp, timestamp) > 30;
   	  
   	INSERT INTO metricstore_curr_load_h (id, timestamp, rec_date, hour, number_of_records)
   	  VALUES (NULL, utc_timestamp, utc_date, EXTRACT(hour FROM utc_timestamp), 1)
   	  ON DUPLICATE KEY UPDATE number_of_records = number_of_records + 1;
   	  
   	SELECT flavour_id INTO this_flavour_id FROM service WHERE id = NEW.service_id; 

	SET has_profile = FALSE;
	SET no_more_profiles = FALSE;
	
	OPEN profile_ids;
		
	profile_loop : WHILE(no_more_profiles = FALSE) DO
	
	  FETCH profile_ids INTO this_profile_id;
	  IF no_more_profiles = TRUE THEN
	    LEAVE profile_loop;
	  END IF;
	  
	  IF checkIfCalculationNeeded(this_profile_id, NEW.service_id, NEW.metric_id, NEW.vo_id, NEW.check_time) THEN
	  	SET has_profile = TRUE;
	  	
	  	UPDATE metricstore_curr_load_h_det
	  	  SET number_of_records = number_of_records + 1
	  	  WHERE ifnull(ngi_name, 'null') = ifnull(get_roc_name(NEW.service_id), 'null')
	  	    AND profile_id = this_profile_id
	  	    AND flavour_id = this_flavour_id
	  	    AND EXTRACT(hour from utc_timestamp) = hour
	  	    AND rec_date = utc_date;
	    -- if update unsuccesful (row does not exist), create new row
	  	IF ROW_COUNT() = 0 THEN
	  	  INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, flavour_id, timestamp, hour, rec_date, number_of_records)
	  	    VALUES (NULL, get_roc_name(NEW.service_id), this_profile_id, this_flavour_id, utc_timestamp, EXTRACT(hour from utc_timestamp), utc_date, 1);
	  	END IF;
	  END IF;
	  
	END WHILE profile_loop;

	CLOSE profile_ids;
	

	IF has_profile = FALSE THEN
	  -- insert on duplicate key does not work here because ngi_name can be null
	  UPDATE metricstore_curr_load_h_det
	  	SET number_of_records = number_of_records + 1
	  	WHERE ifnull(ngi_name, 'null') = ifnull(get_roc_name(NEW.service_id), 'null')
	  	  AND profile_id is NULL
	  	  AND flavour_id = this_flavour_id
	  	  AND EXTRACT(hour from utc_timestamp) = hour
	  	  AND rec_date = utc_date;
	  -- if update unsuccesful (row does not exist), create new row
	  IF ROW_COUNT() = 0 THEN
	    INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, flavour_id, timestamp, hour, rec_date, number_of_records)
	      VALUES (NULL, get_roc_name(NEW.service_id), NULL, this_flavour_id, utc_timestamp, EXTRACT(hour from utc_timestamp), utc_date, 1);
	  END IF;
	END IF;
END //



drop trigger if exists MIG2POEM_METRICDATA_TR;
create TRIGGER MIG2POEM_METRICDATA_TR
BEFORE INSERT ON metricdata
FOR EACH ROW
  BEGIN
      DECLARE v_poem_metric_id INT;
      select p.id into v_poem_metric_id
            from metric m, poem_sync_metric p
            where m.id=NEW.metric_id and m.name=p.name;
      set NEW.poem_sync_metric_id = v_poem_metric_id;
  END//
  
  
  
drop trigger if exists MIG2POEM_METRICDATA_LATEST_TR;
create TRIGGER MIG2POEM_METRICDATA_LATEST_TR
BEFORE INSERT ON metricdata_latest
FOR EACH ROW
  BEGIN
      DECLARE v_poem_metric_id INT;
      select p.id into v_poem_metric_id
            from metric m, poem_sync_metric p
            where m.id=NEW.metric_id and m.name=p.name;
      set NEW.poem_sync_metric_id = v_poem_metric_id;
  END//  
  
  
  
drop trigger if exists MIG2POEM_METRICDATA_SPOOL_TR;
create TRIGGER MIG2POEM_METRICDATA_SPOOL_TR
BEFORE INSERT ON metricdata_spool
FOR EACH ROW
  BEGIN
      DECLARE v_poem_metric_id INT;
      select p.id into v_poem_metric_id
            from metric m, poem_sync_metric p
            where m.id=NEW.metric_id and m.name=p.name;
      set NEW.poem_sync_metric_id = v_poem_metric_id;
  END//  



drop trigger if exists MIG2POEM_STATCHANGESERVPROF_TR;
create TRIGGER MIG2POEM_STATCHANGESERVPROF_TR
BEFORE INSERT ON statuschange_service_profile
FOR EACH ROW
  BEGIN
      DECLARE v_poem_prof_id INT;
      select p.id into v_poem_prof_id
            from profile m, poem_sync_profile p
            where m.id=NEW.profile_id and m.name=p.name;
      set NEW.poem_sync_profile_id = v_poem_prof_id;
  END//  
  
  
  
drop trigger if exists MIG2POEM_STATCHANGEMETSERV_TR;
create TRIGGER MIG2POEM_STATCHANGEMETSERV_TR
BEFORE INSERT ON statuschange_metric_service
FOR EACH ROW
  BEGIN
      DECLARE v_poem_metric_id INT;
      select p.id into v_poem_metric_id
            from metric m, poem_sync_metric p
            where m.id=NEW.metric_id and m.name=p.name;
      set NEW.poem_sync_metric_id = v_poem_metric_id;
  END//  
  
  

drop trigger if exists MIG2POEM_METST_C_L_H_D_TR;
create TRIGGER MIG2POEM_METST_C_L_H_D_TR
BEFORE INSERT ON metricstore_curr_load_h_det
FOR EACH ROW
  BEGIN
      DECLARE v_poem_prof_id INT;
      select p.id into v_poem_prof_id
            from profile m, poem_sync_profile p
            where m.id=NEW.profile_id and m.name=p.name;
      set NEW.poem_sync_profile_id = v_poem_prof_id;
  END// 
  


DELIMITER ;
