CREATE OR REPLACE TRIGGER DELETE_METRICDATA_DETAILS
AFTER DELETE ON METRICDATA 
FOR EACH ROW 
BEGIN
  delete from metricdata_latest where metricdetail_id = :old.metricdetail_id;
  delete from metricdetails where id = :old.metricdetail_id;
END;
/

CREATE OR REPLACE TRIGGER ins_metr_in_sup_services_fer

AFTER INSERT ON metrics_in_supported_services FOR EACH ROW

BEGIN

  insert into metrics_in_supp_serv_history (id, service_id, metric_id, vo_id, fqan_id, profile_id, timestamp, operation)
  values(metrics_in_supp_serv_his_seq.nextval, :NEW.service_id, :NEW.metric_id, :NEW.vo_id, :NEW.fqan_id, :NEW.profile_id, to_unixts(current_timestamp), 'insert');

END;
/

CREATE OR REPLACE TRIGGER del_metr_in_sup_services_fer

AFTER DELETE ON metrics_in_supported_services FOR EACH ROW

BEGIN

  insert into metrics_in_supp_serv_history (id, service_id, metric_id, vo_id, fqan_id, profile_id, timestamp, operation)
  values(metrics_in_supp_serv_his_seq.nextval, :OLD.service_id, :OLD.metric_id, :OLD.vo_id, :OLD.fqan_id, :OLD.profile_id, to_unixts(current_timestamp), 'delete');

END;
/

CREATE OR REPLACE TRIGGER V_MOUT_INSERT_TRG instead of insert on V_METRICOUTPUT

BEGIN

  DataLoader.loadmetricdatatospool(:NEW.serviceflavour, :NEW.metricname, :NEW.metricstatus,
       :NEW.summarydata, :NEW.detailsdata, :NEW.voname, :NEW.vofqan, :NEW.hostname, :NEW.timestamp, :NEW.gatheredat, 7);

END;
/

create or replace trigger load_metricdata_fer
  after insert on metricdata for each row

DECLARE

  CURSOR metric_curs IS SELECT check_time FROM metricdata_latest
    WHERE nvl(fqan_id, -1) = nvl(:NEW.fqan_id, -1) AND nvl(vo_id, -1) = nvl(:NEW.vo_id, -1) AND metric_id = :NEW.metric_id
      AND service_id = :NEW.service_id;

  v_check_time INTEGER;

BEGIN

  OPEN metric_curs;
  FETCH metric_curs INTO v_check_time;
  CLOSE metric_curs;

  IF ((v_check_time IS NOT NULL) AND (v_check_time <= :NEW.check_time)) THEN

    DELETE FROM metricdata_latest WHERE nvl(fqan_id, -1) = nvl(:NEW.fqan_id, -1) AND nvl(vo_id, -1) = nvl(:NEW.vo_id, -1)
      AND metric_id = :NEW.metric_id AND service_id = :NEW.service_id;
    INSERT INTO metricdata_latest(id, metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (MetricData_LatestSeq.nextval, :NEW.metricstatus_id, :NEW.metricdetail_id, :NEW.metric_id, :NEW.service_id, :NEW.fqan_id, :NEW.vo_id, :NEW.summarydata, :NEW.gatheredat_id, :NEW.check_time, :NEW.insert_time);
  ELSIF (v_check_time IS NULL) THEN
    INSERT INTO metricdata_latest(id, metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (MetricData_LatestSeq.nextval, :NEW.metricstatus_id, :NEW.metricdetail_id, :NEW.metric_id, :NEW.service_id, :NEW.fqan_id, :NEW.vo_id, :NEW.summarydata, :NEW.gatheredat_id, :NEW.check_time, :NEW.insert_time);
  END IF;


END;
/

CREATE OR REPLACE TRIGGER V_MOUT_OSG_INSERT_TRG instead of insert on V_METRICOUTPUT_OSG

BEGIN
  DataLoader.loadmetricdatatospool_OSG(:NEW.serviceuri, :NEW.servicetype, :NEW.metricname, :NEW.metricstatus,
       :NEW.summarydata, :NEW.detailsdata, :NEW.voname, :NEW.timestamp, :NEW.gatheredat, 7);
 
END;
/

CREATE OR REPLACE TRIGGER V_MDESC_INSERT_TRG 
 instead of insert on V_METRICDESCRIPTION_OSG
declare
  tid number;
  tname varchar(100);
  stype varchar(100);
begin
  stype:=lower(:new.servicetype);
  for svc in (select f.id, f.flavourname
               from metricgroup_osg m, service_type_flavour f
               where m.flavour_id = f.id and m.SERVICETYPE=stype)
  loop
    tname:=:new.metricname;
    begin
      select id into tid
      from metric
      where name=tname;
      update metric
      set description=:new.metrictitle,
        docurl=:new.metrichelp
      where id=tid;
      insert into metricdescription_osg (id, servicetype, metricname, metric_id)
      values (METRICDESCRIPTION_OSGSeq.nextval, stype, :new.metricname, tid);
    exception
      when no_data_found then
        --SAME_DB_MONITORING.send_mail_to_list('SAM_DB_Monit@no.reply', 'Wojciech.Lapka@cern.ch,David.Collados@cern.ch', 'SAM Prod: metric result rejected at DB insertion', 'ERROR: serviceType ' || lower(:new.servicetype) || '  not defined in metricgroup table?');
        select metric_SEQ.nextval into tid from dual;
        
        insert into metric (id, name, version, creator_id, description, docurl, isdeleted)
        values (tid, tname, '1.0.0', 2 /*Vaibhav - see MDDB_USER*/, :new.metrictitle, :new.metrichelp, 0);
        
        insert into metricdescription_osg (id, servicetype, metricname, metric_id)
        values (METRICDESCRIPTION_OSGSeq.nextval, stype, :new.metricname, tid);
        
      when dup_val_on_index then
        update metricdescription_osg
        set metricname=:new.metricname
        where servicetype=stype and metricname=:new.metricname and metric_id=tid;
    end;
  end loop;
end;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_M" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 
  
DECLARE
  current_hour NUMBER := EXTRACT(hour FROM current_timestamp);
  current_minute NUMBER := EXTRACT(minute FROM current_timestamp);
  
BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  UPDATE metricstore_curr_load_m SET number_of_records = number_of_records + 1
    WHERE current_hour = hour
      AND current_minute = minute
      AND EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) = 0;
      
  IF SQL%rowcount = 0 THEN -- no update was actually made
    DELETE FROM metricstore_curr_load_m
      WHERE current_hour = hour AND current_minute = minute;
    INSERT INTO metricstore_curr_load_m (id, number_of_records) VALUES (metricstore_curr_load_m_seq.nextval, 1);
  END IF;

END;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  UPDATE metricstore_curr_load_h SET number_of_records = number_of_records + 1
    WHERE EXTRACT(hour FROM current_timestamp) = hour 
      AND EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) = 0;
    
  IF SQL%rowcount = 0 THEN -- no update was actually made
    DELETE FROM metricstore_curr_load_h
      WHERE EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) > 30;
    INSERT INTO metricstore_curr_load_h (id, number_of_records) VALUES (metricstore_curr_load_h_seq.nextval, 1);
  END IF;

END;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H_DET"
AFTER INSERT ON METRICDATA_SPOOL
FOR EACH ROW

DECLARE
  current_hour NUMBER(2,0);
  this_flavour_id NUMBER;
  CURSOR profile_ids IS SELECT id FROM poem_sync_profile;
  has_profile BOOLEAN := false;
  prof_id NUMBER;

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  SELECT flavour_id INTO this_flavour_id FROM service WHERE id = :new.service_id;

  FOR this_profile IN profile_ids LOOP
    select p.id into prof_id from profile p, poem_sync_profile pp where pp.id = this_profile.id and p.name = pp.name;
    IF dataloader.checkifcalculationneeded(this_profile.id, :new.service_id, :new.metric_id, :new.vo_id, :new.check_time) THEN
      UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
        WHERE EXTRACT(hour FROM current_timestamp) = hour
          AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
          AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
          AND poem_sync_profile_id = this_profile.id
          and profile_id = prof_id
          AND flavour_id = this_flavour_id;

      IF SQL%rowcount = 0 THEN -- no update was actually made
        INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, poem_sync_profile_id,  flavour_id, number_of_records)
          VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), prof_id, this_profile.id, this_flavour_id, 1);
      END IF;

      has_profile := true;
    END IF;
  END LOOP;

  IF has_profile = false THEN
    UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
      WHERE EXTRACT(hour FROM current_timestamp) = hour
        AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
        AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
        AND profile_id is NULL
        and poem_sync_profile_id is null
        AND flavour_id = this_flavour_id;

    IF SQL%rowcount = 0 THEN -- no update was actually made
      INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, poem_sync_profile_id, flavour_id,  number_of_records)
        VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), NULL, null,  this_flavour_id, 1);
    END IF;
  END IF;

END;
/



create or replace
TRIGGER MIG2POEM_METRICDATA_TR 
BEFORE INSERT ON METRICDATA 
FOR EACH ROW
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATAFORREC_TR 
BEFORE INSERT ON METRICDATAFORRECALCULATION 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATALATEST_TR 
BEFORE INSERT ON METRICDATA_LATEST 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATARECREJ_TR 
BEFORE INSERT ON METRICDATAFORRECALC_REJECTED 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATASPOOL_TR 
BEFORE INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATASPOOLREJ_TR 
BEFORE INSERT ON METRICDATA_SPOOL_REJECTED 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_STATCHANGESERVPROF_TR 
BEFORE INSERT ON STATUSCHANGE_SERVICE_PROFILE 
FOR EACH ROW 
  WHEN (new.poem_sync_profile_id is null)
    BEGIN
      select p.id into :new.poem_sync_profile_id
          from profile m, poem_sync_profile p
          where m.id=:new.profile_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_profile_id := null;
    END;
/



CREATE OR REPLACE TRIGGER MIG2POEM_METRICDATAREC_TR 
BEFORE INSERT ON METRICDATA_REC 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_STATCHANGEMETSERV_TR 
BEFORE INSERT ON STATUSCHANGE_METRIC_SERVICE 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_MS_CURR_LOAD_H_D_TR 
BEFORE INSERT ON METRICSTORE_CURR_LOAD_H_DET 
FOR EACH ROW 
  WHEN (new.poem_sync_profile_id is null)
    BEGIN
      select p.id into :new.poem_sync_profile_id
          from profile m, poem_sync_profile p
          where m.id=:new.profile_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_profile_id := null;
    END;
/