DELIMITER //

USE `mrs`;

DROP PROCEDURE IF EXISTS loadmetricdatatospool;

CREATE PROCEDURE loadmetricdatatospool(in_serviceflavour VARCHAR(100), in_metricname VARCHAR(100), in_metricstatus VARCHAR(10), 
                 in_summarydata VARCHAR(255), in_detailsdata TEXT, in_voname VARCHAR(100), in_fqan VARCHAR(255), 
                 in_hostname VARCHAR(255), in_timestamp DATETIME, in_gatheredat VARCHAR(255))

BEGIN
  DECLARE v_gatheredat_id INT;
  DECLARE v_serviceflavour_name VARCHAR(100);
  DECLARE v_serviceflavour_id INT;
  DECLARE v_metricstatus_id INT;
  DECLARE v_poem_metric_id INT;
  DECLARE v_fqan_id INT;
  DECLARE v_vo_id INT;
  DECLARE v_service_id INT;
  DECLARE v_metricdetail_id INT;
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(100);
  DECLARE v_check_time, v_insert_time INT;
  DECLARE v_metric_age_days INT;

  DECLARE done INT DEFAULT 0;
  DECLARE flavourCurs CURSOR FOR select name from tmpServiceFlavourList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  set autocommit = 0;

  DROP TEMPORARY TABLE IF EXISTS tmpServiceFlavourList;
  CREATE TEMPORARY TABLE tmpServiceFlavourList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_serviceflavour,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_serviceflavour, 1, intFoundPos-1);
    SET in_serviceflavour = REPLACE(in_serviceflavour, CONCAT(strElement,','), '');
    INSERT INTO tmpServiceFlavourList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_serviceflavour,',');
  END WHILE;

  IF in_serviceflavour <> '' THEN
    INSERT INTO tmpServiceFlavourList (`name`) VALUES (in_serviceflavour);
  END IF; 

  OPEN flavourCurs;

  REPEAT 
  
    set done = 0;
    FETCH flavourCurs INTO v_serviceflavour_name;

    IF NOT done THEN
      set v_service_id = NULL; 
      set v_poem_metric_id = NULL;
      /*Temporary hack - See: https://savannah.cern.ch/bugs/?62833*/ 
      set v_serviceflavour_name = IF(v_serviceflavour_name='sBDII', 'Site-BDII', v_serviceflavour_name); 
      select id into v_serviceflavour_id from service_type_flavour where lower(flavourname) = lower(v_serviceflavour_name);
      set v_poem_metric_id = getPoemSyncMetricIdForName(in_metricname);
      select id into v_metricstatus_id from metricstatus where lower(description) = lower(in_metricstatus);
      select id into v_vo_id from vo where lower(voname) = lower(in_voname);
      /*select id into v_service_id from service where lower(hostname) = lower(in_hostname) and flavour_id = v_serviceflavour_id
                                       and id in (select distinct service_id from metrics_in_supported_services);*/
      select id into v_service_id from service where lower(hostname) = lower(in_hostname) and flavour_id = v_serviceflavour_id
                                       and id in (select distinct service_id from mrs_bootstrapper);
      select getOrAddFqan(in_fqan) into v_fqan_id;
      select getOrAddGatheredAt(in_gatheredat) into v_gatheredat_id;
      select UNIX_TIMESTAMP(CONVERT_TZ(in_timestamp, 'UTC', @@session.time_zone)) into v_check_time;
      select UNIX_TIMESTAMP() into v_insert_time;
      select floor((v_insert_time - v_check_time) / 86400) into v_metric_age_days;

      IF v_service_id is not NULL and v_poem_metric_id is not NULL and v_metric_age_days < 7 THEN
        insert into metricdata_spool(gatheredat_id, metricstatus_id, detailsdata, service_id,
                                       vo_id, fqan_id, summarydata, check_time, insert_time, poem_sync_metric_id)
          VALUES (v_gatheredat_id, v_metricstatus_id, in_detailsdata,  v_service_id, v_vo_id, v_fqan_id, in_summarydata,
                  v_check_time, v_insert_time, v_poem_metric_id);
      ELSEIF (v_metric_age_days >= 7) THEN
      insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'metric too old');
      ELSEIF (v_service_id is NULL and v_poem_metric_id is NULL ) THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id and poem_sync_metric_id NULL');
      ELSEIF v_service_id is NULL THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id NULL');
      ELSEIF v_poem_metric_id is NULL THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'poem_metric_id NULL');
      END IF;
      set done = 0; 
    END IF;

    commit; 

  UNTIL done END REPEAT;

  CLOSE flavourCurs;

END //

DROP PROCEDURE IF EXISTS insertRowToMetricData;

CREATE PROCEDURE insertRowToMetricData(in_service_id INT, in_poem_metric_id INT, in_metricstatus_id INT, in_summarydata VARCHAR(255), 
                   in_detailsdata TEXT, in_fqan_id INT, in_vo_id INT, in_gatheredat_id INT, in_check_time INT, in_insert_time INT,
                   in_recalculationNeeded INT, in_maxAgeOfTupleInHours INT)
BEGIN

  DECLARE v_metricdetail_id INT; 
  DECLARE v_status_id INT;
  DECLARE v_num_row INT;
  DECLARE v_poem_fqan_id INT;

  IF in_service_id is not NULL and in_poem_metric_id is not NULL THEN

    set v_status_id = 0;
    set v_num_row = 0;
    set v_poem_fqan_id = 0;

    if in_detailsdata is not null then
      insert into metricdetails(detail, timestamp)
      values (in_detailsdata,  unix_timestamp());
 
      select last_insert_id() into v_metricdetail_id;
    end if;

    if in_fqan_id is not null then
      select p.id into v_poem_fqan_id from poem_sync_fqan p, fqan f
      where lower(p.fqan) = lower(f.name)
      and f.id = in_fqan_id;
    end if;
    
    select id into v_status_id from metricstatus where description = 'REMOVED';
    
    select count(*) into v_num_row
      from mrs_bootstrapper
      where metric_id = in_poem_metric_id
      and service_id = in_service_id
      and (ifnull(vo_id, -1) = ifnull(in_vo_id, -1) or vo_id is null or in_vo_id is null)
      and (ifnull(fqan_id, -1) = ifnull(v_poem_fqan_id, -1) or fqan_id is null or v_poem_fqan_id is null);

    /*insert into metricdata(metricstatus_id, metricdetail_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (in_metricstatus_id, v_metricdetail_id, in_poem_metric_id, in_service_id, in_fqan_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time);

    call calculateStatusChange(in_service_id, in_poem_metric_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours);
*/

    if v_num_row > 0 or in_metricstatus_id = v_status_id then
        insert into metricdata(metricstatus_id, metricdetail_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
            VALUES (in_metricstatus_id, v_metricdetail_id, in_poem_metric_id, in_service_id, in_fqan_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time);

        call calculateStatusChange(in_service_id, in_poem_metric_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours);
    else
        insert into metricdata_spool_rejected(metricstatus_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
            VALUES (in_metricstatus_id, in_poem_metric_id, in_service_id, in_fqan_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time);
    end if;

  END IF;

END //

DROP PROCEDURE IF EXISTS calculateStatusChange;

CREATE PROCEDURE calculateStatusChange(serviceId INT,  poem_metricId INT, metricstatusId INT, check_time INT, voId INT,
                                       recalculationNeeded INT, maxAgeOfTupleInHours INT)
BEGIN

  DECLARE profile_name VARCHAR(256);
  DECLARE poem_profileId INT;
  DECLARE v_supportedVo INT;

  DECLARE done INT DEFAULT 0;
 
  DECLARE poemProfileCurs CURSOR FOR
    select a.profile_id as id, c.name
        from poem_sync_p_m_instances a, poem_sync_metricinstance b, poem_sync_profile c where
          a.profile_id=c.id and c.isdeleted='N' and
          a.metricinstance_id=b.id and
          (b.atp_vo_id = voId or b.atp_vo_id is null) and
           b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId) and
          b.metric_id = poem_metricId /*and
          (b.fqan_id is null or b.fqan_id in (select id from poem_sync_fqan where fqan='/Role/lcgadm'))*/;

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  call calculateStatusChangeMetric(poem_metricId, serviceId, voId, metricstatusId, check_time);

  select id into v_supportedVo from vo where id = voId
  and lower(voname) in (select name from supported_vos);

  IF (v_supportedVo is not null) or (voId is NULL) THEN
    OPEN poemProfileCurs;

    REPEAT

      set done = 0;
      FETCH poemProfileCurs INTO poem_profileId, profile_name;
        
	  IF NOT done THEN
        IF checkIfCalculationNeeded(poem_profileId, serviceId, poem_metricId, voId, check_time) THEN
          IF (ifnull(getServiceStatus(poem_profileId, serviceId, check_time), -1) != metricstatusId) THEN

            call calculateStatusChangeProfile(poem_profileId, serviceId, poem_metricId, metricstatusId, check_time, voId,
                                       recalculationNeeded, maxAgeOfTupleInHours);
          END IF;
        END IF;
        set done = 0;
      END IF;

    UNTIL done END REPEAT;

	CLOSE poemProfileCurs;
  END IF;   

END //


DROP PROCEDURE IF EXISTS calculateStatusChangeMetric;

CREATE PROCEDURE calculateStatusChangeMetric(poem_metricId INT, serviceId INT, voId INT, metricstatusId INT, check_time INT)

BEGIN
  DECLARE num_row INTEGER;
  DECLARE old_metricstatus_id INTEGER;
  DECLARE old_timestamp INTEGER;

  SET num_row = 0;
 
  call findCurrentStatusMetricService(poem_metricId, serviceId, voId, metricstatusId, check_time, old_metricstatus_id, old_timestamp, num_row); 

  IF num_row > 0 THEN
    IF ((old_timestamp != check_time) AND (metricstatusId != old_metricstatus_id)) THEN
      INSERT INTO statuschange_metric_service(poem_sync_metric_id, service_id, vo_id, metricstatus_id, timestamp, insert_time)
      VALUES(poem_metricId, serviceId, voId, metricstatusId, check_time, UNIX_TIMESTAMP()); 
    ELSEIF (getMetricStatusWeightForId(metricstatusId) > getMetricStatusWeightForId(old_metricstatus_id)) THEN
       UPDATE statuschange_metric_service SET metricstatus_id = metricstatusId, insert_time = UNIX_TIMESTAMP()
         WHERE service_id = serviceId and poem_sync_metric_id = poem_metricId and timestamp = check_time
           AND ifnull(vo_id, -1) = ifnull(voId, -1);
    END IF;
  ELSE
    INSERT INTO statuschange_metric_service(poem_sync_metric_id, service_id, vo_id, metricstatus_id, timestamp, insert_time)
    VALUES( poem_metricId, serviceId, voId, metricstatusId, check_time, UNIX_TIMESTAMP()); 
  END IF;

END //

DROP PROCEDURE IF EXISTS calculateStatusChangeProfile;

CREATE PROCEDURE calculateStatusChangeProfile(poem_profileId INT, serviceId INT, poem_metricId INT, metricstatusId INT, 
                     check_time INT, voId INT, recalculationNeeded INT, maxAgeOfTupleInHours INT)

BEGIN
  DECLARE old_metricstatus_id INTEGER; 
  DECLARE old_timestamp INTEGER; 
  DECLARE num_row INTEGER;
  DECLARE curr_servicestatus_id INTEGER;

  SET num_row = 0;

  call findCurrentStatus(poem_profileId, serviceId, metricstatusId, check_time, voId, curr_servicestatus_id, old_metricstatus_id, old_timestamp, num_row); 

  IF num_row > 0 THEN
    IF (getMetricStatusWeightForId(curr_servicestatus_id) > getMetricStatusWeightForId(old_metricstatus_id)) THEN
      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_service_profile(service_id,  poem_sync_profile_id, metricstatus_id, detail, timestamp, insert_time)
        VALUES(serviceId, poem_profileId, curr_servicestatus_id, NULL, check_time, UNIX_TIMESTAMP());
      ELSE
         UPDATE statuschange_service_profile SET metricstatus_id = curr_servicestatus_id, insert_time = UNIX_TIMESTAMP() 
         WHERE service_id = serviceId and poem_sync_profile_id = poem_profileId and timestamp = check_time;
      END IF;

    ELSEIF ((getMetricStatusWeightForId(metricstatusId) < getMetricStatusWeightForId(old_metricstatus_id))
        AND (getMetricStatusWeightForId(curr_servicestatus_id) < getMetricStatusWeightForId(old_metricstatus_id))) THEN

      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_service_profile(service_id,  poem_sync_profile_id, metricstatus_id, detail, timestamp, insert_time)
        VALUES(serviceId, poem_profileId, curr_servicestatus_id, NULL, check_time, UNIX_TIMESTAMP());
      ELSE
         UPDATE statuschange_service_profile SET metricstatus_id = curr_servicestatus_id, insert_time = UNIX_TIMESTAMP() 
         WHERE service_id = serviceId and poem_sync_profile_id = poem_profileId and timestamp = check_time;
      END IF;

    END IF;
  ELSE 
    INSERT INTO statuschange_service_profile(service_id, poem_sync_profile_id, metricstatus_id, detail, timestamp, insert_time)
    VALUES(serviceId, poem_profileId, metricstatusId, NULL, check_time, UNIX_TIMESTAMP()); 
  END IF;

END //

DROP FUNCTION IF EXISTS checkIfCalculationNeeded;

/*TODO: Needs to discuss if should look at void or only at table metrics_in_supported_services (Now this option will not work for profiles
like ROC_CRITICAL, ROC_OPERATORS)*/

CREATE FUNCTION checkIfCalculationNeeded( poem_profileId INT, serviceId INT, poem_metricId INT, voId INT, check_time INT)
RETURNS INT
BEGIN

  DECLARE v_counter INT;

  /*DECLARE metricCurs CURSOR FOR SELECT count(1) FROM profile_metric_map 
    where profile_id = profileId and (test_vo_id = voId or voId is null) 
       and metric_id = metricId and service_type_flavour_id =
        (select flavour_id from service where id = serviceId)
       and (del_time > from_unixtime(check_time) or del_time is null) 
       and profile_id in (select id from profile where lower(name) in (select lower(name) from supported_profiles));*/
       
  DECLARE metricCurs CURSOR FOR select count(*) 
  	from poem_sync_p_m_instances a, poem_sync_metricinstance b where 
  		b.metric_id = poem_metricId and
		a.profile_id = poem_profileId and 
        a.metricinstance_id=b.id and 
	    (b.atp_vo_id = voId or b.atp_vo_id is null or voId is null) and
		b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId); /*add supported_profiles?? check_time??*/

  
  OPEN metricCurs;
  FETCH metricCurs INTO v_counter;
  CLOSE metricCurs;

  IF (v_counter > 0) THEN
    RETURN 1;
  ELSE 
    RETURN 0;
  END IF;

END //

/* TODO: This function will replace checkIfCalculationNeeded
   In the future will add checking if service defined in grouping, e.g. MPI, glexec */

DROP FUNCTION IF EXISTS checkIfMetricDefInProfile;

CREATE FUNCTION checkIfMetricDefInProfile(poem_profileId INT, serviceId INT, poem_metricId INT)
RETURNS INT
BEGIN

  DECLARE v_counter INT;

  /*DECLARE metricCurs CURSOR FOR SELECT count(1) FROM profile_metric_map 
    where profile_id = profileId and metric_id = metricId 
      and service_type_flavour_id =
        (select flavour_id from service where id = serviceId)
      and (del_time > from_unixtime(check_time) or del_time is null) 
      and profile_id in (select id from profile where lower(name) in (select lower(name) from supported_profiles));*/
      
   DECLARE metricCurs CURSOR FOR select count(*) 
  	from poem_sync_p_m_instances a, poem_sync_metricinstance b where 
  		b.metric_id = poem_metricId and
		a.profile_id = poem_profileId and 
        a.metricinstance_id=b.id and
		b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId); /*add supported_profiles?? check_time??*/

  
  OPEN metricCurs;
  FETCH metricCurs INTO v_counter;
  CLOSE metricCurs;

  IF (v_counter > 0) THEN
    RETURN 1;
  ELSE 
    RETURN 0;
  END IF;

END //


DROP PROCEDURE IF EXISTS insertMissingMetrics;

CREATE PROCEDURE insertMissingMetrics(maxAgeInHours INT, ageInMinutes INT)
BEGIN

  DECLARE v_id INT;
  DECLARE v_poem_metric_id INT;
  DECLARE v_service_id INT;
  DECLARE v_flavour_id INT;
  DECLARE v_vo_id INT;
  DECLARE v_fqan_id INT;
  DECLARE v_poem_fqan_id INT;
  DECLARE v_fqan_name VARCHAR(255);

  DECLARE done INT DEFAULT 0;

 DECLARE oldMetricsCurs CURSOR FOR
    select m.id, m.poem_sync_metric_id, m.service_id, m.vo_id, m.fqan_id from metricdata_latest m,
      mrs_bootstrapper s
    where
         m.poem_sync_metric_id = s.metric_id
     and m.service_id = s.service_id
     and (ifnull(m.vo_id, -1) = ifnull(s.vo_id, -1) or m.vo_id is null or s.vo_id is null)
     and (ifnull((select pf.id from fqan f, poem_sync_fqan pf where lower(f.name) = lower(pf.fqan) and f.id = m.fqan_id), -1) = ifnull(s.fqan_id, -1)
     or m.fqan_id is null or s.fqan_id is null)
     and m.check_time <=  UNIX_TIMESTAMP(now() - interval 24 hour - interval 10 minute)
     and m.service_id in (select id from service where isdeleted = 'N')
     and m.metricstatus_id not in (select id from metricstatus where description = 'REMOVED');

  DECLARE newMissingMetricsCurs CURSOR FOR
   select poem_sync_metric_id, service_id, vo_id, fqan_id from
      (select distinct p.metric_id as poem_sync_metric_id, p.service_id, p.vo_id, p.fqan_id, p.metric_id as m_metric_id from
         mrs_bootstrapper p left join metricdata_latest m on m.poem_sync_metric_id = p.metric_id
           and m.service_id = p.service_id
           and  (ifnull(m.vo_id, -1) = ifnull(p.vo_id, -1) or m.vo_id is null or p.vo_id is null)
           and (ifnull(getMRSfqan_id(p.fqan_id), -1) = ifnull(m.fqan_id, -1) or p.fqan_id is null or m.fqan_id is null)
      ) a, service s where a.service_id = s.id and a.m_metric_id is null and s.isdeleted = 'N';

  /*Checking metricdata_spool not needed as spool was cleared by loadmetricdata*/

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  set autocommit = 0;

  OPEN oldMetricsCurs;

  REPEAT
    set done = 0;
    FETCH oldMetricsCurs INTO v_id,  v_poem_metric_id, v_service_id, v_vo_id, v_fqan_id;

    IF NOT done THEN

      insert into metricdata_spool(gatheredat_id, metricstatus_id, detailsdata, poem_sync_metric_id,  service_id, vo_id, fqan_id, summarydata, check_time, insert_time)
        VALUES (NULL, (select id from metricstatus where description = 'MISSING'), NULL,  v_poem_metric_id, v_service_id, v_vo_id, v_fqan_id, NULL, unix_timestamp(now() - interval ageInMinutes minute), unix_timestamp());
     
      set done = 0; 
    END IF;
  UNTIL done END REPEAT;

  CLOSE oldMetricsCurs;

  commit;

  set done = 0;
  OPEN newMissingMetricsCurs;

  REPEAT
    set done = 0;
    FETCH newMissingMetricsCurs INTO v_poem_metric_id, v_service_id, v_vo_id, v_poem_fqan_id;
   
    set v_fqan_id = null;
    if v_poem_fqan_id is not null then
        select fqan into v_fqan_name from poem_sync_fqan where id = v_poem_fqan_id;
        select getOrAddFqan(v_fqan_name) into v_fqan_id;
    end if;
    
    IF NOT done THEN
      insert into metricdata_spool(gatheredat_id, metricstatus_id, detailsdata, poem_sync_metric_id, service_id, vo_id, fqan_id, summarydata, check_time, insert_time)
        VALUES (NULL, (select id from metricstatus where description = 'MISSING'), NULL,  v_poem_metric_id, v_service_id, v_vo_id, v_fqan_id, NULL, unix_timestamp(now() - interval ageInMinutes minute), unix_timestamp());

      set done = 0; 
    END IF;

  UNTIL done END REPEAT;
  
  CLOSE newMissingMetricsCurs;
  commit;

END //

DROP PROCEDURE IF EXISTS purgeMetricStore;

CREATE PROCEDURE purgeMetricStore(maxAgeInDays INT, metricdatalatest_days_kept INT)
BEGIN
  
  DECLARE minInsertTime INT;
  DECLARE last_metricdatalatest_kept INT;

  set minInsertTime = UNIX_TIMESTAMP(now() - interval maxAgeInDays day);
  set last_metricdatalatest_kept = UNIX_TIMESTAMP(now() - interval metricdatalatest_days_kept day); 

  update statuschange_service_profile
    set timestamp = minInsertTime, insert_time = UNIX_TIMESTAMP() 
    where id in (
      select maxId from (
        select max(id) as maxId, service_id,  poem_sync_profile_id from statuschange_service_profile
          where timestamp <= minInsertTime
        group by service_id,  poem_sync_profile_id
      ) a
    );

  -- updating metricstore_curr_load_h_det
  DELETE FROM metricstore_curr_load_h_det
   	WHERE DATEDIFF(utc_timestamp, timestamp) > 30;
   	  
  DELETE FROM statuschange_service_profile
    WHERE timestamp < minInsertTime;


  DELETE FROM metricdata WHERE check_time < minInsertTime;
  
  DELETE FROM metricdata_latest 
  	WHERE check_time < last_metricdatalatest_kept;

  /*Deleting data from metricdata_latest isn't neeeded since insertMissingMetrics will create "MISSING" metrics. 
    If we switch off insertMissingMetrics, we would have to delete entries from tested_services;
  */ 
  DELETE FROM metricdata_latest WHERE check_time < minInsertTime;

  DELETE FROM metricdata_rejected WHERE check_time < minInsertTime;

  DELETE FROM metricdetails where timestamp < minInsertTime;

  /*DELETE FROM statuschange_service_profile WHERE service_id not in
      (select distinct service_id from metricdata_latest)
    and service_id not in (select distinct service_id from metrics_in_supported_services);
    
    DELETE FROM statuschange_metric_service WHERE service_id not in
      (select distinct service_id from metricdata_latest)
    and service_id not in (select distinct service_id from metrics_in_supported_services);

  DELETE FROM tested_services WHERE service_id not in
      (select distinct service_id from metricdata_latest)
    and service_id not in (select distinct service_id from metrics_in_supported_services);*/
    
  DELETE FROM statuschange_service_profile WHERE service_id not in
      (select distinct service_id from metricdata_latest)
    and service_id not in (select distinct service_id from mrs_bootstrapper);

  DELETE FROM statuschange_metric_service WHERE service_id not in
      (select distinct service_id from metricdata_latest)
    and service_id not in (select distinct service_id from mrs_bootstrapper);

  COMMIT;

END //

DROP FUNCTION IF EXISTS getOrAddGatheredAt;

CREATE FUNCTION getOrAddGatheredAt(in_gatheredat VARCHAR(255)) RETURNS INT
BEGIN
  DECLARE v_id INT;

  select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);

  IF v_id is null THEN
    insert into gatheredat(name)
      values(in_gatheredat);
    select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);
  END IF;
   
  RETURN v_id;

END //

DROP FUNCTION IF EXISTS getOrAddFqan;

CREATE FUNCTION getOrAddFqan(in_name VARCHAR(255)) RETURNS INT
BEGIN
  DECLARE v_id INT;
  set v_id = NULL;

  IF in_name is not NULL THEN
    select id into v_id from fqan where lower(name) = lower(in_name);
    
    IF v_id is null THEN

      insert into fqan(name)
        values(in_name);
      select id into v_id from fqan where lower(name) = lower(in_name);
    END IF;
  END IF; 

  RETURN v_id;
END //


DROP PROCEDURE IF EXISTS addSupportedVos;

CREATE PROCEDURE addSupportedVos(in_VoList VARCHAR(255))
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_VoName VARCHAR(255);

  DECLARE done INT DEFAULT 0;
  DECLARE voCurs CURSOR FOR select name from tmpVoList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpVoList;
  CREATE TEMPORARY TABLE tmpVoList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_VoList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_VoList, 1, intFoundPos-1);
    SET in_VoList = REPLACE(in_VoList, CONCAT(strElement,','), '');
    INSERT INTO tmpVoList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_VoList,',');
  END WHILE;

  IF in_VoList <> '' THEN
    INSERT INTO tmpVoList (`name`) VALUES (in_VoList);
  END IF; 

  delete from supported_vos;

  OPEN voCurs;

  REPEAT 
    set done = 0;
    FETCH voCurs INTO v_VoName;

    IF NOT done THEN
      insert into supported_vos(name, insert_time) values(lower(v_VoName), unix_timestamp())
        on duplicate key update name = v_VoName;

      set done = 0;
    END IF;

  UNTIL done END REPEAT;

  CLOSE voCurs;

  COMMIT;

END //

DROP PROCEDURE IF EXISTS addSupportedProfiles;

CREATE PROCEDURE addSupportedProfiles(in_ProfileList VARCHAR(255))
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_ProfileName VARCHAR(255);

  DECLARE done INT DEFAULT 0;
  DECLARE profileCurs CURSOR FOR select name from tmpProfileList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpProfileList;
  CREATE TEMPORARY TABLE tmpProfileList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_ProfileList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_ProfileList, 1, intFoundPos-1);
    SET in_ProfileList = REPLACE(in_ProfileList, CONCAT(strElement,','), '');
    INSERT INTO tmpProfileList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_ProfileList,',');
  END WHILE;

  IF in_ProfileList <> '' THEN
    INSERT INTO tmpProfileList (`name`) VALUES (in_ProfileList);
  END IF; 

  truncate table supported_profiles;

  OPEN profileCurs;

  REPEAT 
  
    set done = 0;
    FETCH profileCurs INTO v_ProfileName;
    SET v_ProfileName = lower(v_ProfileName);

    IF NOT done THEN
      insert into supported_profiles (name, insert_time)
      values(v_ProfileName, unix_timestamp())
       on duplicate key update name = v_ProfileName;

      set done = 0;
    END IF;

  UNTIL done END REPEAT;

  CLOSE profileCurs;

  COMMIT;

END //


DROP FUNCTION IF EXISTS getMetricStatusWeightForId; 

CREATE FUNCTION getMetricStatusWeightForId (in_id INT) RETURNS INT 

BEGIN
  DECLARE v_weight INT;
   
  select weight into v_weight from metricstatus where id = in_id;    
  RETURN v_weight;

END // 

DROP FUNCTION IF EXISTS getServiceStatus;

CREATE FUNCTION getServiceStatus(poem_profileId INT,
  serviceId INT,
  checkTime INT) RETURNS INT

BEGIN

  DECLARE statusId INT;

  select metricstatus_id into statusId from statuschange_service_profile 
    where timestamp = (select max(timestamp) from statuschange_service_profile
                       where service_id = serviceId and poem_sync_profile_id = poem_profileId and timestamp <= checkTime)
    and service_id = serviceId and poem_sync_profile_id = poem_profileId;

  RETURN statusId;

END //

DROP PROCEDURE IF EXISTS findCurrentStatus;

CREATE PROCEDURE findCurrentStatus (poem_profileId INT, serviceId INT, metricstatusId INT, check_time INT,
  voId INT,
  OUT curr_servicestatus_id INT,
  OUT old_metricstatus_id INT,
  OUT old_timestamp INT,
  OUT num_row INT
  )

BEGIN
  DECLARE curr_weight INT;  
 
  select count(*) INTO num_row
  from statuschange_service_profile 
  where service_id = serviceId and poem_sync_profile_id = poem_profileId AND timestamp <= check_time;

  IF num_row > 0 THEN
    SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_service_profile 
    WHERE service_id = serviceId and poem_sync_profile_id = poem_profileId AND timestamp =
      (SELECT max(timestamp) FROM statuschange_service_profile
      WHERE service_id = serviceId and poem_sync_profile_id = poem_profileId and timestamp <= check_time);

    IF (getMetricStatusWeightForId(metricstatusId) != getMetricStatusWeightForId(old_metricstatus_id)) THEN

      /*select max(s.weight) INTO curr_weight from metricdata_latest l, metricstatus s where l.id in (
        select lastId from (
          select max(id) as lastId, metric_id from metricdata_latest where metric_id in (
              select m.metric_id from profile_metric_map m
                 where m.profile_id = profileId
                  and (m.test_vo_id = voId or voId is null)
                  and m.service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
                  and (m.del_time > from_unixtime(check_time) or m.del_time is null)
              )
            and service_id = serviceId 
            and (ifnull(vo_id, -1) = ifnull(voId, -1) or vo_id is null)
            and (vo_id is null or vo_id in ( 
              SELECT m.test_vo_id FROM profile_metric_map m
                where m.profile_id = profileId 
                  and (test_vo_id = voId or voId is null)
                  and m.service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
                  and (m.del_time > from_unixtime(check_time) or m.del_time is null)
              ))                        
            group by metric_id) a
        ) and l.metricstatus_id = s.id;*/
        
      
      select max(s.weight) INTO curr_weight from metricdata_latest l, metricstatus s where l.id in ( 
      	select lastId from (
      		select max(id) as lastId, poem_sync_metric_id from metricdata_latest where poem_sync_metric_id in (
	            select b.metric_id from poem_sync_p_m_instances a, poem_sync_metricinstance b 
	            	where a.profile_id=poem_profileId 
	            	and a.metricinstance_id=b.id 
	            	and (b.atp_vo_id = voId or voId is null) 
	            	and b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
	            )
	        and service_id = serviceId
            and (ifnull(vo_id, -1) = ifnull(voId, -1) or vo_id is null)
            and (vo_id is null or vo_id in (
              select b.atp_vo_id from poem_sync_p_m_instances a, poem_sync_metricinstance b 
            	where a.profile_id=poem_profileId 
            	and a.metricinstance_id=b.id 
            	and (b.atp_vo_id = voId or voId is null) 
            	and b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
              ))
              group by poem_sync_metric_id) a
            ) and l.metricstatus_id = s.id; 
            
       
      select id into curr_servicestatus_id from metricstatus where weight = curr_weight;   

    END IF;
  
  END IF;

END //

DROP PROCEDURE IF EXISTS findCurrentStatusMetricService;

CREATE PROCEDURE findCurrentStatusMetricService (poem_metricId INT, serviceId INT, voId INT, metricstatusId INT, check_time INT,
  OUT old_metricstatus_id INT,
  OUT old_timestamp INT,
  OUT num_row INT)
BEGIN
  
  select count(*) INTO num_row
  from statuschange_metric_service 
  where ifnull(vo_id, -1) = ifnull(voId, -1)
    and poem_sync_metric_id = poem_metricId
    AND service_id = serviceId
    AND timestamp <= check_time;

  IF num_row > 0 THEN
  
  SELECT max(timestamp)  INTO old_timestamp FROM statuschange_metric_service
          WHERE ifnull(vo_id, -1) = ifnull(voId, -1)
            and poem_sync_metric_id = poem_metricId
            AND service_id = serviceId 
            AND timestamp <= check_time;
            
     SELECT max(metricstatus_id) INTO old_metricstatus_id
    FROM statuschange_metric_service 
    WHERE ifnull(vo_id, -1) = ifnull(voId, -1)
      and poem_sync_metric_id = poem_metricId
      AND service_id = serviceId 
      AND timestamp = old_timestamp;

   /* SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_metric_service 
    WHERE ifnull(vo_id, -1) = ifnull(voId, -1)
      and poem_sync_metric_id = poem_metricId
      AND service_id = serviceId 
      AND timestamp =
        (SELECT max(timestamp) FROM statuschange_metric_service
          WHERE ifnull(vo_id, -1) = ifnull(voId, -1)
            and poem_sync_metric_id = poem_metricId
            AND service_id = serviceId 
            AND timestamp <= check_time);*/

  END IF;

END //

DROP PROCEDURE IF EXISTS loadmetricdata;

CREATE PROCEDURE loadmetricdata(maxAgeInHours INT, ageInMinutes INT)
                  
  BEGIN

  DECLARE v_minCheckTime INT;  
  DECLARE done INT DEFAULT 0;
 
  DECLARE v_id, v_fqan_id, v_gatheredat_id, v_metricstatus_id, v_check_time, v_insert_time,  v_poem_metric_id, v_service_id, v_vo_id INT;
  DECLARE v_summarydata VARCHAR(256);
  DECLARE v_detailsdata TEXT;


  DECLARE metricResultsCurs CURSOR FOR select id, gatheredat_id, metricstatus_id, detailsdata, poem_sync_metric_id, service_id,
    fqan_id, vo_id, summarydata, check_time, insert_time from metricdata_spool
      where check_time < unix_timestamp(now() - interval ageInMinutes minute)
      order by check_time; 

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  set autocommit = 0;

  OPEN metricResultsCurs;
 
  REPEAT
    set done = 0;
    FETCH metricResultsCurs INTO v_id, v_gatheredat_id, v_metricstatus_id, v_detailsdata,  v_poem_metric_id, v_service_id,
        v_fqan_id, v_vo_id, v_summarydata, v_check_time, v_insert_time;
     

    IF NOT done THEN
      call insertRowToMetricData(v_service_id,  v_poem_metric_id, v_metricstatus_id, v_summarydata, v_detailsdata,
             v_fqan_id, v_vo_id, v_gatheredat_id, v_check_time, v_insert_time, false, maxAgeInHours);

      delete from metricdata_spool where id = v_id;
      commit;

      set done = 0;
    END IF;

  UNTIL done END REPEAT;

  CLOSE metricResultsCurs;

  call insertMissingMetrics(maxAgeInHours, ageInMinutes);
 
  commit;
END //

DROP FUNCTION IF EXISTS getPoemSyncMetricIdForName;
 
CREATE FUNCTION getPoemSyncMetricIdForName (in_metricname VARCHAR(256)) RETURNS INT
    
  BEGIN
    DECLARE v_metric_id INT;
    
    select id into v_metric_id from poem_sync_metric where lower(name) = lower(in_metricname);

    RETURN v_metric_id;
  
END //

DROP FUNCTION IF EXISTS getPoemSyncProfileIdForName;
 
CREATE FUNCTION getPoemSyncProfileIdForName (in_profilename VARCHAR(256)) RETURNS INT
    
  BEGIN
    DECLARE v_profile_id INT;
    
    select id into v_profile_id from poem_sync_profile where lower(name) = lower(in_profilename);

    RETURN v_profile_id;
  
END //

DROP FUNCTION IF EXISTS getMRSfqan_id;

CREATE FUNCTION getMRSfqan_id (in_poem_sync_fqan_id INTEGER) RETURNS INT
  
BEGIN
    DECLARE  v_fqan_id INT;
    
    IF in_poem_sync_fqan_id is NULL THEN
      BEGIN
        return null;
      END;
    END IF;
    
    set v_fqan_id = null;
    select m.id into v_fqan_id from fqan m, poem_sync_fqan p
    where lower(m.name) = lower(p.fqan)
      and p.id = in_poem_sync_fqan_id;

    RETURN v_fqan_id;

END // 


DELIMITER ;
