USE mrs; 

CREATE TABLE IF NOT EXISTS event_logger (
  id INT NOT NULL AUTO_INCREMENT,
  severity VARCHAR(45) NOT NULL,
  name VARCHAR(45) NOT NULL,
  message TEXT NOT NULL,
  inserttime  timestamp DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (id)
) ENGINE = InnoDB;

create table if not exists dataloader_lastchecktime
(
  check_time int not null,
  constraint uk_dataloader_lastchecktime unique (check_time)
) ENGINE = InnoDB;

create table if not exists gatheredat
(
  id int auto_increment not null,
  name varchar(256),
  primary key(id)  
) ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS fqan (
  id INT NOT NULL AUTO_INCREMENT ,
  name VARCHAR(256) NOT NULL ,
  PRIMARY KEY (id) )
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricstatus` (
  `id` INT NOT NULL,
  `weight` INT NOT NULL,
  `description` VARCHAR(45) NULL,
  `description_nagios` VARCHAR(45) NULL,
  `comments` VARCHAR(45) NULL,
  PRIMARY KEY (`id`),
  CONSTRAINT uk_metricstatus_weight UNIQUE(weight))
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdetails` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `detail` TEXT NOT NULL,
  timestamp INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX metricdetails_timestamp (timestamp ASC))
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan_id` INT,
  `gatheredat_id` INT,
  `metricstatus_id` INT NOT NULL,
  `metricdetail_id` INT NULL,
  `service_id` INT not null,
  `vo_id` INT,
  `summarydata` VARCHAR(256) NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  poem_sync_metric_id INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_metricdata_fqan_id` (`fqan_id` ASC),
  INDEX `fk_metricdata_gatheredat_id` (`gatheredat_id` ASC),
  INDEX `fk_metricdata_sam_nodes` (`service_id` ASC),
  INDEX `fk_metricdata_sam_vos` (`vo_id` ASC),
  INDEX `fk_metricresults_metricstatus` (`metricstatus_id` ASC),
  INDEX `fk_metricdata_metricdetails` (`metricdetail_id` ASC),
  INDEX `fk_metricdata_check_time` (`check_time` ASC),
  INDEX metricdata_p_s_metric_id_idx (poem_sync_metric_id ASC),
  UNIQUE INDEX uk_metricdata (metricstatus_id ASC, service_id ASC, vo_id ASC, check_time ASC, poem_sync_metric_id ASC) ,
  CONSTRAINT `fk_metricdata_fqan_id`
    FOREIGN KEY (`fqan_id`)
    REFERENCES `fqan` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_gatheredat_id`
    FOREIGN KEY (`gatheredat_id`)
    REFERENCES `gatheredat` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id))
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS metricdata_spool (
  id INT NOT NULL AUTO_INCREMENT,
  fqan_id INT,
  gatheredat_id INT,
  metricstatus_id INT NOT NULL,
  detailsdata text,
  service_id INT NOT NULL,
  vo_id INT,
  summarydata VARCHAR(256) NULL,
  check_time INT NOT NULL,
  insert_time INT NOT NULL,
  poem_sync_metric_id INT NOT NULL,
  PRIMARY KEY (id),
  INDEX fk_metricdata_spool_fqan_id (fqan_id ASC),
  INDEX fk_metricdata_sp_gatheredat_id (gatheredat_id ASC),
  INDEX fk_metricdata_sp_poem_metrics (poem_sync_metric_id ASC),
  INDEX fk_metricdata_sp_nodes (service_id ASC),
  INDEX fk_metricdata_sp_vos (vo_id ASC),
  INDEX fk_metricidata_sp_metricstatus (metricstatus_id ASC),
  INDEX fk_metricdata_sp_check_time (check_time ASC),
  CONSTRAINT fk_metricdata_spool_fqan_id
    FOREIGN KEY (fqan_id)
    REFERENCES fqan (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_gatheredat_id 
    FOREIGN KEY (gatheredat_id)
    REFERENCES gatheredat (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_nodes
    FOREIGN KEY (service_id)
    REFERENCES service (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_vos
    FOREIGN KEY (vo_id)
    REFERENCES vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_metricstatus
    FOREIGN KEY (metricstatus_id)
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_s_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id)
) ENGINE = InnoDB ;


CREATE TABLE IF NOT EXISTS metricdata_spool_rejected (
  id INT NOT NULL AUTO_INCREMENT,
  fqan_id INT,
  gatheredat_id INT,
  metricstatus_id INT NOT NULL,
  detailsdata text,
  service_id INT NOT NULL,
  vo_id INT,
  summarydata VARCHAR(256) NULL,
  check_time INT NOT NULL,
  insert_time INT NOT NULL,
  poem_sync_metric_id INT NOT NULL,
  PRIMARY KEY (id),
  CONSTRAINT fk_metricdata_spool_r_fqan_id
    FOREIGN KEY (fqan_id)
    REFERENCES fqan (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_gatheredat_id 
    FOREIGN KEY (gatheredat_id)
    REFERENCES gatheredat (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_nodes
    FOREIGN KEY (service_id)
    REFERENCES service (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_vos
    FOREIGN KEY (vo_id)
    REFERENCES vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_metricstatus
    FOREIGN KEY (metricstatus_id)
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sr_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id)
) ENGINE = InnoDB ;


CREATE TABLE IF NOT EXISTS `statuschange_service_profile` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan_id` INT NULL,
  `service_id` INT NULL,
  `metricstatus_id` INT NOT NULL DEFAULT 0,
  `detail` VARCHAR(256) NULL,
  `timestamp` INT NOT NULL,
  `insert_time` INT NOT NULL,
  is_considered_ace enum('N', 'Y', 'C') default 'N' NOT NULL,
  poem_sync_profile_id INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_stch_serv_profile_fqan_id` (`fqan_id` ASC),
  INDEX `fk_statuschange_service_p_metricstatus` (`metricstatus_id` ASC),
  INDEX `fk_statuschange_service_p_services` (`service_id` ASC),
  INDEX statuschange_service_prof_b (is_considered_ace ASC),  
  INDEX statuschg_s_p_poem_s_prof_idx (poem_sync_profile_id ASC),
  CONSTRAINT `fk_stch_serv_profile_fqan_id`
    FOREIGN KEY (`fqan_id`)
    REFERENCES `fqan` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `uk_statuschange_service_profile`
    UNIQUE KEY (`service_id`, `poem_sync_profile_id`, `timestamp`),
  CONSTRAINT `fk_statuschange_service_p_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_services`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_stchg_service_p_poem_prof
    FOREIGN KEY (poem_sync_profile_id)
    REFERENCES poem_sync_profile (id))

ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata_latest` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan_id` INT,
  `gatheredat_id` INT,
  `metricstatus_id` INT NOT NULL,
  `metricdetail_id` INT NULL,
  `service_id` INT NULL,
  `vo_id` INT NULL,
  `summarydata` VARCHAR(256) NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  poem_sync_metric_id  INT not null,
  PRIMARY KEY (`id`),
  INDEX `fk_metricdata_latest_fqan_id` (`fqan_id` ASC),
  INDEX `fk_metricdata_latest_gatheredat_id` (`gatheredat_id` ASC),
  INDEX `fk_metricdata_latest_nodes` (`service_id` ASC),
  INDEX `fk_metricdata_latest_vos` (`vo_id` ASC),
  INDEX `fk_metricresults_metricstatus_latest` (`metricstatus_id` ASC),
  INDEX `fk_metricdata_metricdetails_latest` (`metricdetail_id` ASC),
  INDEX `fk_metricdata_check_time_latest` (`check_time` ASC),
  INDEX metricdata_l_p_s_metric_id_idx (poem_sync_metric_id ASC),
  UNIQUE INDEX uk_metricdata_latest (metricstatus_id ASC, service_id ASC, vo_id ASC, check_time ASC, poem_sync_metric_id ASC) ,
  CONSTRAINT `fk_metricdata_latest_fqan_id`
    FOREIGN KEY (`fqan_id`)
    REFERENCES `fqan` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_gatheredat_id` 
    FOREIGN KEY  (`gatheredat_id`)
    REFERENCES `gatheredat` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus_latest`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails_latest`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_l_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id))
ENGINE = InnoDB;



CREATE  TABLE IF NOT EXISTS `metricdata_rejected` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `fqan` VARCHAR(256),
  `gatheredat` VARCHAR(255),
  `metricstatus` VARCHAR(25),
  `metric` VARCHAR(256),
  `service` VARCHAR(256),
  `service_flavour` VARCHAR(256),
  `vo` VARCHAR(256),
  `summarydata` TEXT NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  `reason` VARCHAR(256),
  PRIMARY KEY (`id`),
  INDEX `mdata_rej_checktime_idx` (`check_time` ASC))
ENGINE = InnoDB;

INSERT INTO metricstatus(id, weight, description, description_nagios, comments) VALUES 
(1, 1, 'OK', 'OK', NULL),
(2, 2, 'WARNING', 'WARNING', NULL),
(3, 3, 'UNKNOWN', 'UNKNOWN', NULL), 
(4, 4, 'MISSING', '', 'Automatically calculate by Metric Store'),
(5, 5, 'CRITICAL', 'CRITICAL', NULL),
(6, 6, 'REMOVED', '', 'Service removed');

CREATE TABLE  IF NOT EXISTS schema_details (
     id INT AUTO_INCREMENT NOT NULL,
     ver VARCHAR(20) NOT NULL,
     creation_date DATETIME NOT NULL,
     remarks VARCHAR(1000) BINARY NULL,
     PRIMARY KEY (id)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_vos (
  id INT AUTO_INCREMENT NOT NULL,
  name VARCHAR(256) NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  CONSTRAINT `uk_supported_vos`
    UNIQUE KEY (`name`)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS statuschange_metric_service (
  id INT NOT NULL AUTO_INCREMENT ,
  service_id INT NOT NULL ,
  vo_id INT NULL ,
  metricstatus_id INT NOT NULL ,
  timestamp INT NOT NULL ,
  insert_time INT NOT NULL,
  poem_sync_metric_id INT NOT NULL,
  PRIMARY KEY (id) ,
  UNIQUE INDEX uk_statuschange_metric_service (service_id ASC, vo_id ASC, timestamp ASC, poem_sync_metric_id ASC) ,
  INDEX fk_statuschange_m_s_poem_metrics (poem_sync_metric_id ASC),
  INDEX fk_statuschange_metric_s_metricstatus (metricstatus_id ASC),
  INDEX fk_statuschange_metric_s_services (service_id ASC),
  INDEX fk_statuschange_metric_s_vos (vo_id ASC),
  CONSTRAINT fk_statuschange_metric_s_metricstatus
    FOREIGN KEY (metricstatus_id )
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_statuschange_metric_s_services
    FOREIGN KEY (service_id )
    REFERENCES service (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_stchg_metric_s_poem_serv
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric (id),
  CONSTRAINT fk_statuschange_metric_s_vos
    FOREIGN KEY (vo_id)
    REFERENCES vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

DELIMITER //

DROP FUNCTION IF EXISTS getVoName;
CREATE FUNCTION getVoName (in_atp_vo_id INT) RETURNS VARCHAR(255)
  BEGIN
    DECLARE v_vo_name VARCHAR(255) default NULL;
    select voname into v_vo_name
        from vo where vo.id = in_atp_vo_id;
    RETURN v_vo_name;
END //

DELIMITER ;
  		
 create or replace view metric_history_in_poem_s_prof as
    SELECT
        pp.name profile_name,  
        pn.name namespace,
        getVoName(pp.atp_vo_id) defined_by_vo, 
        s.hostname, 
        st.serviceabbr service_type,
        t.flavourname service_flavour, 
        md.check_time metric_exec_time,
        m.name metric_name, 
        f.name fqan, 
        v.voname tested_by_vo,
        ms.description metric_status, 
        md.summarydata metric_output_summary,
        g.name gathered_at, 
        md.insert_time metric_result_insert_time
    from         
        service s, service_type st, service_type_flavour t,       
        poem_sync_metric m, metricstatus ms, gatheredat g,       
        poem_sync_profile pp, poem_sync_namespace pn,  
        poem_sync_p_m_instances ppm, poem_sync_metricinstance pm,
        vo v right outer join metricdata md on v.id = md.vo_id
        left outer join fqan f on md.fqan_id = f.id
    where  
        g.id = md.gatheredat_id
        and st.id = t.service_type_id
        and ms.id = md.metricstatus_id
        and s.id = md.service_id
        and m.id = md.poem_sync_metric_id
        and t.id = s.flavour_id
        and ppm.profile_id = pp.id
        and pm.atp_service_type_flavour_id = s.flavour_id
        and pm.metric_id = md.poem_sync_metric_id
        and (pm.atp_vo_id = md.vo_id or md.vo_id is null)
        and pm.id = ppm.metricinstance_id
        and pp.namespace_id = pn.id;
  		

CREATE OR REPLACE VIEW statuschange_service_profile_l_maxtimestamp AS
  SELECT i.service_id,  i.poem_sync_profile_id, MAX(i.timestamp) as max_timestamp
  FROM statuschange_service_profile i
  GROUP BY i.service_id, i.poem_sync_profile_id;

CREATE OR REPLACE VIEW statuschange_service_profile_l AS
  SELECT s.id, s.service_id,  s.poem_sync_profile_id, s.metricstatus_id, s.detail, s.timestamp
  FROM statuschange_service_profile s, statuschange_service_profile_l_maxtimestamp r
  WHERE s.service_id = r.service_id 
  	AND s.timestamp = r.max_timestamp 
  	and s.poem_sync_profile_id = r.poem_sync_profile_id;

CREATE TABLE if not exists metricstore_curr_load_h
(
	id INT
		NOT NULL 
		AUTO_INCREMENT,
	rec_date VARCHAR(20)
		NOT NULL,
    hour NUMERIC(2,0)
    	NOT NULL,
    timestamp TIMESTAMP
    	NOT NULL,
    number_of_records INT
    	NOT NULL,
	CONSTRAINT metricstore_curr_load_h_pk
    	PRIMARY KEY (id),
    CONSTRAINT metricstore_curr_load_h_chk1
    	CHECK (hour >= 0 AND hour <= 23
  			AND number_of_records >= 0),
    CONSTRAINT metricstore_curr_load_h_uk1
    	UNIQUE (rec_date, hour)
) ENGINE=InnoDB;


CREATE TABLE if not exists metricstore_curr_load_m
(
	id INT
		NOT NULL
		AUTO_INCREMENT,
    timestamp TIMESTAMP
    	NOT NULL,
    hour NUMERIC(2,0) 
    	NOT NULL,
    minute NUMERIC(2,0)  
    	NOT NULL,
    number_of_records INT 
    	NOT NULL,
    CONSTRAINT metricstore_curr_load_m_pk
    	PRIMARY KEY (id),
    CONSTRAINT metricstore_curr_load_m_uk1
    UNIQUE (hour, minute),
    CONSTRAINT metricstore_curr_load_m_chk1
    	CHECK (hour >= 0 AND hour <= 23 
    		AND minute >= 0 AND minute <= 59 
    		AND minute >= 0)
) ENGINE=InnoDB;


CREATE TABLE if not exists metricstore_curr_load_h_det
  (
    id INT
    	NOT NULL
		AUTO_INCREMENT,
    rec_date VARCHAR(20)
    	NOT NULL,
    hour NUMERIC(2,0) 
    	NOT NULL,
    timestamp TIMESTAMP
    	NOT NULL,
    ngi_name VARCHAR(20),
    poem_sync_profile_id INT,
    flavour_id INT 
    	NOT NULL,
    number_of_records INT,
    CONSTRAINT metricstore_curr_load_h_d_chk1 
    	CHECK (hour >= 0 AND hour <= 23
  			AND number_of_records >= 0),
    CONSTRAINT metricstore_curr_load_h_d_pk 
    	PRIMARY KEY (id),
    CONSTRAINT metricstore_curr_load_h_d_uk1 
    	UNIQUE (rec_date, hour, ngi_name, flavour_id, poem_sync_profile_id),
    INDEX metricstore_curr_load_h_d_fk2 (flavour_id ASC),
    CONSTRAINT metricstore_curr_load_h_d_fk2
    	FOREIGN KEY (flavour_id) 
    	REFERENCES service_type_flavour (id)
    	ON DELETE NO ACTION
    	ON UPDATE NO ACTION,
    INDEX metricstore_curr_load_h_d_fk3 (poem_sync_profile_id ASC),
    CONSTRAINT metricstore_curr_load_h_d_fk3
    	FOREIGN KEY (poem_sync_profile_id) 
    	REFERENCES poem_sync_profile (id)
    	ON DELETE NO ACTION
    	ON UPDATE NO ACTION
) ENGINE=InnoDB;


CREATE TABLE if not exists  mrs_bootstrapper
  (
    service_id INT NOT NULL,
    metric_id INT NOT NULL,
    fqan_id INT,
    vo_id INT,
    timestamp timestamp DEFAULT CURRENT_TIMESTAMP,
    CONSTRAINT bootstrapper_fk1 FOREIGN KEY (service_id) REFERENCES service (id),
    CONSTRAINT bootstrapper_fk2 FOREIGN KEY (metric_id) REFERENCES poem_sync_metric (id),
    CONSTRAINT bootstrapper_fk3 FOREIGN KEY (fqan_id) REFERENCES poem_sync_fqan (id),
    CONSTRAINT bootstrapper_fk4 FOREIGN KEY (vo_id) REFERENCES vo (id)
  )
  ENGINE = InnoDB;  
  
CREATE TABLE if not exists mrs_bootstrapper_history
  (
    service_id INT NOT NULL,
    metric_id INT NOT NULL,
    fqan_id INT,
    vo_id INT,
    timestamp timestamp DEFAULT CURRENT_TIMESTAMP,
    operation varchar(256),
    CONSTRAINT bootstrapper_h_fk1 FOREIGN KEY (service_id) REFERENCES service (id),
    CONSTRAINT bootstrapper_h_fk2 FOREIGN KEY (metric_id) REFERENCES poem_sync_metric (id),
    CONSTRAINT bootstrapper_h_fk3 FOREIGN KEY (fqan_id) REFERENCES poem_sync_fqan (id),
    CONSTRAINT bootstrapper_h_fk4 FOREIGN KEY (vo_id) REFERENCES vo (id)
  )
  ENGINE = InnoDB;
  
  CREATE INDEX metricdata_VoMetServ_idx ON metricdata(vo_id, poem_sync_metric_id, service_id);
  
  CREATE TABLE Logger (
  id INT NOT NULL AUTO_INCREMENT,
  inserttime  timestamp DEFAULT CURRENT_TIMESTAMP,
  proc_name VARCHAR(256),
  severity VARCHAR(50),
  message VARCHAR(1024),
  CONSTRAINT logger_pk PRIMARY KEY (id)
);

delimiter //

drop procedure if exists logger;
create PROCEDURE logger(in_procname VARCHAR(256), in_severity VARCHAR(50), in_message VARCHAR(1024)) 

BEGIN

  INSERT INTO Logger(proc_name, severity, message)
    values(in_procname, in_severity, in_message);

END //

delimiter ;
  
