CREATE OR REPLACE TRIGGER DELETE_METRICDATA_DETAILS
AFTER DELETE ON METRICDATA 
FOR EACH ROW 
BEGIN
  delete from metricdata_latest where metricdetail_id = :old.metricdetail_id;
  delete from metricdetails where id = :old.metricdetail_id;
END;
/

CREATE OR REPLACE TRIGGER V_MOUT_INSERT_TRG instead of insert on V_METRICOUTPUT

BEGIN

  DataLoader.loadmetricdatatospool(:NEW.serviceflavour, :NEW.metricname, :NEW.metricstatus,
       :NEW.summarydata, :NEW.detailsdata, :NEW.voname, :NEW.vofqan, :NEW.hostname, :NEW.timestamp, :NEW.gatheredat, 7);

END;
/

create or replace
trigger load_metricdata_fer
  after insert on metricdata for each row

DECLARE

  CURSOR metric_curs IS SELECT check_time FROM metricdata_latest
    WHERE ((nvl(fqan_id, -1) = nvl(:NEW.fqan_id, -1)) or (fqan_id is NULL))
      AND poem_sync_metric_id = :NEW.poem_sync_metric_id AND service_id = :NEW.service_id;

  v_check_time INTEGER;

BEGIN

  OPEN metric_curs;
  FETCH metric_curs INTO v_check_time;
  CLOSE metric_curs;

  IF ((v_check_time IS NOT NULL) AND (v_check_time <= :NEW.check_time)) THEN

    DELETE FROM metricdata_latest WHERE ((nvl(fqan_id, -1) = nvl(:NEW.fqan_id, -1)) or (fqan_id is NULL)) 
      AND nvl(vo_id, -1) = nvl(:NEW.vo_id, -1)
      AND poem_sync_metric_id = :NEW.poem_sync_metric_id AND service_id = :NEW.service_id;
    INSERT INTO metricdata_latest(id, metricstatus_id, metricdetail_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time, poem_sync_metric_id)
    VALUES (MetricData_LatestSeq.nextval, :NEW.metricstatus_id, :NEW.metricdetail_id, :NEW.service_id, :NEW.fqan_id, :NEW.vo_id, :NEW.summarydata, :NEW.gatheredat_id, :NEW.check_time, :NEW.insert_time, :NEW.poem_sync_metric_id);
  ELSIF (v_check_time IS NULL) THEN
    INSERT INTO metricdata_latest(id, metricstatus_id, metricdetail_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time, poem_sync_metric_id)
    VALUES (MetricData_LatestSeq.nextval, :NEW.metricstatus_id, :NEW.metricdetail_id, :NEW.service_id, :NEW.fqan_id, :NEW.vo_id, :NEW.summarydata, :NEW.gatheredat_id, :NEW.check_time, :NEW.insert_time, :NEW.poem_sync_metric_id);
  END IF;


END;
/

create or replace
TRIGGER V_MOUT_OSG_INSERT_TRG instead of insert on V_METRICOUTPUT_OSG

BEGIN

  DataLoader.loadmetricdatatospool_OSG(:NEW.serviceuri, :NEW.servicetype, :NEW.metricname, :NEW.metricstatus,
       :NEW.summarydata, :NEW.detailsdata, nvl(:NEW.voname, 'ops'), :NEW.timestamp, :NEW.gatheredat, 7);

END;
/

create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_M" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 
  
DECLARE
  current_hour NUMBER := EXTRACT(hour FROM current_timestamp);
  current_minute NUMBER := EXTRACT(minute FROM current_timestamp);
  
BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  UPDATE metricstore_curr_load_m SET number_of_records = number_of_records + 1
    WHERE current_hour = hour
      AND current_minute = minute
      AND EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) = 0;
      
  IF SQL%rowcount = 0 THEN -- no update was actually made
    DELETE FROM metricstore_curr_load_m
      WHERE current_hour = hour AND current_minute = minute;
    INSERT INTO metricstore_curr_load_m (id, number_of_records) VALUES (metricstore_curr_load_m_seq.nextval, 1);
  END IF;

END;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  UPDATE metricstore_curr_load_h SET number_of_records = number_of_records + 1
    WHERE EXTRACT(hour FROM current_timestamp) = hour 
      AND EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) = 0;
    
  IF SQL%rowcount = 0 THEN -- no update was actually made
    DELETE FROM metricstore_curr_load_h
      WHERE EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) > 30;
    INSERT INTO metricstore_curr_load_h (id, number_of_records) VALUES (metricstore_curr_load_h_seq.nextval, 1);
  END IF;

END;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H_DET"
AFTER INSERT ON METRICDATA_SPOOL
FOR EACH ROW

DECLARE
  current_hour NUMBER(2,0);
  this_flavour_id NUMBER;
  CURSOR profile_ids IS SELECT id FROM poem_sync_profile;
  has_profile BOOLEAN := false;
  prof_id NUMBER;

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  SELECT flavour_id INTO this_flavour_id FROM service WHERE id = :new.service_id;

  FOR this_profile IN profile_ids LOOP
     IF dataloader.checkIfMetricDefinedInProfile(this_profile.id, :new.service_id, :new.poem_sync_metric_id, :new.vo_id, :new.check_time) THEN
      UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
        WHERE EXTRACT(hour FROM current_timestamp) = hour
          AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
          AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
          AND poem_sync_profile_id = this_profile.id
          AND flavour_id = this_flavour_id;

      IF SQL%rowcount = 0 THEN -- no update was actually made
        INSERT INTO metricstore_curr_load_h_det (id, ngi_name, poem_sync_profile_id,  flavour_id, number_of_records)
          VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), this_profile.id, this_flavour_id, 1);
      END IF;

      has_profile := true;
    END IF;
  END LOOP;

  IF has_profile = false THEN
    UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
      WHERE EXTRACT(hour FROM current_timestamp) = hour
        AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
        AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
        and poem_sync_profile_id is null
        AND flavour_id = this_flavour_id;

    IF SQL%rowcount = 0 THEN -- no update was actually made
      INSERT INTO metricstore_curr_load_h_det (id, ngi_name, poem_sync_profile_id, flavour_id,  number_of_records)
        VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id),  null,  this_flavour_id, 1);
    END IF;
  END IF;

END;
/

------------------------------------------------------------
-- Begin: Bootstrapping
------------------------------------------------------------

CREATE OR REPLACE TRIGGER mrs_bootstrap_hist_ins
AFTER INSERT ON mrs_bootstrapper FOR EACH ROW
BEGIN
  insert into mrs_bootstrapper_history (service_id, metric_id, fqan_id, vo_id, timestamp, operation)
  values(:NEW.service_id, :NEW.metric_id, :NEW.fqan_id, :NEW.vo_id, :NEW.timestamp, 'insert');
  logstatus('mrs_bootstrap_hist_ins', 'DEBUG', 'New entry in MRS bootstrapper (metric_id ' ||  :NEW.metric_id || ' service_id ' || :NEW.service_id || ' fqan_id ' || nvl(:NEW.fqan_id, -1) || ' vo_id '  || nvl(:NEW.vo_id, -1) || ')');
END;
/

create or replace
TRIGGER mrs_bootstrap_hist_del
AFTER DELETE ON mrs_bootstrapper FOR EACH ROW
BEGIN
update  mrs_bootstrapper_history  set operation = 'delete', timestamp = current_timestamp
          where  service_id = :OLD.service_id and
          metric_id = :OLD.metric_id and
          nvl(fqan_id, -1) = nvl(:OLD.fqan_id, -1) and
          nvl(vo_id, -1) = nvl(:OLD.vo_id, -1) and
          operation like 'insert';
  logstatus('mrs_bootstrap_hist_del', 'DEBUG', 'Entry deleted in MRS bootstrapper (metric_id ' ||  :OLD.metric_id || ' service_id ' || :OLD.service_id || ' fqan_id ' || nvl(:OLD.fqan_id, -1) || ' vo_id '  || nvl(:OLD.vo_id, -1) || ')');
END;
/

create or replace
TRIGGER spool_bootstrapper_ins
AFTER INSERT ON mrs_bootstrapper FOR EACH ROW
DECLARE
   v_metricstatus_id NUMBER;
BEGIN
  select id into v_metricstatus_id from metricstatus where description = 'MISSING';
  
 insert into metricdata_spool (id, service_id,  poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
 		 values(MetricData_SpoolSeq.nextval, :NEW.service_id,  :NEW.metric_id, v_metricstatus_id, :NEW.fqan_id, :NEW.vo_id, 'MISSING', to_unixts(:NEW.timestamp), to_unixts(:NEW.timestamp));
END;
/

create or replace
TRIGGER spool_bootstrapper_del
AFTER DELETE ON mrs_bootstrapper FOR EACH ROW
DECLARE
  v_metricstatus_id NUMBER;
BEGIN

 select id into v_metricstatus_id from metricstatus where description = 'REMOVED';
   insert into metricdata_spool (id, service_id,  poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
   values(MetricData_SpoolSeq.nextval, :OLD.service_id,  :OLD.metric_id, v_metricstatus_id,  :OLD.fqan_id, :OLD.vo_id, 'REMOVED', to_unixts(current_timestamp), to_unixts(current_timestamp));
END;
/

------------------------------------------------------------
-- End: Bootstrapping
------------------------------------------------------------

CREATE OR REPLACE TRIGGER STCHG_SERV_PROF_MAXTS_TR 
AFTER INSERT ON STATUSCHANGE_SERVICE_PROFILE for each row
declare 
      num_row INT;
begin
  select count(*) into num_row from statuschange_serv_prof_maxts 
  where service_id = :NEW.service_id and poem_sync_profile_id = :NEW.poem_sync_profile_id; 
  if num_row = 0 then
      insert into statuschange_serv_prof_maxts (service_id, poem_sync_profile_id, timestamp)
      values (:NEW.service_id, :NEW.poem_sync_profile_id, :NEW.timestamp);
  else
      select count(*) into num_row from statuschange_serv_prof_maxts 
      where service_id = :NEW.service_id and poem_sync_profile_id = :NEW.poem_sync_profile_id and timestamp < :NEW.timestamp;
      if num_row > 0 then
          update statuschange_serv_prof_maxts set timestamp = :NEW.timestamp
          where service_id = :NEW.service_id and poem_sync_profile_id = :NEW.poem_sync_profile_id;
      end if;
  end if;
  
END;
/

CREATE OR REPLACE TRIGGER STCHG_SERV_PROF_MAXTS_UTR 
AFTER UPDATE ON STATUSCHANGE_SERVICE_PROFILE for each row
declare 
      num_row INT;
begin
  select count(*) into num_row from statuschange_serv_prof_maxts 
  where service_id = :NEW.service_id and poem_sync_profile_id = :NEW.poem_sync_profile_id; 
  if num_row = 0 then
      insert into statuschange_serv_prof_maxts (service_id, poem_sync_profile_id, timestamp)
      values (:NEW.service_id, :NEW.poem_sync_profile_id, :NEW.timestamp);
  else
      select count(*) into num_row from statuschange_serv_prof_maxts 
      where service_id = :NEW.service_id and poem_sync_profile_id = :NEW.poem_sync_profile_id and timestamp < :NEW.timestamp;
      if num_row > 0 then
          update statuschange_serv_prof_maxts set timestamp = :NEW.timestamp
          where service_id = :NEW.service_id and poem_sync_profile_id = :NEW.poem_sync_profile_id;
      end if;
  end if;
  
END;
/

CREATE OR REPLACE TRIGGER STCHG_SERV_PROF_MAXTS_DTR 
AFTER DELETE ON STATUSCHANGE_SERVICE_PROFILE for each row
declare 
      num_row INT;
      max_timestamp INT;
begin
  select count(*) into num_row from statuschange_serv_prof_maxts 
  where service_id = :OLD.service_id and poem_sync_profile_id = :OLD.poem_sync_profile_id and timestamp = :OLD.timestamp; 
  if num_row > 0 then
    delete from statuschange_serv_prof_maxts 
        where service_id = :OLD.service_id and poem_sync_profile_id = :OLD.poem_sync_profile_id and timestamp = :OLD.timestamp;
    select count(*) into num_row from statuschange_service_profile 
        where service_id = :OLD.service_id and poem_sync_profile_id = :OLD.poem_sync_profile_id;
    if num_row > 0 then
          select max(timestamp) into max_timestamp from statuschange_service_profile 
                where service_id = :OLD.service_id and poem_sync_profile_id = :OLD.poem_sync_profile_id;
          insert into statuschange_serv_prof_maxts (service_id, poem_sync_profile_id, timestamp)
                values (:OLD.service_id, :OLD.poem_sync_profile_id, max_timestamp);
    end if;
  end if;
END;
/

