DELIMITER //

USE `mrs`;

DROP TRIGGER IF EXISTS load_metricdata_fer;

CREATE TRIGGER load_metricdata_fer

AFTER INSERT ON metricdata FOR EACH ROW

BEGIN
  DECLARE v_check_time INT;
  DECLARE done INT DEFAULT 0;
  DECLARE metric_curs CURSOR FOR SELECT check_time FROM metricdata_latest 
    WHERE ((fqan_id <=> NEW.fqan_id ) or (fqan_id is NULL)) AND vo_id <=> NEW.vo_id 
      AND poem_sync_metric_id <=> NEW.poem_sync_metric_id AND service_id <=> NEW.service_id;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  OPEN metric_curs;
  FETCH metric_curs INTO v_check_time;
  CLOSE metric_curs;

  IF ((v_check_time IS NOT NULL) AND (v_check_time <= NEW.check_time)) THEN

    DELETE FROM metricdata_latest WHERE ((fqan_id <=> NEW.fqan_id) or (fqan_id is NULL)) 
     AND vo_id <=> NEW.vo_id AND poem_sync_metric_id <=> NEW.poem_sync_metric_id AND service_id <=> NEW.service_id;
    INSERT INTO metricdata_latest(metricstatus_id, metricdetail_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (NEW.metricstatus_id, NEW.metricdetail_id, NEW.poem_sync_metric_id, NEW.service_id, NEW.fqan_id, NEW.vo_id, NEW.summarydata, NEW.gatheredat_id, NEW.check_time, NEW.insert_time);
  ELSEIF (v_check_time IS NULL) THEN 
    INSERT INTO metricdata_latest(metricstatus_id, metricdetail_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (NEW.metricstatus_id, NEW.metricdetail_id, NEW.poem_sync_metric_id, NEW.service_id, NEW.fqan_id, NEW.vo_id, NEW.summarydata, NEW.gatheredat_id, NEW.check_time, NEW.insert_time);
  END IF;

END //

DROP TRIGGER IF EXISTS ins_metricdetails_fer;

CREATE TRIGGER ins_metricdetails_fer 

BEFORE INSERT ON metricdetails FOR EACH ROW

BEGIN

  SET NEW.timestamp = unix_timestamp();

END //


DROP TRIGGER IF EXISTS metricdata_spool_insert;

CREATE TRIGGER metricdata_spool_insert
AFTER INSERT ON metricdata_spool
FOR EACH ROW 
  
trig: BEGIN
	DECLARE current_hour NUMERIC DEFAULT EXTRACT(hour FROM utc_timestamp);
	DECLARE current_minute NUMERIC DEFAULT EXTRACT(minute FROM utc_timestamp);
	DECLARE this_flavour_id INT;
	DECLARE this_poem_profile_id INT;
	DECLARE no_more_profiles BOOLEAN DEFAULT FALSE;
	DECLARE has_profile BOOLEAN DEFAULT FALSE;
	DECLARE profile_ids CURSOR FOR SELECT id FROM poem_sync_profile;
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET no_more_profiles = TRUE;

    IF NEW.metricstatus_id = 5 OR NEW.metricstatus_id = 6 THEN
   		LEAVE trig;
    END IF;
    
    -- updating metricstore_curr_load_m
    DELETE FROM metricstore_curr_load_m
      WHERE DATEDIFF(utc_timestamp, timestamp) > 0;
	
	 INSERT INTO metricstore_curr_load_m (id, timestamp, hour, minute, number_of_records)
      VALUES (NULL, utc_timestamp, EXTRACT(hour FROM utc_timestamp), EXTRACT(minute FROM utc_timestamp), 1)
      ON DUPLICATE KEY UPDATE number_of_records = number_of_records + 1;
      
      
    -- updating metricstore_curr_load_h
    DELETE FROM metricstore_curr_load_h
   	  WHERE DATEDIFF(utc_timestamp, timestamp) > 30;
   	  
   	INSERT INTO metricstore_curr_load_h (id, timestamp, rec_date, hour, number_of_records)
   	  VALUES (NULL, utc_timestamp, UPPER(DATE_FORMAT(utc_date, '%d-%b-%Y')), EXTRACT(hour FROM utc_timestamp), 1)
   	  ON DUPLICATE KEY UPDATE number_of_records = number_of_records + 1;  
   	SELECT flavour_id INTO this_flavour_id FROM service WHERE id = NEW.service_id; 

	SET has_profile = FALSE;
	SET no_more_profiles = FALSE;
	
	OPEN profile_ids;
		
	profile_loop : WHILE(no_more_profiles = FALSE) DO
	
	  FETCH profile_ids INTO this_poem_profile_id;
	  IF no_more_profiles = TRUE THEN
	    LEAVE profile_loop;
	  END IF;
	  
	  IF checkIfCalculationNeeded(this_poem_profile_id, NEW.service_id, NEW.poem_sync_metric_id, NEW.vo_id, NEW.check_time) THEN
	  	SET has_profile = TRUE;
	  	
	  	UPDATE metricstore_curr_load_h_det
	  	  SET number_of_records = number_of_records + 1
	  	  WHERE ifnull(ngi_name, 'null') = ifnull(get_roc_name(NEW.service_id), 'null')
	  	    and poem_sync_profile_id = this_poem_profile_id
	  	    AND flavour_id = this_flavour_id
	  	    AND EXTRACT(hour from utc_timestamp) = hour
	  	    AND rec_date = UPPER(DATE_FORMAT(utc_date, '%d-%b-%Y'));
	    -- if update unsuccesful (row does not exist), create new row
	  	IF ROW_COUNT() = 0 THEN
	  	  INSERT INTO metricstore_curr_load_h_det (id, ngi_name, poem_sync_profile_id, flavour_id, timestamp, hour, rec_date, number_of_records)
	  	    VALUES (NULL, get_roc_name(NEW.service_id), this_poem_profile_id, this_flavour_id, utc_timestamp, EXTRACT(hour from utc_timestamp), UPPER(DATE_FORMAT(utc_date, '%d-%b-%Y')), 1);
	  	END IF;
	  END IF;
	  
	END WHILE profile_loop;

	CLOSE profile_ids;
	

	IF has_profile = FALSE THEN
	  -- insert on duplicate key does not work here because ngi_name can be null
	  UPDATE metricstore_curr_load_h_det
	  	SET number_of_records = number_of_records + 1
	  	WHERE ifnull(ngi_name, 'null') = ifnull(get_roc_name(NEW.service_id), 'null')
	  	  and poem_sync_profile_id is null
	  	  AND flavour_id = this_flavour_id
	  	  AND EXTRACT(hour from utc_timestamp) = hour
	  	  AND rec_date = UPPER(DATE_FORMAT(utc_date, '%d-%b-%Y'));
	  -- if update unsuccesful (row does not exist), create new row
	  IF ROW_COUNT() = 0 THEN
	    INSERT INTO metricstore_curr_load_h_det (id, ngi_name,  poem_sync_profile_id, flavour_id, timestamp, hour, rec_date, number_of_records)
	      VALUES (NULL, get_roc_name(NEW.service_id), NULL, this_flavour_id, utc_timestamp, EXTRACT(hour from utc_timestamp), UPPER(DATE_FORMAT(utc_date, '%d-%b-%Y')), 1);
	  END IF;
	END IF;
END //


drop trigger if exists bootstrapper_ins;
create TRIGGER bootstrapper_ins
AFTER INSERT ON mrs_bootstrapper FOR EACH ROW

BEGIN

  insert into mrs_bootstrapper_history (service_id, metric_id, fqan_id, vo_id, timestamp, operation)
  values(NEW.service_id, NEW.metric_id, NEW.fqan_id, NEW.vo_id, NEW.timestamp, 'insert');
  call  logger('bootstrapper_ins', 'DEBUG', 'New entry in MRS bootstrapper (metric_id ' ||  NEW.metric_id || ' service_id ' || NEW.service_id || ' fqan_id ' || ifnull(NEW.fqan_id, -1) || ' vo_id '  || ifnull(NEW.vo_id, -1) || ')');
  

 insert into metricdata_spool (service_id,  poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
  values(NEW.service_id,  NEW.metric_id, (select id from metricstatus where description = 'MISSING'), NEW.fqan_id, NEW.vo_id, 'MISSING', unix_timestamp(NEW.timestamp), unix_timestamp(NEW.timestamp));
  
END //


drop trigger if exists bootstrapper_del;
create TRIGGER bootstrapper_del
AFTER DELETE ON mrs_bootstrapper FOR EACH ROW

BEGIN

  update  mrs_bootstrapper_history  set operation = 'delete', timestamp = current_timestamp
          where operation = 'insert' and
          service_id = OLD.service_id and
          metric_id = OLD.metric_id and
          ifnull(fqan_id, 'null') = ifnull(OLD.fqan_id, 'null') and
          ifnull(vo_id, 'null') = ifnull(OLD.vo_id, 'null');
  call logger('bootstrapper_del', 'DEBUG', 'Entry deleted in MRS bootstrapper (metric_id ' ||  OLD.metric_id || ' service_id ' || OLD.service_id || ' fqan_id ' || ifnull(OLD.fqan_id, -1) || ' vo_id '  || ifnull(OLD.vo_id, -1) || ')');


 insert into metricdata_spool (service_id,  poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
   values(OLD.service_id,  OLD.metric_id, (select id from metricstatus where description = 'REMOVED'),  OLD.fqan_id, OLD.vo_id, 'REMOVED', unix_timestamp(), unix_timestamp());
   
END //


DELIMITER ;


