DELIMITER //

USE `metricstore`;

DROP PROCEDURE IF EXISTS loadmetricdatatospool;

CREATE PROCEDURE loadmetricdatatospool(in_serviceflavour VARCHAR(100), in_metricname VARCHAR(100), in_metricstatus VARCHAR(10), 
                 in_summarydata VARCHAR(255), in_detailsdata TEXT, in_voname VARCHAR(100), 
                 in_hostname VARCHAR(255), in_timestamp DATETIME, in_gatheredat VARCHAR(255))

BEGIN
  DECLARE v_gatheredat_id INT;
  DECLARE v_serviceflavour_name VARCHAR(100);
  DECLARE v_serviceflavour_id INT;
  DECLARE v_metricstatus_id INT;
  DECLARE v_metric_id INT;
  DECLARE v_vo_id INT;
  DECLARE v_service_id INT;
  DECLARE v_metricdetail_id INT;
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(100);
  DECLARE v_check_time, v_insert_time INT;

  DECLARE done INT DEFAULT 0;
  DECLARE flavourCurs CURSOR FOR select name from tmpServiceFlavourList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  set autocommit = 0;

  DROP TEMPORARY TABLE IF EXISTS tmpServiceFlavourList;
  CREATE TEMPORARY TABLE tmpServiceFlavourList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_serviceflavour,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_serviceflavour, 1, intFoundPos-1);
    SET in_serviceflavour = REPLACE(in_serviceflavour, CONCAT(strElement,','), '');
    INSERT INTO tmpServiceFlavourList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_serviceflavour,',');
  END WHILE;

  IF in_serviceflavour <> '' THEN
    INSERT INTO tmpServiceFlavourList (`name`) VALUES (in_serviceflavour);
  END IF; 

  OPEN flavourCurs;

  REPEAT 
  
    set done = 0;
    FETCH flavourCurs INTO v_serviceflavour_name;

    IF NOT done THEN
      set v_service_id = NULL; 
      set v_metric_id = NULL; 

      /*Temporary hack - See: https://savannah.cern.ch/bugs/?62833*/ 
      set v_serviceflavour_name = IF(v_serviceflavour_name='sBDII', 'Site-BDII', v_serviceflavour_name); 
     
      select id into v_serviceflavour_id from atp.service_type_flavour where lower(flavourname) = lower(v_serviceflavour_name);
      select id into v_metric_id from mddb.metric where lower(name) = lower(in_metricname);
      select id into v_metricstatus_id from metricstatus where lower(description) = lower(in_metricstatus);
      select id into v_vo_id from atp.vo where lower(voname) = lower(in_voname);
      select id into v_service_id from atp.service where lower(hostname) = lower(in_hostname) and flavour_id = v_serviceflavour_id
                                                         and atp.get_roc_name(id) in (select name from supported_rocs);
      select getOrAddGatheredAt(in_gatheredat) into v_gatheredat_id;
      select UNIX_TIMESTAMP(CONVERT_TZ(in_timestamp, 'UTC', @@session.time_zone)) into v_check_time;
      select UNIX_TIMESTAMP() into v_insert_time;

      IF v_service_id is not NULL and v_metric_id is not NULL THEN
        insert into metricdata_spool(gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id,
                                       vo_id, summarydata, check_time, insert_time)
          VALUES (v_gatheredat_id, v_metricstatus_id, in_detailsdata, v_metric_id, v_service_id, v_vo_id, in_summarydata,
                  v_check_time, v_insert_time);
      ELSEIF (v_service_id is NULL and v_metric_id is NULL) THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id and metric_id NULL');
      ELSEIF v_service_id is NULL THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id NULL');
      ELSEIF v_metric_id is NULL THEN
        insert into metricdata_rejected(metricstatus, metric, service, service_flavour, vo, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'metric_id NULL');
      END IF;
      
    END IF;

    commit; 

  UNTIL done END REPEAT;

  CLOSE flavourCurs;

END //

DROP PROCEDURE IF EXISTS insertRowToMetricData;

CREATE PROCEDURE insertRowToMetricData(in_service_id INT, in_metric_id INT, in_metricstatus_id INT, in_summarydata VARCHAR(255), 
                   in_detailsdata TEXT, in_vo_id INT, in_gatheredat_id INT, in_check_time INT, in_insert_time INT,
                   in_recalculationNeeded INT, in_maxAgeOfTupleInHours INT)
BEGIN

  DECLARE v_metricdetail_id INT; 

  IF in_service_id is not NULL and in_metric_id is not NULL THEN

    if in_detailsdata is not null then
      insert into metricdetails(detail)
      values (in_detailsdata);
  
      select last_insert_id() into v_metricdetail_id;
  
    end if;


    DELETE FROM metricdata_latest WHERE vo_id <=> in_vo_id and metric_id <=> in_metric_id and service_id <=> in_service_id;

    insert into metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (in_metricstatus_id, v_metricdetail_id, in_metric_id, in_service_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time);

    call calculateStatusChange(in_service_id, in_metric_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours);

  END IF;

END //

DROP PROCEDURE IF EXISTS calculateStatusChange;

CREATE PROCEDURE calculateStatusChange(serviceId INT, metricId INT, metricstatusId INT, check_time INT, voId INT,
                                       recalculationNeeded INT, maxAgeOfTupleInHours INT)
BEGIN

  DECLARE v_profileId INT;
  DECLARE v_supportedVo INT;

  DECLARE done INT DEFAULT 0;
  DECLARE profileCurs CURSOR FOR select id from mddb.profile;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  
  select id into v_supportedVo from atp.vo where id = voId 
  and lower(voname) in (select name from supported_vos);


  IF (v_supportedVo is not null) or (voId is NULL) THEN
    OPEN profileCurs;
  
    REPEAT
  
      set done = 0;
      FETCH profileCurs INTO v_profileId;
  
      IF NOT done THEN
        IF checkIfCalculationNeeded(v_profileId, serviceId, metricId, voId) THEN
          IF (ifnull(getServiceStatus(v_profileId, serviceId, check_time), -1) != metricstatusId) THEN

            call calculateStatusChangeProfile(v_profileId, serviceId, metricId, metricstatusId, check_time, voId,
                                       recalculationNeeded, maxAgeOfTupleInHours);
            insert into tested_services (service_id, profile_id, last_insert_time)
            values(serviceId, v_profileId, unix_timestamp())
            on duplicate key update last_insert_time = unix_timestamp();
          END IF; 
        END IF;
      END IF;
        
    UNTIL done END REPEAT;
   
  
    CLOSE profileCurs;
  END IF;   

END //


DROP PROCEDURE IF EXISTS calculateStatusChangeProfile;

CREATE PROCEDURE calculateStatusChangeProfile(profileId INT, serviceId INT, metricId INT, metricstatusId INT, 
                     check_time INT, voId INT, recalculationNeeded INT, maxAgeOfTupleInHours INT)

BEGIN
  DECLARE old_metricstatus_id INTEGER; 
  DECLARE old_timestamp INTEGER; 
  DECLARE num_row INTEGER;
  DECLARE curr_servicestatus_id INTEGER;

  SET num_row = 0;

  select count(*) INTO num_row
  from statuschange_service_profile 
  where service_id = serviceId and profile_id = profileId;

--  IF (recalculationNeeded) THEN
--    call findCurrentStatusRecalculation(profileId, serviceId, metricstatusId, check_time, voId, maxAgeOfTupleInHours, curr_servicestatus_id, old_metricstatus_id, old_timestamp, num_row);
--  ELSE
    call findCurrentStatus(profileId, serviceId, metricstatusId, check_time, voId, curr_servicestatus_id, old_metricstatus_id, old_timestamp, num_row); 
--  END IF;


  IF num_row > 0 THEN

    IF (metricstatusId > old_metricstatus_id) THEN
      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_service_profile(service_id, profile_id, metricstatus_id, detail, timestamp)
        VALUES(serviceId, profileId, metricstatusId, NULL, check_time);
      ELSE
         UPDATE statuschange_service_profile SET metricstatus_id = metricstatusId WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
      END IF;

    ELSEIF ((metricstatusId < old_metricstatus_id) AND (curr_servicestatus_id < old_metricstatus_id)) THEN

      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_service_profile(service_id, profile_id, metricstatus_id, detail, timestamp)
        VALUES(serviceId, profileId, metricstatusId, NULL, check_time);
      ELSE
         UPDATE statuschange_service_profile SET metricstatus_id = curr_servicestatus_id WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
      END IF;

    END IF;
  ELSE 
    INSERT INTO statuschange_service_profile(service_id, profile_id, metricstatus_id, detail, timestamp)
    VALUES(serviceId, profileId, metricstatusId, NULL, check_time); 
  END IF;

END //

DROP FUNCTION IF EXISTS checkIfCalculationNeeded;

CREATE FUNCTION checkIfCalculationNeeded(profileId INT, serviceId INT, metricId INT, voId INT)
RETURNS INT
BEGIN

  DECLARE v_counter INT;

  DECLARE metricCurs CURSOR FOR SELECT count(1) FROM mddb.profile_metric_map 
    where profile_id = profileId and (test_vo_id = voId or voId is null) and metric_id = metricId and service_type_flavour_id =
    (select flavour_id from atp.service where id = serviceId)
    and profile_id in (select id from mddb.profile where lower(name) in (select lower(name) from supported_profiles));

  
  OPEN metricCurs;
  FETCH metricCurs INTO v_counter;
  CLOSE metricCurs;

  IF (v_counter > 0) THEN
    RETURN 1;
  ELSE 
    RETURN 0;
  END IF;

END //


DROP PROCEDURE IF EXISTS addMissingMetricsForService;

CREATE PROCEDURE addMissingMetricsForService(in_service_id INT, in_flavour_id INT)
BEGIN

  DECLARE done INT DEFAULT 0;
  DECLARE v_metric_id, v_vo_id INT;

  DECLARE metricCurs CURSOR FOR select metric_id, test_vo_id from mddb.profile_metric_map 
  where service_type_flavour_id = in_flavour_id and test_vo_id in 
    (select id from atp.vo where lower(voname) in (select name from supported_vos));


  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  OPEN metricCurs;

  REPEAT

    set done = 0;
    FETCH metricCurs INTO v_metric_id, v_vo_id;

    IF NOT done THEN
      insert into metricdata_spool(gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id, vo_id, summarydata, check_time, insert_time)
        VALUES (NULL, 5, NULL, v_metric_id, in_service_id, v_vo_id, NULL, unix_timestamp(), unix_timestamp());
          
    END IF;
  UNTIL done END REPEAT;

  CLOSE metricCurs;



END //

DROP PROCEDURE IF EXISTS find_supported_sites;

CREATE PROCEDURE find_supported_sites()
BEGIN
  DECLARE v_rocname VARCHAR(256);
  DECLARE v_roclist VARCHAR(1024);
  DECLARE done INT DEFAULT 0;

  DECLARE rocCurs CURSOR FOR select name from supported_rocs;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  OPEN rocCurs;
 
  SET v_roclist = "";

  REPEAT
    set done = 0;
    FETCH rocCurs INTO v_rocname;
  
    IF NOT done THEN 
      SET v_roclist = concat(v_roclist, v_rocname, ',');
    END IF;

  UNTIL done END REPEAT;

  CLOSE rocCurs;

  call serviceendpoints_per_regionlist(v_roclist);

END;

DROP PROCEDURE IF EXISTS serviceendpoints_per_regionlist;

CREATE PROCEDURE serviceendpoints_per_regionlist(in_regionList VARCHAR(1024))
BEGIN

  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_region_name VARCHAR(255);
  DECLARE done INT DEFAULT 0;
  DECLARE regionCurs CURSOR FOR select name from tmpregionList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;


  DROP TEMPORARY TABLE IF EXISTS tmpregionList;
  CREATE TEMPORARY TABLE tmpregionList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  DROP TEMPORARY TABLE IF EXISTS tmp_serviceendpoints_per_regionlist;
  CREATE TEMPORARY TABLE tmp_serviceendpoints_per_regionlist( roc_name VARCHAR(50) NOT NULL, service_id INT NOT NULL, site_id INT NOT NULL) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_regionlist,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_regionlist, 1, intFoundPos-1);
    SET in_regionlist = REPLACE(in_regionlist, CONCAT(strElement,','), '');
    INSERT INTO tmpregionList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_regionlist,',');
  END WHILE;

  IF in_regionlist <> '' THEN
    INSERT INTO tmpregionList (`name`) VALUES (in_regionlist);
  END IF; 

  OPEN regionCurs;

  REPEAT 
  
    set done = 0;
    FETCH regionCurs INTO v_region_name;
    SET v_region_name = lower(v_region_name);

    IF NOT done THEN
      call atp.SERVICEENDPOINTS_PER_REGION(v_region_name);

      insert into tmp_serviceendpoints_per_regionlist(roc_name, service_id, site_id)
        SELECT v_region_name, service_id, site_id FROM atp.TMP_SERVICEENDPOINTS_REGION;
      
    END IF;

    commit; 

  UNTIL done END REPEAT;

  CLOSE regionCurs;  

END //

DROP PROCEDURE IF EXISTS insertMissingMetrics;

CREATE PROCEDURE insertMissingMetrics(maxAgeInHours INT, ageInMinutes INT)
BEGIN

  DECLARE v_id INT;
  DECLARE v_metric_id INT;
  DECLARE v_service_id INT;
  DECLARE v_flavour_id INT;
  DECLARE v_vo_id INT;

  DECLARE done INT DEFAULT 0;


  DECLARE oldMetricsCurs CURSOR FOR select id, metric_id, service_id, vo_id from metricdata_latest
    where check_time <=  UNIX_TIMESTAMP(now() - interval maxAgeInHours hour - interval ageInMinutes minute)
    and service_id in (select id from atp.service where isdeleted = 'N')
    and service_id in (select service_id from tmp_serviceendpoints_per_regionlist);

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  call find_supported_sites();

  /*
  DECLARE newMissingMetricsCurs CURSOR FOR select r.service_id, s.flavour_id from tmp_serviceendpoints_per_regionlist r, atp.service s 
    where r.service_id not in (select service_id from metricdata_latest) and r.service_id = s.id;
  */

  set autocommit = 0;

  OPEN oldMetricsCurs;

  REPEAT
    set done = 0;
    FETCH oldMetricsCurs INTO v_id, v_metric_id, v_service_id, v_vo_id;

    IF NOT done THEN

      insert into metricdata_spool(gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id, vo_id, summarydata, check_time, insert_time)
        VALUES (NULL, 5, NULL, v_metric_id, v_service_id, v_vo_id, NULL, unix_timestamp(now() - interval ageInMinutes minute), unix_timestamp());
       
    END IF;
  UNTIL done END REPEAT;

  CLOSE oldMetricsCurs;

  commit;

/*  set done = 0;
  OPEN newMissingMetricsCurs;

  REPEAT
    FETCH newMissingMetricsCurs INTO v_service_id, v_flavour_id;
    
    IF NOT done THEN
      select 'service: ', iLoop, ' ', v_service_id, ' ', v_flavour_id;
      set iLoop = iLoop + 1;
      call addMissingMetricsForService(v_service_id, v_flavour_id);
    END IF;
  UNTIL done END REPEAT;
  CLOSE newMissingMetricsCurs;
*/
END //

DROP PROCEDURE IF EXISTS purgeMetricStore;

CREATE PROCEDURE purgeMetricStore(maxAgeInDays INT)
BEGIN
  
  DECLARE minInsertTime INT;

  set minInsertTime = UNIX_TIMESTAMP(now() - interval maxAgeInDays day); 

  update statuschange_service_profile
    set timestamp = minInsertTime 
    where id in (
      select maxId from (
        select max(id) as maxId, service_id, profile_id from statuschange_service_profile
          where timestamp <= minInsertTime
        group by service_id, profile_id
      ) a
    );


  DELETE FROM statuschange_service_profile
    WHERE timestamp < minInsertTime;

  DELETE FROM metricdata WHERE check_time < minInsertTime;

  /*Deleting data from metricdata_latest isn't neeeded since insertMissingMetrics will create "MISSING" metrics. 
    If we switch off insertMissingMetrics, we would have to delete entries from tested_services;
  */ 
  DELETE FROM metricdata_latest WHERE check_time < minInsertTime;

  DELETE FROM metricdata_rejected WHERE check_time < minInsertTime;

  call find_supported_sites();

  DELETE FROM statuschange_service_profile WHERE service_id not in (select distinct service_id from metricdata_latest)
    and service_id not in (select service_id from tmp_serviceendpoints_per_regionlist);


  COMMIT;

END //

DROP FUNCTION IF EXISTS getOrAddGatheredAt;

CREATE FUNCTION getOrAddGatheredAt(in_gatheredat VARCHAR(255)) RETURNS INT
BEGIN
  DECLARE v_id INT;

  select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);

  IF v_id is null THEN
    insert into gatheredat(name)
      values(in_gatheredat);
    select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);
  END IF;
   
  RETURN v_id;

END //

DROP PROCEDURE IF EXISTS addSupportedVos;

CREATE PROCEDURE addSupportedVos(in_VoList VARCHAR(255))
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_VoName VARCHAR(255);

  DECLARE done INT DEFAULT 0;
  DECLARE voCurs CURSOR FOR select name from tmpVoList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpVoList;
  CREATE TEMPORARY TABLE tmpVoList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_VoList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_VoList, 1, intFoundPos-1);
    SET in_VoList = REPLACE(in_VoList, CONCAT(strElement,','), '');
    INSERT INTO tmpVoList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_VoList,',');
  END WHILE;

  IF in_VoList <> '' THEN
    INSERT INTO tmpVoList (`name`) VALUES (in_VoList);
  END IF; 

  delete from supported_vos;

  OPEN voCurs;

  REPEAT 
    set done = 0;
    FETCH voCurs INTO v_VoName;

    IF NOT done THEN
      insert into supported_vos(name, insert_time) values(lower(v_VoName), unix_timestamp())
        on duplicate key update name = v_VoName;

    END IF;

  UNTIL done END REPEAT;

  CLOSE voCurs;

  COMMIT;

END //

DROP PROCEDURE IF EXISTS addSupportedProfiles;

CREATE PROCEDURE addSupportedProfiles(in_ProfileList VARCHAR(255))
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_ProfileName VARCHAR(255);

  DECLARE done INT DEFAULT 0;
  DECLARE profileCurs CURSOR FOR select name from tmpProfileList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpProfileList;
  CREATE TEMPORARY TABLE tmpProfileList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_ProfileList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_ProfileList, 1, intFoundPos-1);
    SET in_ProfileList = REPLACE(in_ProfileList, CONCAT(strElement,','), '');
    INSERT INTO tmpProfileList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_ProfileList,',');
  END WHILE;

  IF in_ProfileList <> '' THEN
    INSERT INTO tmpProfileList (`name`) VALUES (in_ProfileList);
  END IF; 

  truncate table supported_profiles;

  OPEN profileCurs;

  REPEAT 
  
    set done = 0;
    FETCH profileCurs INTO v_ProfileName;
    SET v_ProfileName = lower(v_ProfileName);

    IF NOT done THEN
      insert into supported_profiles (name, insert_time)
      values(v_ProfileName, unix_timestamp())
       on duplicate key update name = v_ProfileName;
    END IF;

  UNTIL done END REPEAT;

  CLOSE profileCurs;

  COMMIT;

END //


DROP PROCEDURE IF EXISTS addSupportedRocs;

CREATE PROCEDURE addSupportedRocs(in_RocList VARCHAR(255))
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_RocName VARCHAR(255);

  DECLARE done INT DEFAULT 0;
  DECLARE rocCurs CURSOR FOR select name from tmpRocList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpRocList;
  CREATE TEMPORARY TABLE tmpRocList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_RocList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_RocList, 1, intFoundPos-1);
    SET in_RocList = REPLACE(in_RocList, CONCAT(strElement,','), '');
    INSERT INTO tmpRocList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_RocList,',');
  END WHILE;

  IF in_RocList <> '' THEN
    INSERT INTO tmpRocList (`name`) VALUES (in_RocList);
  END IF; 

  truncate table supported_rocs;

  OPEN rocCurs;

  REPEAT 
  
    set done = 0;
    FETCH rocCurs INTO v_RocName;
    SET v_RocName = lower(v_RocName);

    IF NOT done THEN
      insert into supported_rocs (name, insert_time)
      values(v_RocName, unix_timestamp())
       on duplicate key update name = v_RocName;
    END IF;

  UNTIL done END REPEAT;

  CLOSE rocCurs;

  COMMIT;

END //

DROP PROCEDURE IF EXISTS cleanSupportedSites;

DROP PROCEDURE IF EXISTS addSupportedSites;

CREATE PROCEDURE addSupportedSites(in_SiteList TEXT)
BEGIN
  DECLARE intFoundPos INT;
  DECLARE strElement VARCHAR(255);
  DECLARE v_SiteName VARCHAR(255);
  DECLARE v_site_id INT;

  DECLARE done INT DEFAULT 0;
  DECLARE siteCurs CURSOR FOR select name from tmpSiteList;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  DROP TEMPORARY TABLE IF EXISTS tmpSiteList;
  CREATE TEMPORARY TABLE tmpSiteList ( `name` VARCHAR(50) NOT NULL DEFAULT '' ) ENGINE = MEMORY;

  SET intFoundPos = INSTR(in_SiteList,',');

  WHILE intFoundPos <> 0 do
    SET strElement = SUBSTRING(in_SiteList, 1, intFoundPos-1);
    SET in_SiteList = REPLACE(in_SiteList, CONCAT(strElement,','), '');
    INSERT INTO tmpSiteList (`name`) VALUES ( strElement);
    SET intFoundPos = INSTR(in_SiteList,',');
  END WHILE;

  IF in_SiteList <> '' THEN
    INSERT INTO tmpSiteList (`name`) VALUES (in_SiteList);
  END IF; 

  OPEN siteCurs;

  truncate table supported_sites;

  REPEAT 
    set done = 0;
    FETCH siteCurs INTO v_SiteName;

    IF NOT done THEN
      select id into v_site_id from atp.site where lower(sitename) = lower(v_SiteName);
      
      IF v_site_id is not null THEN
        insert into supported_sites (site_id, insert_time)
        values(v_site_id, unix_timestamp())
          on duplicate key update site_id = v_site_id;
      END IF;

      set done = 0;
    END IF;

  UNTIL done END REPEAT;

  CLOSE siteCurs;

  COMMIT;

END //


DROP FUNCTION IF EXISTS getServiceStatus;

CREATE FUNCTION getServiceStatus(
  profileId INT,
  serviceId INT,
  checkTime INT) RETURNS INT

BEGIN

  DECLARE statusId INT;

  select metricstatus_id into statusId from statuschange_service_profile 
    where id = (select max(id) from statuschange_service_profile
                       where service_id = serviceId and profile_id = profileId
                             and timestamp <= checkTime);

  RETURN statusId;

END //

DROP PROCEDURE IF EXISTS findCurrentStatus;

CREATE PROCEDURE findCurrentStatus (profileId INT, serviceId INT, metricstatusId INT, check_time INT,
  voId INT,
  OUT curr_servicestatus_id INT,
  OUT old_metricstatus_id INT,
  OUT old_timestamp INT,
  OUT num_row INT
  )

BEGIN
  
  select count(*) INTO num_row
  from statuschange_service_profile 
  where service_id = serviceId and profile_id = profileId;

  IF num_row > 0 THEN
    SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_service_profile 
    WHERE service_id = serviceId AND profile_id = profileId AND timestamp =
      (SELECT max(timestamp) FROM statuschange_service_profile
      WHERE service_id = serviceId AND profile_id = profileId and timestamp <= check_time);

    IF (metricstatusId < old_metricstatus_id) THEN
        select max(metricstatus_id) INTO curr_servicestatus_id from metricdata_latest where metric_id in (
          SELECT m.metric_id FROM mddb.profile_metric_map m
            where m.profile_id = profileId 
              and (test_vo_id = voId or voId is null)
              and m.service_type_flavour_id = (select flavour_id from atp.service l where l.id = serviceId)
          ) 
          and service_id = serviceId;
    END IF;
  
  END IF;

END //

DROP PROCEDURE IF EXISTS loadmetricdata;

CREATE PROCEDURE loadmetricdata(maxAgeInHours INT, ageInMinutes INT)
                  
  BEGIN

  DECLARE v_maxCheckTime INT;
  DECLARE v_minCheckTime INT;  
  DECLARE done INT DEFAULT 0;
 
  DECLARE v_id, v_gatheredat_id, v_metricstatus_id, v_check_time, v_insert_time,  v_metric_id, v_service_id, v_vo_id INT;
  DECLARE v_summarydata VARCHAR(256);
  DECLARE v_detailsdata TEXT;

  DECLARE metricResultsCurs CURSOR FOR select id, gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id,
    vo_id, summarydata, check_time, insert_time from metricdata_spool
      where check_time < v_maxCheckTime /*and check_time >= v_minCheckTime --WLAPKA: Will be uncommented if recalculation function will be introduced*/
      order by check_time; 

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  set autocommit = 0;

  select max(check_time) into v_minCheckTime from dataloader_lastchecktime;

  IF v_minCheckTime is null THEN
    SET v_minCheckTime = 0;
  END IF;

  SET v_maxCheckTime = unix_timestamp(now()-interval ageInMinutes minute);

  /*call findMetricsForStatusRecalc(v_maxCheckTime, v_minCheckTime);*/
  
  OPEN metricResultsCurs;
 
  REPEAT
    
    set done = 0;
    FETCH metricResultsCurs INTO v_id, v_gatheredat_id, v_metricstatus_id, v_detailsdata, v_metric_id, v_service_id,
        v_vo_id, v_summarydata, v_check_time, v_insert_time;
     

    IF NOT done THEN

      call insertRowToMetricData(v_service_id, v_metric_id, v_metricstatus_id, v_summarydata, v_detailsdata,
             v_vo_id, v_gatheredat_id, v_check_time, v_insert_time, false, maxAgeInHours);

      delete from metricdata_spool where id = v_id;
      commit;
    END IF;

  UNTIL done END REPEAT;

  CLOSE metricResultsCurs;


  /*call loadmetricdatawithrecalc (v_maxCheckTime, v_minCheckTime);*/

  delete from dataloader_lastchecktime;
  insert into dataloader_lastchecktime(check_time) values(v_maxCheckTime);
 
  call insertMissingMetrics(maxAgeInHours, ageInMinutes);
 
  commit;
END //

DELIMITER ;
