
CREATE DATABASE /*!32312 IF NOT EXISTS*/ metricstore /*!40100 DEFAULT CHARACTER SET latin1 */;

USE metricstore; 

create table if not exists dataloader_lastchecktime
(
  check_time int not null,
  constraint uk_dataloader_lastchecktime unique (check_time)
) ENGINE = InnoDB;

create table if not exists gatheredat
(
  id int auto_increment not null,
  name varchar(256),
  primary key(id)  
) ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricstatus` (
  `id` INT NOT NULL,
  `description` VARCHAR(45) NULL,
  `description_nagios` VARCHAR(45) NULL,
  `comment` VARCHAR(45) NULL,
  `comments` VARCHAR(45) NULL,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdetails` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `detail` TEXT NOT NULL,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `gatheredat_id` INT,
  `metricstatus_id` INT NOT NULL,
  `metricdetail_id` INT NULL,
  `metric_id` INT,
  `service_id` INT,
  `vo_id` INT,
  `summarydata` VARCHAR(256) NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_metricdata_gatheredat_id` (`gatheredat_id` ASC),
  INDEX `fk_metricdata_sam_metrics` (`metric_id` ASC),
  INDEX `fk_metricdata_sam_nodes` (`service_id` ASC),
  INDEX `fk_metricdata_sam_vos` (`vo_id` ASC),
  INDEX `fk_metricresults_metricstatus` (`metricstatus_id` ASC),
  INDEX `fk_metricdata_metricdetails` (`metricdetail_id` ASC),
  INDEX `fk_metricdata_check_time` (`check_time` ASC),

  CONSTRAINT `fk_metricdata_gatheredat_id`
    FOREIGN KEY (`gatheredat_id`)
    REFERENCES `gatheredat` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_metrics`
    FOREIGN KEY (`metric_id` )
    REFERENCES `mddb`.`metric` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_sam_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `atp`.`vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS metricdata_spool (
  id INT NOT NULL AUTO_INCREMENT,
  gatheredat_id INT,
  metricstatus_id INT NOT NULL,
  detailsdata text,
  metric_id INT NOT NULL,
  service_id INT NOT NULL,
  vo_id INT,
  summarydata VARCHAR(256) NULL,
  check_time INT NOT NULL,
  insert_time INT NOT NULL,
  PRIMARY KEY (id),
  INDEX fk_metricdata_sp_gatheredat_id (gatheredat_id ASC),
  INDEX fk_metricdata_sp_metrics (metric_id ASC),
  INDEX fk_metricdata_sp_nodes (service_id ASC),
  INDEX fk_metricdata_sp_vos (vo_id ASC),
  INDEX fk_metricidata_sp_metricstatus (metricstatus_id ASC),
  INDEX fk_metricdata_sp_check_time (check_time ASC),
  
  CONSTRAINT fk_metricdata_sp_gatheredat_id 
    FOREIGN KEY (gatheredat_id)
    REFERENCES gatheredat (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_metrics
    FOREIGN KEY (metric_id )
    REFERENCES mddb.metric (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_nodes
    FOREIGN KEY (service_id)
    REFERENCES atp.service (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_vos
    FOREIGN KEY (vo_id)
    REFERENCES atp.vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sp_metricstatus
    FOREIGN KEY (metricstatus_id)
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
) ENGINE = InnoDB ;

CREATE  TABLE IF NOT EXISTS `statuschange_service_profile` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `service_id` INT NULL,
  `profile_id` INT NULL,
  `metricstatus_id` INT NOT NULL DEFAULT 0,
  `detail` VARCHAR(256) NULL,
  `timestamp` INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_statuschange_service_p_metricstatus` (`metricstatus_id` ASC),
  INDEX `fk_statuschange_service_p_services` (`service_id` ASC),
  INDEX `fk_statuschange_service_p_profiles` (`profile_id` ASC),

  CONSTRAINT `uk_statuschange_service_profile`
    UNIQUE KEY (`service_id`, `profile_id`, `timestamp`),
  CONSTRAINT `fk_statuschange_service_p_metricstatus`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_services`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_statuschange_service_p_profiles`
    FOREIGN KEY (`profile_id` )
    REFERENCES `mddb`.`profile` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)

ENGINE = InnoDB;

CREATE  TABLE IF NOT EXISTS `metricdata_latest` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `gatheredat_id` INT,
  `metricstatus_id` INT NOT NULL,
  `metricdetail_id` INT NULL,
  `metric_id` INT NULL,
  `service_id` INT NULL,
  `vo_id` INT NULL,
  `summarydata` VARCHAR(256) NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_metricdata_latest_gatheredat_id` (`gatheredat_id` ASC),
  INDEX `fk_metricdata_latest_metrics` (`metric_id` ASC),
  INDEX `fk_metricdata_latest_nodes` (`service_id` ASC),
  INDEX `fk_metricdata_latest_vos` (`vo_id` ASC),
  INDEX `fk_metricresults_metricstatus_latest` (`metricstatus_id` ASC),
  INDEX `fk_metricdata_metricdetails_latest` (`metricdetail_id` ASC),
  INDEX `fk_metricdata_check_time_latest` (`check_time` ASC),

  CONSTRAINT `fk_metricdata_latest_gatheredat_id` 
    FOREIGN KEY  (`gatheredat_id`)
    REFERENCES `gatheredat` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_metrics`
    FOREIGN KEY (`metric_id` )
    REFERENCES `mddb`.`metric` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_nodes`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_latest_vos`
    FOREIGN KEY (`vo_id` )
    REFERENCES `atp`.`vo` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricresults_metricstatus_latest`
    FOREIGN KEY (`metricstatus_id` )
    REFERENCES `metricstatus` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_metricdata_metricdetails_latest`
    FOREIGN KEY (`metricdetail_id` )
    REFERENCES `metricdetails` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;



CREATE  TABLE IF NOT EXISTS `metricdata_rejected` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `gatheredat` VARCHAR(255),
  `metricstatus` VARCHAR(25),
  `metric` VARCHAR(256),
  `service` VARCHAR(256),
  `service_flavour` VARCHAR(256),
  `vo` VARCHAR(256),
  `summarydata` TEXT NULL,
  `check_time` INT NOT NULL,
  `insert_time` INT NOT NULL,
  `reason` VARCHAR(256),
  PRIMARY KEY (`id`))
ENGINE = InnoDB;

INSERT INTO metricstatus(id, description,
description_nagios, comment) VALUES (1, 'OK','OK',NULL),(2,
'WARNING','WARNING',NULL),(3, 'CRITICAL','CRITICAL',NULL),(4,
'UNKNOWN','UNKNOWN',NULL), (5,'MISSING', '', 'Automatically calculate by Metric Store');

update metricstatus set comments = comment;

CREATE  TABLE IF NOT EXISTS `tested_services` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `service_id` INT NOT NULL,
  `profile_id` INT NOT NULL,
  `last_insert_time` INT NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_tested_services_service` (`service_id` ASC),
  INDEX `fk_tested_services_profile` (`profile_id` ASC),
  INDEX `tested_services_time` (`last_insert_time` ASC),

  CONSTRAINT `uk_tested_services`
    UNIQUE KEY (`service_id`, `profile_id`),
  CONSTRAINT `fk_tested_services_service`
    FOREIGN KEY (`service_id` )
    REFERENCES `atp`.`service` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_tested_services_profile`
    FOREIGN KEY (`profile_id` )
    REFERENCES `mddb`.`profile` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)

ENGINE = InnoDB;

CREATE TABLE  IF NOT EXISTS schema_details (
     id INT AUTO_INCREMENT NOT NULL,
     ver VARCHAR(20) NOT NULL,
     creation_date DATETIME NOT NULL,
     remarks VARCHAR(1000) BINARY NULL,
     PRIMARY KEY (id)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_sites (
  id INT AUTO_INCREMENT NOT NULL,
  site_id INT NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  INDEX `fkSupported_sites` (`site_id` ASC) ,
  CONSTRAINT `uk_supported_sites`
    UNIQUE KEY (`site_id`),
  CONSTRAINT `fkSupported_sites`
    FOREIGN KEY (`site_id` )
       REFERENCES `atp`.`site` (`id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_rocs (
  id INT AUTO_INCREMENT NOT NULL,
  name VARCHAR(256) NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  CONSTRAINT `uk_supported_rocs`
    UNIQUE KEY (`name`)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_vos (
  id INT AUTO_INCREMENT NOT NULL,
  name VARCHAR(256) NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  CONSTRAINT `uk_supported_vos`
    UNIQUE KEY (`name`)
)
ENGINE=INNODB;

CREATE TABLE IF NOT EXISTS supported_profiles (
  id INT AUTO_INCREMENT NOT NULL,
  name VARCHAR(256) NOT NULL,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id),
  CONSTRAINT `uk_supported_profiles`
    UNIQUE KEY (`name`)
)
ENGINE=INNODB;

