/*
	ATP:
		grant select, references  on service to lcg_sam_msdb_dev;
		grant select, references  on SERVICE_TYPE_FLAVOUR to lcg_sam_msdb_dev;
		grant select, references  on vo to lcg_sam_msdb_dev;

	MDDB:
		grant select, references on metric to lcg_sam_msdb_dev;
		grant select, references on profile to lcg_sam_msdb_dev;
		grant select, references on PROFILE_metric_MAP to lcg_sam_msdb_dev;
		grant select, references on PROFILE_RESOURCE to lcg_sam_msdb_dev;
*/

CREATE TABLE metricstatus (
  id INT NOT NULL ,
  description VARCHAR(45) NULL ,
  description_nagios VARCHAR(45) NULL ,
  "comment" VARCHAR(45) NULL ,
  PRIMARY KEY (id),
  CONSTRAINT uk_metricstatus UNIQUE (description)
);

CREATE  TABLE metricdetails (
  id INT NOT NULL  ,
  detail CLOB NOT NULL ,
  PRIMARY KEY (id)
);

CREATE  TABLE metricdata (
  id INT NOT NULL  ,
  metricstatus_id INT NOT NULL ,
  metricdetail_id INT NULL ,
  metric_id INT,
  service_id INT,
  vo_id INT,                           
  summarydata VARCHAR(256) NULL ,
  check_time INT NOT NULL ,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id) ,
  CONSTRAINT fk_metricdata_metricstatus
    FOREIGN KEY (metricstatus_id )
    REFERENCES metricstatus (id ),
  CONSTRAINT fk_metricdata_metricdetails
    FOREIGN KEY (metricdetail_id )
    REFERENCES metricdetails (id ),
  CONSTRAINT fk_metricdata_sam_metrics
    FOREIGN KEY (metric_id )
    REFERENCES metric (id ),
  CONSTRAINT fk_metricdata_sam_nodes
    FOREIGN KEY (service_id )
    REFERENCES service (id ),
  CONSTRAINT fk_metricdata_sam_vos
    FOREIGN KEY (vo_id )
    REFERENCES vo (id ),
  CONSTRAINT uk_metricdata UNIQUE (metricstatus_id, metric_id, service_id, vo_id, check_time))
  PARTITION BY RANGE (check_time) (
    PARTITION METRICDATA_09_2009 VALUES LESS THAN (1254355200),
    PARTITION METRICDATA_10_2009 VALUES LESS THAN (1257033600),
    PARTITION METRICDATA_11_2009 VALUES LESS THAN (1259625600),
    PARTITION METRICDATA_12_2009 VALUES LESS THAN (1262304000),
    PARTITION METRICDATA_01_2010 VALUES LESS THAN (1264982400),
    PARTITION METRICDATA_02_2010 VALUES LESS THAN (1267401600),
    PARTITION METRICDATA_03_2010 VALUES LESS THAN (1270080000),
    PARTITION METRICDATA_04_2010 VALUES LESS THAN (1272672000),
    PARTITION METRICDATA_05_2010 VALUES LESS THAN (1275350400),
    PARTITION METRICDATA_06_2010 VALUES LESS THAN (1277942400),
    PARTITION METRICDATA_07_2010 VALUES LESS THAN (1280620800),
    PARTITION METRICDATA_08_2010 VALUES LESS THAN (1283299200),
    PARTITION METRICDATA_09_2010 VALUES LESS THAN (1285891200),
    PARTITION METRICDATA_10_2010 VALUES LESS THAN (1288569600),
    PARTITION METRICDATA_11_2010 VALUES LESS THAN (1291161600),
    PARTITION METRICDATA_12_2010 VALUES LESS THAN (1293840000),
    PARTITION METRICDATA_01_2011 VALUES LESS THAN (1296518400),
    PARTITION METRICDATA_02_2011 VALUES LESS THAN (1298937600),
    PARTITION METRICDATA_03_2011 VALUES LESS THAN (1301616000),
    PARTITION METRICDATA_04_2011 VALUES LESS THAN (1304208000),
    PARTITION METRICDATA_05_2011 VALUES LESS THAN (1306886400),
    PARTITION METRICDATA_06_2011 VALUES LESS THAN (1309478400),
    PARTITION METRICDATA_07_2011 VALUES LESS THAN (1312156800),
    PARTITION METRICDATA_08_2011 VALUES LESS THAN (1314835200),
    PARTITION METRICDATA_09_2011 VALUES LESS THAN (1317427200),
    PARTITION METRICDATA_10_2011 VALUES LESS THAN (1320105600),
    PARTITION METRICDATA_11_2011 VALUES LESS THAN (1322697600),
    PARTITION METRICDATA_12_2011 VALUES LESS THAN (1325376000)
 );

CREATE TABLE statuschange_service_profile (
  id INT NOT NULL  ,
  service_id INT NULL ,
  profile_id INT NULL ,
  metricstatus_id INT NOT NULL,
  detail VARCHAR(256) NULL ,
  timestamp INT NOT NULL ,
  PRIMARY KEY (id), 
  CONSTRAINT uk_statuschange_service_prof
    UNIQUE (service_id, profile_id, timestamp),

  CONSTRAINT fk_stchg_serv_p_metricst
    FOREIGN KEY (metricstatus_id )
    REFERENCES metricstatus (id ),
  CONSTRAINT fk_stchg_service_p_services
    FOREIGN KEY (service_id )
    REFERENCES service (id ),
  CONSTRAINT fk_stchg_service_p_profiles
    FOREIGN KEY (profile_id )
    REFERENCES profile (id ),
  CONSTRAINT uk_statuschange_service_p UNIQUE (service_id, profile_id, metricstatus_id, timestamp))
  PARTITION BY RANGE (timestamp) (
    PARTITION STATUSCHANGE_S_P_09_2009 VALUES LESS THAN (1254355200),
    PARTITION STATUSCHANGE_S_P_10_2009 VALUES LESS THAN (1257033600),
    PARTITION STATUSCHANGE_S_P_11_2009 VALUES LESS THAN (1259625600),
    PARTITION STATUSCHANGE_S_P_12_2009 VALUES LESS THAN (1262304000),
    PARTITION STATUSCHANGE_S_P_01_2010 VALUES LESS THAN (1264982400),
    PARTITION STATUSCHANGE_S_P_02_2010 VALUES LESS THAN (1267401600),
    PARTITION STATUSCHANGE_S_P_03_2010 VALUES LESS THAN (1270080000),
    PARTITION STATUSCHANGE_S_P_04_2010 VALUES LESS THAN (1272672000),
    PARTITION STATUSCHANGE_S_P_05_2010 VALUES LESS THAN (1275350400),
    PARTITION STATUSCHANGE_S_P_06_2010 VALUES LESS THAN (1277942400),
    PARTITION STATUSCHANGE_S_P_07_2010 VALUES LESS THAN (1280620800),
    PARTITION STATUSCHANGE_S_P_08_2010 VALUES LESS THAN (1283299200),
    PARTITION STATUSCHANGE_S_P_09_2010 VALUES LESS THAN (1285891200),
    PARTITION STATUSCHANGE_S_P_10_2010 VALUES LESS THAN (1288569600),
    PARTITION STATUSCHANGE_S_P_11_2010 VALUES LESS THAN (1291161600),
    PARTITION STATUSCHANGE_S_P_12_2010 VALUES LESS THAN (1293840000),
    PARTITION STATUSCHANGE_S_P_01_2011 VALUES LESS THAN (1296518400),
    PARTITION STATUSCHANGE_S_P_02_2011 VALUES LESS THAN (1298937600),
    PARTITION STATUSCHANGE_S_P_03_2011 VALUES LESS THAN (1301616000),
    PARTITION STATUSCHANGE_S_P_04_2011 VALUES LESS THAN (1304208000),
    PARTITION STATUSCHANGE_S_P_05_2011 VALUES LESS THAN (1306886400),
    PARTITION STATUSCHANGE_S_P_06_2011 VALUES LESS THAN (1309478400),
    PARTITION STATUSCHANGE_S_P_07_2011 VALUES LESS THAN (1312156800),
    PARTITION STATUSCHANGE_S_P_08_2011 VALUES LESS THAN (1314835200),
    PARTITION STATUSCHANGE_S_P_09_2011 VALUES LESS THAN (1317427200),
    PARTITION STATUSCHANGE_S_P_10_2011 VALUES LESS THAN (1320105600),
    PARTITION STATUSCHANGE_S_P_11_2011 VALUES LESS THAN (1322697600),
    PARTITION STATUSCHANGE_S_P_12_2011 VALUES LESS THAN (1325376000)
);


CREATE  TABLE  metricdata_latest (
  id INT NOT NULL  ,
  metricstatus_id INT NOT NULL ,
  metricdetail_id INT NULL ,
  metric_id INT NULL ,
  service_id INT NULL ,
  vo_id INT NULL ,
  summarydata VARCHAR(256) NULL ,
  check_time INT NOT NULL ,
  insert_time INT NOT NULL ,
  PRIMARY KEY (id) ,
  CONSTRAINT fk_metricdata_latest_metrics
    FOREIGN KEY (metric_id )
    REFERENCES metric (id ),
  CONSTRAINT fk_metricdata_latest_nodes
    FOREIGN KEY (service_id )
    REFERENCES service (id ),
  CONSTRAINT fk_metricdata_latest_vos
    FOREIGN KEY (vo_id )
    REFERENCES vo (id ),
  CONSTRAINT fk_metricdata_l_metricstatus
    FOREIGN KEY (metricstatus_id )
    REFERENCES metricstatus (id ),
  CONSTRAINT fk_metricdata_l_metricdetails
    FOREIGN KEY (metricdetail_id )
    REFERENCES metricdetails (id ),
  CONSTRAINT uk_metricdata_l UNIQUE (metricstatus_id, metric_id, service_id, vo_id, check_time)
);

CREATE  TABLE  metricdata_rejected (
  id INT NOT NULL  ,
  metricstatus VARCHAR(25),
  metric VARCHAR(256),
  serviceURI VARCHAR(256),
  hostname VARCHAR(256),
  service_flavour VARCHAR(256),
  vo VARCHAR(256),
  summarydata CLOB NULL,
  check_time INT NOT NULL ,
  insert_time INT NOT NULL ,
  reason VARCHAR(256),
  PRIMARY KEY (id))
  PARTITION BY RANGE (check_time) (
    PARTITION METRICDATA_09_2009 VALUES LESS THAN (1254355200),
    PARTITION METRICDATA_10_2009 VALUES LESS THAN (1257033600),
    PARTITION METRICDATA_11_2009 VALUES LESS THAN (1259625600),
    PARTITION METRICDATA_12_2009 VALUES LESS THAN (1262304000),
    PARTITION METRICDATA_01_2010 VALUES LESS THAN (1264982400),
    PARTITION METRICDATA_02_2010 VALUES LESS THAN (1267401600),
    PARTITION METRICDATA_03_2010 VALUES LESS THAN (1270080000),
    PARTITION METRICDATA_04_2010 VALUES LESS THAN (1272672000),
    PARTITION METRICDATA_05_2010 VALUES LESS THAN (1275350400),
    PARTITION METRICDATA_06_2010 VALUES LESS THAN (1277942400),
    PARTITION METRICDATA_07_2010 VALUES LESS THAN (1280620800),
    PARTITION METRICDATA_08_2010 VALUES LESS THAN (1283299200),
    PARTITION METRICDATA_09_2010 VALUES LESS THAN (1285891200),
    PARTITION METRICDATA_10_2010 VALUES LESS THAN (1288569600),
    PARTITION METRICDATA_11_2010 VALUES LESS THAN (1291161600),
    PARTITION METRICDATA_12_2010 VALUES LESS THAN (1293840000),
    PARTITION METRICDATA_01_2011 VALUES LESS THAN (1296518400),
    PARTITION METRICDATA_02_2011 VALUES LESS THAN (1298937600),
    PARTITION METRICDATA_03_2011 VALUES LESS THAN (1301616000),
    PARTITION METRICDATA_04_2011 VALUES LESS THAN (1304208000),
    PARTITION METRICDATA_05_2011 VALUES LESS THAN (1306886400),
    PARTITION METRICDATA_06_2011 VALUES LESS THAN (1309478400),
    PARTITION METRICDATA_07_2011 VALUES LESS THAN (1312156800),
    PARTITION METRICDATA_08_2011 VALUES LESS THAN (1314835200),
    PARTITION METRICDATA_09_2011 VALUES LESS THAN (1317427200),
    PARTITION METRICDATA_10_2011 VALUES LESS THAN (1320105600),
    PARTITION METRICDATA_11_2011 VALUES LESS THAN (1322697600),
    PARTITION METRICDATA_12_2011 VALUES LESS THAN (1325376000)
);

INSERT INTO metricstatus(id, description, description_nagios, "comment")
VALUES (1, 'OK','OK',NULL);
INSERT INTO metricstatus(id, description, description_nagios, "comment")
VALUES(2,'WARNING','WARNING',NULL);
INSERT INTO metricstatus(id, description, description_nagios, "comment")
VALUES(3, 'CRITICAL','CRITICAL',NULL);
INSERT INTO metricstatus(id, description, description_nagios, "comment")
VALUES(4,'UNKNOWN','UNKNOWN',NULL);
INSERT INTO metricstatus(id, description, description_nagios, "comment")
VALUES(5,'MISSING','MISSING',NULL);

CREATE TABLE tested_services (
  id INT NOT NULL,
  service_id INT NOT NULL,
  profile_id INT NOT NULL,
  last_insert_time INT NOT NULL,
  PRIMARY KEY (id),
  CONSTRAINT uk_tested_services
    UNIQUE (service_id, profile_id),
  CONSTRAINT fk_tested_services_service
    FOREIGN KEY (service_id)
    REFERENCES service (id),
  CONSTRAINT fk_tested_services_profile
    FOREIGN KEY (profile_id)
    REFERENCES profile (id)
);

CREATE TABLE schema_details (
  id INT NOT NULL,
  ver VARCHAR(20) NOT NULL,
  creation_date TIMESTAMP NOT NULL,
  remarks VARCHAR(1000) NULL,
  PRIMARY KEY (id)
);

CREATE SEQUENCE MetricDetailsSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE SEQUENCE MetricDataSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE SEQUENCE StatusChange_ServiceProfileSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE SEQUENCE MetricData_LatestSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;


CREATE SEQUENCE MetricData_RejectedSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE SEQUENCE SchemaDetailsSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE SEQUENCE TestedServicesSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

  CREATE INDEX metricdata_metric_id_idx ON metricdata(metric_id);
  CREATE INDEX metricdata_service_id_idx ON metricdata(service_id);
  CREATE INDEX metricdata_vo_id_idx ON metricdata(vo_id);
  CREATE INDEX metricdata_metricstatus_id_idx ON metricdata(metricstatus_id);
  CREATE INDEX metricdata_metricdetail_id_idx ON metricdata(metricdetail_id);
  CREATE INDEX metricdata_check_time_idx ON metricdata(check_time);

  CREATE INDEX metricdata_l_metric_id_idx ON metricdata_latest(metric_id);
  CREATE INDEX metricdata_l_service_id_idx ON metricdata_latest(service_id);
  CREATE INDEX metricdata_l_vo_id_idx ON metricdata_latest(vo_id);
  CREATE BITMAP INDEX metricdata_l_metricst_id_idx ON metricdata_latest(metricstatus_id);
  CREATE INDEX metricdata_l_metricdet_id_idx ON metricdata_latest(metricdetail_id);
  CREATE INDEX metricdata_l_check_time_idx ON metricdata_latest(check_time);

  CREATE INDEX statuschg_s_p_metricst_id_idx ON statuschange_service_profile(metricstatus_id);
  CREATE INDEX statuschg_s_p_service_id_idx ON statuschange_service_profile(service_id);
  CREATE INDEX statuschg_s_p_profile_id_idx ON statuschange_service_profile(profile_id);

  CREATE INDEX tested_services_service_id_idx ON tested_services(service_id);
  CREATE INDEX tested_services_profile_id_idx ON tested_services(profile_id);


/* BEGIN: Temporary hack to make msg-consume2oracle working */

CREATE TABLE T_METRICOUTPUT
(
  SERVICEFLAVOUR VARCHAR2(256),
  METRICNAME NVARCHAR2(256),
  METRICSTATUS VARCHAR2(45),
  SUMMARYDATA VARCHAR2(256),
  DETAILSDATA CLOB,
  VONAME VARCHAR2(256),
  HOSTNAME VARCHAR2(512),
  "TIMESTAMP" VARCHAR2(256),
  SERVICEURI VARCHAR2(256)
);

create or replace view v_metricoutput as select * from t_metricoutput;

/* END: Temporary hack to make msg-consume2oracle working */


/* Additional functions */

create or replace type split_tbl as table of varchar2(32767);
/

create or replace function split
(
    p_list varchar2,
    p_del varchar2 := ','
) return split_tbl pipelined
is
    l_idx    pls_integer;
    l_list    varchar2(32767) := p_list;
    l_value    varchar2(32767);
begin
    loop
        l_idx := instr(l_list,p_del);
        if l_idx > 0 then
            pipe row(substr(l_list,1,l_idx-1));
            l_list := substr(l_list,l_idx+length(p_del));

        else
            pipe row(l_list);
            exit;
        end if;
    end loop;
    return;
end split;
/


CREATE OR REPLACE FUNCTION  w3ctime_to_uts (str char) RETURN PLS_INTEGER IS

  ora_date DATE := to_date(str,'yyyy-mm-dd"T"hh24:mi:ss"Z"');
  unix_epoch DATE := TO_DATE('19700101000000','YYYYMMDDHH24MISS');
  max_date DATE := TO_DATE('20380101000000','YYYYMMDDHH24MISS');
  min_date DATE := TO_DATE('19030101000000','YYYYMMDDHH24MISS');
  unix_ts PLS_INTEGER;

BEGIN

  IF ora_date > max_date THEN
    RAISE_APPLICATION_ERROR( -20902,
      'Date too large for 32bit UNIX timestamp' );
  ELSIF ora_date < min_date THEN
    RAISE_APPLICATION_ERROR( -20902,
      'Date too small for 32bit UNIX timestamp' );
  ELSE
    unix_ts := (ora_date - unix_epoch)
                                * 86400;
END IF;

RETURN (unix_ts);

END;

/


CREATE OR REPLACE FUNCTION to_unixts(oracle_ts IN TIMESTAMP) RETURN PLS_INTEGER IS

/**
* Converts an Oracle timestamp to a UNIX timestamp
*/

  unix_epoch DATE := TO_DATE('19700101000000','YYYYMMDDHH24MISS');
  max_date DATE := TO_DATE('20380101000000','YYYYMMDDHH24MISS');
  min_date DATE := TO_DATE('19030101000000','YYYYMMDDHH24MISS');
  unix_ts PLS_INTEGER;

BEGIN

  IF cast(oracle_ts as date) > max_date THEN
    RAISE_APPLICATION_ERROR( -20902,
      'Date too large for 32bit UNIX timestamp' );
  ELSIF cast(oracle_ts as date) < min_date THEN
    RAISE_APPLICATION_ERROR( -20902,
     'Date too small for 32bit UNIX timestamp' );
  ELSE
    unix_ts := (cast(sys_extract_utc(oracle_ts) as date) - unix_epoch) * 86400;
  END IF;

  RETURN (unix_ts);
END;
/

CREATE OR REPLACE FUNCTION unixts_to_date(unixts IN PLS_INTEGER) RETURN DATE IS
/**
 * Converts a UNIX timestamp into an Oracle DATE
 */
unix_epoch DATE := TO_DATE('19700101000000','YYYYMMDDHH24MISS');
max_ts PLS_INTEGER := 2145916799; -- 2938-12-31 23:59:59
min_ts PLS_INTEGER := -2114380800; -- 1903-01-01 00:00:00
oracle_date DATE;

BEGIN
  IF unixts > max_ts THEN
      RAISE_APPLICATION_ERROR( -20901,'UNIX timestamp too large for 32 bit limit');
  ELSIF unixts < min_ts THEN
      RAISE_APPLICATION_ERROR(-20901, 'UNIX timestamp too small for 32 bit limit');
  ELSE
      oracle_date := unix_epoch + NUMTODSINTERVAL(unixts, 'SECOND');
  END IF;

  RETURN (oracle_date);
  
END;
/

CREATE OR REPLACE FUNCTION uts_to_w3ctime (uts INTEGER)
RETURN VARCHAR2 AS
BEGIN
  RETURN to_char(unixts_to_date(uts),'yyyy-mm-dd"T"hh24:mi:ss"Z"');
END;
/

CREATE OR REPLACE FUNCTION getMetricFreshness (in_service_id INTEGER, in_profile_id INTEGER, maxAgeInHours INTEGER)
RETURN INTEGER AS
  metricFreshness INTEGER;
BEGIN

  --"metricFreshness = urrent_timestamp - (1/24)*maxAgeInHours" if any metric with status "MISSING" found  
  select min(to_unixts(current_timestamp - (1/24)*maxAgeInHours)) into metricFreshness from METRICDATA_LATEST
  where metric_id in
    (select distinct metric_id from profile_metric_map where profile_id = in_profile_id
       and service_type_flavour_id in
         (select flavour_id from service where id = in_service_id)
    )
    and metricstatus_id = 5 /*MISSING METRIC RESULT*/
    and service_id = in_service_id;

  if metricFreshness is null then
	  select min(check_time) into metricFreshness from METRICDATA_LATEST
	  where metric_id in
	    (select distinct metric_id from profile_metric_map where profile_id = in_profile_id
	       and service_type_flavour_id in
	         (select flavour_id from service where id = in_service_id)
	    )
	    and metricstatus_id != 5 /*MISSING METRIC RESULT*/
	    and service_id = in_service_id;
  
  end if;

  return(metricFreshness);    
END;
/

  CREATE OR REPLACE PROCEDURE test_sam_ms_jobs

  AS
    sqlStatement VARCHAR2(2000);

    cursor jobs_cursor is select 'begin dbms_scheduler.run_job(''' ||job_name || ''', TRUE); end;' from all_scheduler_jobs
      where job_name like 'SAM_MS%'
      order by job_name;
  BEGIN
    OPEN jobs_cursor;
      LOOP
        FETCH jobs_cursor INTO sqlStatement;
        EXIT WHEN jobs_cursor%NOTFOUND;

        dbms_output.put_line('Executing sql command: ' || sqlStatement);
        execute immediate sqlStatement;


      END LOOP;
    CLOSE jobs_cursor;
  END;
  
/  

CREATE TABLE statuschangeRecalculated (
  id INT NOT NULL,
  statuschange_service_id INT NULL,
  insert_time INT,
  PRIMARY KEY (id),
  CONSTRAINT uk_statuschangeRecalculated UNIQUE (statuschange_service_id)
);

CREATE SEQUENCE statuschangeRecalculatedSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE TABLE DataLoader_LastCheckTime
(
  check_time INT NOT NULL,
  CONSTRAINT uk_DataLoader_LastCheckTime UNIQUE (check_time)
);

CREATE TABLE MetricDataForRecalculation
(
  ID INT NOT NULL,
  MetricData_Spool_id INT NOT NULL,
  SERVICEFLAVOUR VARCHAR2(256),
  METRICNAME NVARCHAR2(256),
  METRICSTATUS VARCHAR2(45),
  SUMMARYDATA VARCHAR2(256),
  DETAILSDATA CLOB,
  VONAME VARCHAR2(256),
  HOSTNAME VARCHAR2(512),
  SERVICEURI VARCHAR2(256),
  check_time INT,
  insert_time INT,
  CONSTRAINT uk_MetricDataForRecalculation UNIQUE (MetricData_Spool_id)
);

CREATE SEQUENCE MetricDataForRecalculationSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE TABLE MetricDataForRecalc_rejected
(
  ID INT NOT NULL,
  MetricData_Spool_id INT NOT NULL,
  SERVICEFLAVOUR VARCHAR2(256),
  METRICNAME NVARCHAR2(256),
  METRICSTATUS VARCHAR2(45),
  SUMMARYDATA VARCHAR2(256),
  DETAILSDATA CLOB,
  VONAME VARCHAR2(256),
  HOSTNAME VARCHAR2(512),
  SERVICEURI VARCHAR2(256),
  check_time INT,
  insert_time INT,
  CONSTRAINT uk_MetricDataForRecalc_r UNIQUE (MetricData_Spool_id)
);


CREATE OR REPLACE TRIGGER MetricData_ForRec_trig BEFORE INSERT ON MetricDataForRecalculation
FOR EACH ROW
BEGIN
SELECT MetricDataForRecalculationSeq.NEXTVAL into :new.id FROM dual;
END;
/

CREATE TABLE MetricData_Spool
(
  ID INT NOT NULL,
  SERVICEFLAVOUR VARCHAR2(256),
  METRICNAME NVARCHAR2(256),
  METRICSTATUS VARCHAR2(45),
  SUMMARYDATA VARCHAR2(256),
  DETAILSDATA CLOB,
  VONAME VARCHAR2(256),
  HOSTNAME VARCHAR2(512),
  SERVICEURI VARCHAR2(256),
  check_time INT,
  insert_time INT
);


CREATE SEQUENCE MetricData_SpoolSeq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  noCYCLE;
  
CREATE TABLE MetricData_Spool_Rejected
(
  ID INT NOT NULL,
  SERVICEFLAVOUR VARCHAR2(256),
  METRICNAME NVARCHAR2(256),
  METRICSTATUS VARCHAR2(45),
  SUMMARYDATA VARCHAR2(256),
  DETAILSDATA CLOB,
  VONAME VARCHAR2(256),
  HOSTNAME VARCHAR2(512),
  SERVICEURI VARCHAR2(256),
  check_time INT,
  insert_time INT
);  
