DELIMITER //

USE `metricstore`;

DROP TRIGGER IF EXISTS load_metricdata_fer;

CREATE TRIGGER load_metricdata_fer

AFTER INSERT ON metricdata FOR EACH ROW

BEGIN
  DECLARE v_check_time INT;
  DECLARE done INT DEFAULT 0;
  DECLARE metric_curs CURSOR FOR SELECT check_time FROM metricdata_latest 
    WHERE fqan_id <=> NEW.fqan_id AND vo_id <=> NEW.vo_id AND metric_id <=> NEW.metric_id 
      AND service_id <=> NEW.service_id;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  OPEN metric_curs;
  FETCH metric_curs INTO v_check_time;
  CLOSE metric_curs;

  IF ((v_check_time IS NOT NULL) AND (v_check_time <= NEW.check_time)) THEN

    DELETE FROM metricdata_latest WHERE fqan_id <=> NEW.fqan_id AND vo_id <=> NEW.vo_id AND metric_id <=> NEW.metric_id AND service_id <=> NEW.service_id;
    INSERT INTO metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (NEW.metricstatus_id, NEW.metricdetail_id, NEW.metric_id, NEW.service_id, NEW.fqan_id, NEW.vo_id, NEW.summarydata, NEW.gatheredat_id, NEW.check_time, NEW.insert_time);
  ELSEIF (v_check_time IS NULL) THEN 
    INSERT INTO metricdata_latest(metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (NEW.metricstatus_id, NEW.metricdetail_id, NEW.metric_id, NEW.service_id, NEW.fqan_id, NEW.vo_id, NEW.summarydata, NEW.gatheredat_id, NEW.check_time, NEW.insert_time);
  END IF;

END //

DROP TRIGGER IF EXISTS ins_metrics_in_sup_services_fer;

CREATE TRIGGER ins_metrics_in_sup_services_fer

AFTER INSERT ON metrics_in_supported_services FOR EACH ROW

BEGIN
  insert into metrics_in_supp_serv_history (service_id, metric_id, vo_id, fqan_id, profile_id, timestamp, operation)
  values(NEW.service_id, NEW.metric_id, NEW.vo_id, NEW.fqan_id, NEW.profile_id, unix_timestamp(), 'insert');
END //


DROP TRIGGER IF EXISTS del_metrics_in_sup_services_fer;

CREATE TRIGGER del_metrics_in_sup_services_fer

AFTER DELETE ON metrics_in_supported_services FOR EACH ROW

BEGIN

  insert into metrics_in_supp_serv_history (service_id, metric_id, vo_id, fqan_id, profile_id, timestamp, operation)
  values(OLD.service_id, OLD.metric_id, OLD.vo_id, OLD.fqan_id, OLD.profile_id, unix_timestamp(), 'delete');

  insert into metricdata(metricstatus_id, metric_id, service_id, vo_id, check_time, insert_time, fqan_id)
  values((select id from metricstatus where description = 'REMOVED'), OLD.metric_id, OLD.service_id, OLD.vo_id, unix_timestamp(), unix_timestamp(), OLD.fqan_id);

END //

DELIMITER ;
