CREATE OR REPLACE PACKAGE DataLoader AS  -- specification

  PROCEDURE calculateStatusChange(
    serviceId in INTEGER, 
    metricId in INTEGER, 
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN, 
    maxAgeOfTupleInHours in INTEGER);

  FUNCTION checkIfCalculationNeeded(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER)
    RETURN BOOLEAN;

  PROCEDURE calculateStatusChangeFqan(fqanId INT, serviceId INT, metricId INT, metricstatusId INT, check_time INT, voId INT,
                                       recalculationNeeded BOOLEAN, maxAgeOfTupleInHours INT);
                                       
  PROCEDURE calculateStatusChangeMetric(metricId INT, serviceId INT, voId INT, metricstatusId INT, check_time INT);

  PROCEDURE calculateStatusChangeMetFqan(metricId INT, serviceId INT, fqanId INT, voId INT, metricstatusId INT, check_time INT);

  PROCEDURE calculateStatusChangeProfile(
    profileId in INTEGER,
    serviceId in INTEGER,
    metricId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN,  
    maxAgeOfTupleInHours in INTEGER);  

  PROCEDURE calculateStatusChangeProfileFq(profileId INT, serviceId INT, fqanId INT, metricId INT, metricstatusId INT, 
                     check_time INT, voId INT, recalculationNeeded BOOLEAN, maxAgeOfTupleInHours INT);

  PROCEDURE findCurrentStatusFqan (profileId INT, serviceId INT, fqanId INT, metricstatusId INT, check_time INT,
              voId INT, curr_servicestatus_id OUT INT, old_metricstatus_id OUT INT, old_timestamp OUT INT, num_row OUT INT);

  PROCEDURE findMetricsForStatusRecalc (in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER);
 
  PROCEDURE loadmetricdata(maxAgeInHours IN INT, ageInMinutes IN INT);
  
  PROCEDURE loadmetricdatatospool (in_serviceflavour IN VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2, 
                 in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2, in_fqan IN VARCHAR2, 
                 in_hostname IN VARCHAR2, in_timestamp IN VARCHAR2, in_gatheredat IN VARCHAR2);


  PROCEDURE loadmetricdatatospool_OSG (in_servicetype IN VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2, 
                 in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2, in_serviceuri IN VARCHAR2, 
                 in_timestamp IN VARCHAR2);

  PROCEDURE markRemovedMetrics;
                        
  PROCEDURE insertRowToMetricData(in_service_id IN INT, in_metric_id IN INT, in_metricstatus_id IN INT, in_summarydata IN VARCHAR,
                   in_detailsdata IN CLOB, in_fqan_id IN INT, in_vo_id IN INT, in_gatheredat_id IN INT, in_check_time IN INT, in_insert_time IN INT,
                   in_recalculationNeeded IN BOOLEAN, in_maxAgeOfTupleInHours IN INT);
                   
  PROCEDURE loadMetricDataWithRecalc(in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER);

  PROCEDURE insertMissingMetrics(maxAgeInHours INTEGER, ageInMinutes INTEGER);
 
  FUNCTION getServiceStatus(   
    profileId IN INTEGER,
    serviceId IN INTEGER,
    checkTime IN INTEGER)
  RETURN INTEGER;

  FUNCTION getServiceStatusFqan(
    fqanId IN INTEGER,   
    profileId IN INTEGER,
    serviceId IN INTEGER,
    checkTime IN INTEGER)
  RETURN INTEGER;


 PROCEDURE addFqanToSupportedServices(in_service_type_flavour VARCHAR2, in_hostname VARCHAR2, 
                 in_fqan VARCHAR2, in_profile VARCHAR2);
  
 PROCEDURE addMetricToSupportedServices(in_service_type_flavour VARCHAR2, in_hostname VARCHAR2, in_vo VARCHAR2,
                 in_fqan VARCHAR2, in_profile VARCHAR2, in_metricname VARCHAR2, in_frequency INT);
   
END DataLoader;
/

CREATE OR REPLACE PACKAGE BODY DataLoader AS 

  PROCEDURE calculateStatusChange(
    serviceId in INTEGER, 
    metricId in INTEGER, 
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN, 
    maxAgeOfTupleInHours in INTEGER) AS
  
    v_profileId INTEGER;
    CURSOR profileCurs is select id from profile;  
  
  BEGIN
    
    calculateStatusChangeMetric(metricId, serviceId, voId, metricstatusId, check_time);
  
    OPEN profileCurs;
  
    LOOP
  
      FETCH profileCurs INTO v_profileId;
      EXIT WHEN profileCurs%NOTFOUND;
  
      IF checkIfCalculationNeeded(v_profileId, serviceId, metricId, voId) THEN
        IF (nvl(getServiceStatus(v_profileId, serviceId, check_time), -1) != metricstatusId) THEN
          calculateStatusChangeProfile(v_profileId, serviceId, metricId, metricstatusId, check_time, voId,
                                       recalculationNeeded, maxAgeOfTupleInHours);

          BEGIN
            insert into tested_services (id, service_id, profile_id, last_insert_time)
            values(TESTEDSERVICESSEQ.nextval, serviceId, v_profileId,to_unixts(current_timestamp));
          EXCEPTION
            WHEN dup_val_on_index then
              null;    -- don't do anything, this record already exists
          END;
                                                        
        END IF;          
      END IF;
                  
    END LOOP;
   
  
    CLOSE profileCurs;
  
  END calculateStatusChange; 

  FUNCTION checkIfCalculationNeeded(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER)
  RETURN BOOLEAN 
  
  AS  
    v_counter INTEGER;
    
    CURSOR metricCurs IS SELECT count(1) FROM profile_METRIC_map 
    where profile_id = profileId and (test_vo_id = voId or voId is null) and metric_id = metricId and service_type_flavour_id =
    (select flavour_id from service where id = serviceId);
  
  BEGIN
  
    OPEN metricCurs;
    FETCH metricCurs INTO v_counter;
    CLOSE metricCurs;

    IF (v_counter > 0) THEN
      RETURN TRUE;
    ELSE 
      RETURN FALSE;
    END IF;

  END checkIfCalculationNeeded;

  /*WLAPKA - do the same for FQANs*/
  FUNCTION findCurrentStatusRecalculation(profileId in INTEGER,
    serviceId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    maxAgeOfTupleInHours in INTEGER,
    curr_servicestatus_id out INTEGER,
    old_metricstatus_id out INTEGER,
    old_timestamp out INTEGER         
  ) RETURN INTEGER AS

    num_row INTEGER;
  
  BEGIN
  
    num_row := 0;
  
    select count(*) INTO num_row
    from statuschange_service_profile 
    where service_id = serviceId and profile_id = profileId and timestamp <= check_time;
 
    IF num_row > 0 THEN
      SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
      FROM statuschange_service_profile 
      WHERE service_id = serviceId AND profile_id = profileId AND timestamp =
        (SELECT max(timestamp) FROM statuschange_service_profile
        WHERE service_id = serviceId AND profile_id = profileId and timestamp <= check_time);
  
      IF (metricstatusId < old_metricstatus_id) THEN
        select max(metricstatus_id) INTO curr_servicestatus_id from metricdata m where
          m.service_id = serviceId
          and m.check_time >= to_unixts(current_timestamp-(maxAgeOfTupleInHours+1)/24)
          and m.metric_id || ' ' || (m.check_time) in (
            select metric_id || ' ' || max(check_time) from metricdata where metric_id in (
              SELECT m.metric_id FROM profile_METRIC_map m
                 where m.profile_id = profileId
                   and (test_vo_id = voId or voId is null)
                   and m.service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
              )
            and service_id = serviceId        
            and check_time >= to_unixts(current_timestamp-(maxAgeOfTupleInHours+1)/24)
            group by metric_id, service_id
          );                               
      END IF;
      
      return num_row;
    ELSE
      return 0;
    END IF;
       
  END;
                            
  /*WLAPKA - Check this function, compare with MySQL*/
  FUNCTION findCurrentStatus (profileId in INTEGER,
    serviceId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    curr_servicestatus_id out INTEGER,
    old_metricstatus_id out INTEGER,
    old_timestamp out INTEGER
    ) RETURN INTEGER AS

    num_row INTEGER;

  BEGIN
    num_row := 0;
  
    select count(*) INTO num_row
    from statuschange_service_profile 
    where service_id = serviceId and profile_id = profileId;
  
    IF num_row > 0 THEN
      SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
      FROM statuschange_service_profile 
      WHERE service_id = serviceId AND profile_id = profileId AND timestamp =
        (SELECT max(timestamp) FROM statuschange_service_profile
        WHERE service_id = serviceId AND profile_id = profileId and timestamp <= check_time);
  
      IF (metricstatusId < old_metricstatus_id) THEN
          select max(metricstatus_id) INTO curr_servicestatus_id from metricdata_latest where metric_id in ( --metricdata_latest_tmp
            SELECT m.metric_id FROM profile_METRIC_map m
              where m.profile_id = profileId 
                and (test_vo_id = voId or voId is null)
                and m.service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
            ) 
            and service_id = serviceId;
      END IF;
      
      return num_row;           
    ELSE
      return 0;  
    END IF;
           
  END;
  


  PROCEDURE calculateStatusChangeProfile(
    profileId in INTEGER,
    serviceId in INTEGER,
    metricId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN,  
    maxAgeOfTupleInHours in INTEGER) AS
  
    old_metricstatus_id INTEGER; 
    old_timestamp INTEGER; 
    num_row INTEGER;
    curr_servicestatus_id INTEGER;
    wasStatusChange BOOLEAN;
  BEGIN
    wasStatusChange := true;
    
    IF (recalculationNeeded) THEN
      num_row := findCurrentStatusRecalculation(profileId, serviceId, metricstatusId, check_time, voId, maxAgeOfTupleInHours, curr_servicestatus_id, old_metricstatus_id, old_timestamp);
    ELSE
      num_row := findCurrentStatus(profileId, serviceId, metricstatusId, check_time, voId, curr_servicestatus_id, old_metricstatus_id, old_timestamp); 
    END IF;  
  
     
    IF num_row > 0 THEN
                   
      IF (metricstatusId > old_metricstatus_id) THEN
        IF old_timestamp != check_time THEN
          INSERT INTO statuschange_service_profile(id, service_id, profile_id, metricstatus_id, detail, timestamp)
          VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, NULL, check_time);
        ELSE
           UPDATE statuschange_service_profile SET metricstatus_id = metricstatusId WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
        END IF;
  
      ELSIF ((metricstatusId < old_metricstatus_id) AND (curr_servicestatus_id < old_metricstatus_id)) THEN
  
        IF old_timestamp != check_time THEN
          INSERT INTO statuschange_service_profile(id, service_id, profile_id, metricstatus_id, detail, timestamp)
          VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, NULL, check_time);
        ELSE
           UPDATE statuschange_service_profile SET metricstatus_id = curr_servicestatus_id WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
        END IF;
  	  
      ELSE
        wasStatusChange := false;
      END IF;
    ELSE 
      INSERT INTO statuschange_service_profile(id, service_id, profile_id, metricstatus_id, detail, timestamp)
      VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, NULL, check_time); 
    END IF;

    IF (wasStatusChange AND recalculationNeeded) THEN
      delete from statuschangeRecalculated
        where statuschange_service_id in (
          select id from statuschange_service_profile 
            where service_id = serviceId and profile_id = profileId and timestamp > check_time
        );

      delete from statuschange_service_profile 
      where service_id = serviceId and profile_id = profileId and timestamp > check_time;
      
      insert into statuschangeRecalculated(id, statuschange_service_id, insert_time)
      values (statuschangeRecalculatedSeq.nextval , StatusChange_ServiceProfileSeq.currval, to_unixts(current_timestamp));
    END IF;	
   
  END;  

  FUNCTION getOrAddFqan(in_name VARCHAR2) RETURN INT
  AS
  
    v_id INT DEFAULT NULL;
  BEGIN
  
    v_id := NULL;
  
    IF in_name is not NULL THEN
      BEGIN
        select id into v_id from fqan where lower(name) = lower(in_name);
  
        EXCEPTION when NO_DATA_FOUND THEN     
          insert into fqan(id, name)
            values(fqan_seq.nextval, in_name);
          
        select id into v_id from fqan where lower(name) = lower(in_name);
  
      END;
    END IF; 
  
    RETURN v_id;
  
  END;

  FUNCTION getOrAddGatheredAt(in_gatheredat IN VARCHAR2) RETURN INT
  AS
    v_id INT DEFAULT NULL;
  BEGIN
    
    IF in_gatheredat is not NULL THEN
      BEGIN
        select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);
    
        EXCEPTION when NO_DATA_FOUND THEN     
          insert into gatheredat(id, name)
             values(gatheredat_seq.nextval, in_gatheredat);
              
         select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);
         
      END; 
    END IF;
    
    RETURN v_id;
  END;

  FUNCTION getVoIdForName (in_voname in VARCHAR2) RETURN INTEGER AS  
    v_vo_id INT;  
  BEGIN
    select id into v_vo_id from vo where lower(voname) = lower(in_voname);
    
    RETURN v_vo_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  FUNCTION getMetricIdForName (in_metricname in VARCHAR2) RETURN INTEGER AS  
    v_metric_id INT;  
  BEGIN
    select id into v_metric_id from metric where lower(name) = lower(in_metricname)
      and version = (select max(version) from metric where lower(name) = lower(in_metricname));

    RETURN v_metric_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  FUNCTION getMetricStatusIdForName (in_name in VARCHAR2) RETURN INTEGER AS  
    v_id INT;  
  BEGIN
    select id into v_id from metricstatus where lower(description) = lower(in_name);    
    RETURN v_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;
  
  FUNCTION getProfileIdForName (in_profile in VARCHAR2) RETURN INTEGER AS  
    v_profile_id INT;  
  BEGIN
    select id into v_profile_id from profile where lower(name) = lower(in_profile);
    
    RETURN v_profile_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;
                               
  FUNCTION getServiceFlavourIdForName (in_name in VARCHAR2) RETURN INTEGER AS  
    v_id INT;  
  BEGIN
    select id into v_id from service_type_flavour where lower(flavourname) = lower(in_name);
    RETURN v_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  /* WLAPKA: add usage of metrics_in_supported_services, when this table will be in use.
       select id into v_service_id from atp.service where lower(hostname) = lower(in_hostname) and flavour_id = v_serviceflavour_id
                                       and id in (select distinct service_id from metrics_in_supported_services);   
  */
  FUNCTION getServiceIdForHostAndFlavour (in_hostname in VARCHAR2, in_flavour in VARCHAR2) RETURN INTEGER AS  
    v_id INT;  
  BEGIN
    select id into v_id from service where lower(hostname) = lower(in_hostname) and flavour_id = in_flavour;
    RETURN v_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  PROCEDURE findMetricsForStatusRecalc (in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER) AS
                                              
  BEGIN

    insert into MetricDataForRecalculation(id, metricdata_spool_id, gatheredat_id, metricstatus_id, detailsdata, 
                            metric_id, service_id, fqan_id, vo_id, summarydata, check_time, insert_time)
      select METRICDATAFORRECALCULATIONSEQ.nextval, id, gatheredat_id, metricstatus_id, detailsdata, 
                            metric_id, service_id, fqan_id, vo_id, summarydata, check_time, insert_time from metricdata_spool
        where check_time < in_minCheckTime 
          /*and id not in (
            select id from MetricData_Spool
              where check_time < in_maxCheckTime
                and check_time >= in_minCheckTime
          )*/;
    
    delete from metricdata_spool where id in (select metricdata_spool_id from MetricDataForRecalculation);
    
    insert into MetricDataForRecalculation(id, metricdata_spool_id, gatheredat_id, metricstatus_id, detailsdata, 
                            metric_id, service_id, fqan_id, vo_id, summarydata, check_time, insert_time)
      select  METRICDATAFORRECALCULATIONSEQ.nextval, id, gatheredat_id, metricstatus_id, detailsdata, 
                            metric_id, service_id, fqan_id, vo_id, summarydata, check_time, insert_time
        from metricdata_spool 
          where service_id in (
            select service_id from MetricDataForRecalculation
          );

    delete from metricdata_spool where id in (select MetricData_Spool_id from MetricDataForRecalculation);
 
    commit;   
  
  END;

  PROCEDURE loadMetricDataWithRecalc (in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER) AS

    CURSOR metricResultsCurs IS select id, gatheredat_id, metricstatus_id,
                            metric_id, service_id, fqan_id, vo_id, summarydata, check_time, insert_time from MetricDataForRecalculation; 

    metricResultRec metricResultsCurs%ROWTYPE;
    v_detailsdata CLOB;
        
  BEGIN
   
    findMetricsForStatusRecalc(in_maxCheckTime, in_minCheckTime);
  
    OPEN metricResultsCurs;

    LOOP 
    
      FETCH metricResultsCurs INTO metricResultRec;
      EXIT WHEN metricResultsCurs%NOTFOUND;
      
      BEGIN
      
        select detailsdata into v_detailsdata from MetricDataForRecalculation where id = metricResultRec.id;      

        insertRowToMetricData(metricResultRec.service_id, metricResultRec.metric_id, metricResultRec.metricstatus_id, metricResultRec.summarydata,
                   v_detailsdata, metricResultRec.fqan_id, metricResultRec.vo_id, metricResultRec.gatheredat_id, metricResultRec.check_time,
                   metricResultRec.insert_time, true, 24);
      
        exception
          when DUP_VAL_ON_INDEX then
            dbms_output.put_line('Row already exists: ' || metricResultRec.id);
          when NO_DATA_FOUND then
            insert into MetricDataForRecalc_rejected select * from MetricDataForRecalculation where id = metricResultRec.id;			             		    
      END;

      delete from MetricDataForRecalculation where id = metricResultRec.id;
      commit;
     
    END LOOP;    
  
  END loadMetricDataWithRecalc;

  PROCEDURE loadmetricdatatospool_OSG (in_servicetype IN VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2, 
                 in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2, in_serviceuri IN VARCHAR2, 
                 in_timestamp IN VARCHAR2) AS
                 
  BEGIN
  
  --osgServiceMap={'OSG-CE':'OSGCE','OSG-SRMv1':'OSGSRMv1','OSG-SRMv2':'OSGSRMv2','OSG-BestmanXrootd':'OSGBestm','OSG-GridFtp':'OSGGdFTP'}
  
  /*Looking at old SAM DB: 
  select distinct m.servicetype, s.serviceabbr from testdef t, metricdescription m, service s
    where t.testid = m.testid and m.servicetype like '%osg%'
    and t.serviceid = s.serviceid;
osg-ce-general	OSGGridF
osg-ce-gums	OSGCE
osg-ce-gums	OSGGridF
osg-gridftp	OSGGridF
osg-ce-general	OSGCE
osg-ce-gums	OSGSRMv2
osg-srmv2	OSGSRMv2    
  */
      
  

   
    dbms_output.put_line(in_servicetype || ' ' || in_metricname || ' ' || in_metricstatus || ' ' || 
                 in_summarydata || ' ' || in_detailsdata || ' ' || in_voname || ' ' || in_serviceuri || ' ' || 
                 in_timestamp);
  
  END;                 

  PROCEDURE loadmetricdatatospool (in_serviceflavour IN VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2, 
                 in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2, in_fqan IN VARCHAR2, 
                 in_hostname IN VARCHAR2, in_timestamp IN VARCHAR2, in_gatheredat IN VARCHAR2) AS

  v_gatheredat_id INT;
  v_serviceflavour_name VARCHAR2(100);
  v_serviceflavour_id INT;
  v_metricstatus_id INT;
  v_metric_id INT;
  v_fqan_id INT;
  v_vo_id INT;
  v_service_id INT;
  v_metricdetail_id INT;
  intFoundPos INT;
  strElement VARCHAR2(100);
  v_check_time INT;
  v_insert_time INT;
  /*Temporary hack - See: https://savannah.cern.ch/bugs/?62833*/ 
  CURSOR flavourCurs IS select decode(COLUMN_VALUE, 'sBDII', 'Site-BDII' /*, 'SRMv2', 'SRMv1'*/, COLUMN_VALUE) from table(split(in_serviceflavour));

  BEGIN
    OPEN flavourCurs;     
    
    LOOP
    
      FETCH flavourCurs INTO v_serviceflavour_name;
      EXIT WHEN flavourCurs%NOTFOUND;    

      v_serviceflavour_name := trim(v_serviceflavour_name);
      v_metric_id := getMetricIdForName(in_metricname);                                                    
      v_metricstatus_id := getMetricStatusIdForName(in_metricstatus);                                                            
      v_vo_id := getVoIdForName(in_voname);                                                                
      v_fqan_id := getOrAddFqan(in_fqan);
      v_serviceflavour_id := getServiceFlavourIdForName(v_serviceflavour_name);
      v_service_id := getServiceIdForHostAndFlavour(in_hostname, v_serviceflavour_id);        
      v_gatheredat_id := getOrAddGatheredAt(in_gatheredat);
      v_check_time := w3ctime_to_uts(in_timestamp);
      v_insert_time := to_unixts(current_timestamp);
         
      IF v_service_id is not NULL and v_metric_id is not NULL THEN
        insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id,
                                       vo_id, fqan_id, summarydata, check_time, insert_time)
          VALUES (MetricData_SpoolSeq.nextval, v_gatheredat_id, v_metricstatus_id, in_detailsdata, v_metric_id, v_service_id, v_vo_id, v_fqan_id, in_summarydata,
                  v_check_time, v_insert_time);
      ELSIF (v_service_id is NULL and v_metric_id is NULL) THEN
        insert into metricdata_rejected(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id and metric_id NULL');
      ELSIF v_service_id is NULL THEN
        insert into metricdata_rejected(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id NULL');
      ELSIF v_metric_id is NULL THEN
        insert into metricdata_rejected(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'metric_id NULL');
      END IF;                 
                      
    END LOOP;
  
  END loadmetricdatatospool;


--WLAPKA
  PROCEDURE loadmetricdata(maxAgeInHours IN INT, ageInMinutes IN INT) AS
  
    v_maxCheckTime INT;
    v_minCheckTime INT;
    v_detailsdata CLOB;
       
    CURSOR metricResultsCurs IS select id, gatheredat_id, metricstatus_id, metric_id, service_id,
    fqan_id, vo_id, summarydata, check_time, insert_time from metricdata_spool         
      where check_time < v_maxCheckTime and check_time >= v_minCheckTime
      order by check_time; 
                  
    metricResultRec metricResultsCurs%ROWTYPE;  
       
  BEGIN
	
	BEGIN
      select max(check_time) into v_minCheckTime from DataLoader_LastCheckTime;
 
      exception
        when NO_DATA_FOUND
          then v_minCheckTime := 0;
	END;

    v_maxCheckTime := to_unixts(sysdate - ageInMinutes/(24*60));
    
    findMetricsForStatusRecalc(v_maxCheckTime, v_minCheckTime);
      
    OPEN metricResultsCurs;
  
    LOOP 
    
      FETCH metricResultsCurs INTO metricResultRec;
      EXIT WHEN metricResultsCurs%NOTFOUND;
      
      BEGIN
         select detailsdata into v_detailsdata from metricdata_spool where id = metricResultRec.id;
         
         insertRowToMetricData(metricResultRec.service_id, metricResultRec.metric_id, metricResultRec.metricstatus_id, metricResultRec.summarydata,
                   v_detailsdata, metricResultRec.fqan_id, metricResultRec.vo_id, metricResultRec.gatheredat_id, 
                   metricResultRec.check_time, metricResultRec.insert_time, false, maxAgeInHours);

        dbms_output.put_line(metricResultRec.id);
        exception
          when DUP_VAL_ON_INDEX then
            dbms_output.put_line('Row already exists: ' || metricResultRec.id);
          when NO_DATA_FOUND then
            insert into metricdata_spool_rejected select * from metricdata_spool where id = metricResultRec.id;			 
      END;

      	  	            
      delete from metricdata_spool where id = metricResultRec.id;
      commit;
     
    END LOOP;
  
    CLOSE metricResultsCurs;

    loadMetricDataWithRecalc (v_maxCheckTime, v_minCheckTime);

    delete from DataLoader_LastCheckTime;
    insert into DataLoader_LastCheckTime(check_time) values(v_maxCheckTime);
    commit;

    insertMissingMetrics(maxAgeInHours, ageInMinutes);

  END;


--WLAPKA



  --WLAPKA
  PROCEDURE insertRowToMetricData(in_service_id IN INT, in_metric_id IN INT, in_metricstatus_id IN INT, in_summarydata IN VARCHAR,
                   in_detailsdata IN CLOB, in_fqan_id IN INT, in_vo_id IN INT, in_gatheredat_id IN INT, in_check_time IN INT, in_insert_time IN INT,
                   in_recalculationNeeded IN BOOLEAN, in_maxAgeOfTupleInHours IN INT) AS

                  
    v_metricdetail_id INT;
      
  BEGIN
 
    if in_detailsdata is not null then
      insert into metricdetails(id, detail, timestamp)
      values (MetricDetailsSeq.nextval, in_detailsdata, in_insert_time);
        
      select MetricDetailsSeq.currval into v_metricdetail_id from dual;        
    end if; 
    
    insert into metricdata(id, metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
    VALUES (MetricDataSeq.nextval, in_metricstatus_id, v_metricdetail_id, in_metric_id, in_service_id, in_fqan_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time); 


   calculateStatusChange(in_service_id, in_metric_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours);
   calculateStatusChangeFqan(in_fqan_id, in_service_id, in_metric_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours);
    
--   calculateStatusChangeFqan(in_fqan_id, in_service_id, in_metric_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours);
   
  END;  
  --WLAPKA
                                   
  PROCEDURE insertMissingMetrics(maxAgeInHours INTEGER, ageInMinutes INTEGER) AS

  v_metric_id INTEGER;
  v_service_id INTEGER;
  v_vo_id INTEGER;
  v_fqan_id INTEGER;
  
  CURSOR oldMetricsCurs IS select m.metric_id, m.service_id, m.vo_id, m.fqan_id from metricdata_latest m,
      metrics_in_supported_services s
    where
         m.metric_id = s.metric_id
     and m.service_id = s.service_id
     and nvl(m.vo_id, -1) = nvl(s.vo_id, -1)
     and nvl(m.fqan_id, -1) = nvl(s.fqan_id, -1)
     and m.check_time <= (select to_unixts(current_timestamp - (1/24)*maxAgeInHours - (1/1440)*ageInMinutes) from dual)
     and m.service_id in (select id from service where isdeleted = 'N')
     and m.metricstatus_id not in (select id from metricstatus where description = 'REMOVED');
  
  
  CURSOR newMissingMetricsCurs is
    select metric_id, service_id, vo_id, fqan_id from
      (select distinct p.metric_id, p.service_id, p.vo_id, p.fqan_id, m.metric_id as m_metric_id from
         metrics_in_supported_services p left join metricdata_latest m on m.metric_id = p.metric_id
           and m.service_id = p.service_id
           and nvl(m.vo_id, -1) = nvl(p.vo_id, -1)
           and nvl(m.fqan_id, -1) = nvl(p.fqan_id, -1)
      ) a, service s where a.service_id = s.id and a.m_metric_id is null and s.isdeleted = 'N';


  BEGIN
  
    OPEN oldMetricsCurs;
  
    LOOP
  
      FETCH oldMetricsCurs INTO v_metric_id, v_service_id, v_vo_id, v_fqan_id;
      EXIT WHEN oldMetricsCurs%NOTFOUND;

      insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id, vo_id, fqan_id, summarydata, check_time, insert_time)
        VALUES (MetricData_SpoolSeq.nextval, NULL, 5, NULL, v_metric_id, v_service_id, v_vo_id, v_fqan_id, NULL, to_unixts(current_timestamp - (1/1440)*ageInMinutes), to_unixts(current_timestamp));
                     
    END LOOP;
    
    CLOSE oldMetricsCurs;
    
    OPEN newMissingMetricsCurs;  

    LOOP
  
      FETCH newMissingMetricsCurs INTO v_metric_id, v_service_id, v_vo_id, v_fqan_id;
      EXIT WHEN newMissingMetricsCurs%NOTFOUND;

      insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id, vo_id, fqan_id, summarydata, check_time, insert_time)
      VALUES (MetricData_SpoolSeq.nextval ,NULL, 5, NULL, v_metric_id, v_service_id, v_vo_id, v_fqan_id, NULL, to_unixts(current_timestamp - (1/1440)*ageInMinutes), to_unixts(current_timestamp));

    END LOOP;      

    CLOSE newMissingMetricsCurs;
    
    commit;
    
  END;

  FUNCTION getServiceStatusFqan(   
    fqanId IN INTEGER,
    profileId IN INTEGER,
    serviceId IN INTEGER,
    checkTime IN INTEGER) RETURN INTEGER AS

    statusId INTEGER; 
  BEGIN
    
    select metricstatus_id into statusId from STATUSCHANGE_SERV_PROFILE_FQAN where timestamp =
      (select max(timestamp) from STATUSCHANGE_SERVICE_PROFILE
      where service_id = serviceId and profile_id = profileId
      and fqan_id = fqanId
      and timestamp <= checkTime)
      and service_id = serviceId and profile_id = profileId and fqan_id = fqanId;

    RETURN statusId;

  exception

    when NO_DATA_FOUND

    then return NULL;
   
  END;

      
  FUNCTION getServiceStatus(   
    profileId IN INTEGER,
    serviceId IN INTEGER,
    checkTime IN INTEGER) RETURN INTEGER AS

    statusId INTEGER; 
  BEGIN
    
    select metricstatus_id into statusId from STATUSCHANGE_SERVICE_PROFILE where timestamp =
      (select max(timestamp) from STATUSCHANGE_SERVICE_PROFILE
      where service_id = serviceId and profile_id = profileId
      and timestamp <= checkTime)
      and service_id = serviceId and profile_id = profileId;

    RETURN statusId;

  exception

    when NO_DATA_FOUND

    then return NULL;
   
  END;

--WLAPKA

  PROCEDURE addFqanToSuppServicesForProf(v_service_id INT, v_fqan_id INT, v_profile_id INT) AS
  
    v_id INT DEFAULT 0;
    CURSOR fqan_curs IS select id from fqans_in_supported_services 
      where service_id = v_service_id 
        and fqan_id = v_fqan_id and profile_id = v_profile_id;

  BEGIN

  OPEN fqan_curs;

  FETCH fqan_curs INTO v_id;
    IF v_id != 0 THEN
      update fqans_in_supported_services set timestamp = to_unixts(current_timestamp), updated = 1 where id = v_id;
    ELSE
      insert into fqans_in_supported_services (id, service_id, fqan_id, profile_id, timestamp, updated)
      values(fqans_in_supported_s_seq.nextval, v_service_id, v_fqan_id, v_profile_id, to_unixts(current_timestamp), 1);
    END IF;

  CLOSE fqan_curs;

END;

  PROCEDURE addFqanToSupportedServices(in_service_type_flavour VARCHAR2, in_hostname VARCHAR2, 
                 in_fqan VARCHAR2, in_profile VARCHAR2) AS
  v_fqan_id INT;
  v_serviceflavour_id INT;
  v_service_id INT;
  v_profile_id INT;
  v_rejection_reason VARCHAR2(255);
  v_service_type_flavour VARCHAR2(255);
  
  CURSOR profile_curs IS select id from profile where lower(name) in (select lower(name) from supported_profiles);
  
  BEGIN

  /*TODO: Check if service_id is supported by given profile should be added when it will be supported by MDDB and ATP*/

  select decode(in_service_type_flavour, 'sBDII', 'Site-BDII'/*, 'SRMv2', 'SRMv1'*/, in_service_type_flavour) into v_service_type_flavour from dual;
  
  v_service_type_flavour := trim(v_service_type_flavour);
  v_serviceflavour_id := getServiceFlavourIdForName(v_service_type_flavour);  
  v_service_id := getServiceIdForHostAndFlavour(in_hostname, v_serviceflavour_id);
  v_fqan_id := getOrAddFqan(in_fqan);

  OPEN profile_curs;


  LOOP

    FETCH profile_curs INTO v_profile_id;
    EXIT WHEN profile_curs%NOTFOUND;  
    
    IF (v_service_id is not NULL) AND (v_profile_id is not NULL) THEN
      addFqanToSuppServicesForProf(v_service_id, v_fqan_id, v_profile_id);
    ELSE
      IF v_service_id is NULL THEN
        v_rejection_reason := 'service_id is NULL';
      END IF; 
      IF v_profile_id is NULL THEN
        v_rejection_reason := v_rejection_reason ||  ', profile_id is NULL';
      END IF;
      insert into fqans_in_supp_serv_rejected (id, service_type_flavour, hostname, fqan, profile, timestamp, reason)
      values(fqans_in_supp_serv_r_seq.nextval, in_service_type_flavour, in_hostname, in_fqan, in_profile, to_unixts(current_timestamp), v_rejection_reason);
  
    END IF;

  END LOOP;
  commit;

  CLOSE profile_curs;

END;
    
--WLAPKA


 PROCEDURE addMetricToSupportedServices(in_service_type_flavour VARCHAR2, in_hostname VARCHAR2, in_vo VARCHAR2,
                 in_fqan VARCHAR2, in_profile VARCHAR2, in_metricname VARCHAR2, in_frequency INT) AS

  v_service_type_flavour VARCHAR2(256);
  v_metric_id INT;
  v_fqan_id INT;
  v_vo_id INT;
  v_serviceflavour_id INT;
  v_service_id INT;
  v_id INT;
  v_profile_id INT;
  v_rejection_reason VARCHAR2(255);
  CURSOR servicesCurs IS select id from metrics_in_supported_services 
    where service_id = v_service_id 
    and metric_id = v_metric_id and nvl(vo_id, -1) = nvl(v_vo_id, -1) 
    and nvl(fqan_id, -1) = nvl(v_fqan_id, -1) and profile_id = v_profile_id;

  BEGIN

  select decode(in_service_type_flavour, 'sBDII', 'Site-BDII'/*, 'SRMv2', 'SRMv1'*/, in_service_type_flavour) into v_service_type_flavour from dual;

  v_metric_id := getMetricIdForName(in_metricname);
  v_vo_id := getVoIdForName(in_vo);
  v_service_type_flavour := trim(v_service_type_flavour);
  v_serviceflavour_id := getServiceFlavourIdForName(v_service_type_flavour);
  v_service_id := getServiceIdForHostAndFlavour(in_hostname, v_serviceflavour_id);
  v_profile_id := getProfileIdForName(in_profile);                                                                                                 
  v_fqan_id := getOrAddFqan(in_fqan);

  OPEN servicesCurs;

   v_id := 0;

  FETCH servicesCurs INTO v_id;
  IF (v_metric_id is not NULL) AND (v_service_id is not NULL) AND (v_profile_id is not NULL) THEN
    IF v_id != 0 THEN
      update metrics_in_supported_services set timestamp = to_unixts(current_timestamp), frequency = in_frequency, updated = 1 where id = v_id;
    ELSE
      insert into metrics_in_supported_services (id, service_id, metric_id, vo_id, fqan_id, profile_id, frequency, timestamp, updated)
      values(metrics_in_supp_services_seq.nextval, v_service_id, v_metric_id, v_vo_id, v_fqan_id, v_profile_id, in_frequency, to_unixts(current_timestamp), 1);
    END IF;
  ELSE
    IF v_metric_id is NULL THEN
      v_rejection_reason := 'metric_id is NULL';
    END IF; 
    IF v_service_id is NULL THEN
      v_rejection_reason :=  v_rejection_reason || ', service_id is NULL';
    END IF; 
    IF v_profile_id is NULL THEN
      v_rejection_reason := v_rejection_reason ||  ', profile_id is NULL';
    END IF;
    

    insert into metrics_in_supp_serv_rejected (id, service_type_flavour, hostname, metric_name, vo, fqan, profile, frequency, timestamp, reason)
    values(metrics_in_supp_serv_rej_seq.nextval, v_service_type_flavour, in_hostname, in_metricname, in_vo, in_fqan, in_profile, in_frequency, to_unixts(current_timestamp), v_rejection_reason);
  END IF;
  commit;

  END;

--WLAPKA

PROCEDURE findCurrentStatusMetServFqan (metricId INT, serviceId INT, fqanId INT, voId INT, metricstatusId INT, check_time INT,
   old_metricstatus_id OUT INT,
   old_timestamp OUT INT,
   num_row OUT INT) AS 
  
BEGIN
  
  select count(*) INTO num_row
  from statuschange_metric_serv_fqan 
  where nvl(fqan_id, -1) = nvl(fqanId, -1)
    AND metric_id = metricId 
    AND service_id = serviceId
    AND timestamp <= check_time;

  IF num_row > 0 THEN

    SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_metric_serv_fqan 
    WHERE nvl(fqan_id, -1) = nvl(fqanId, -1)
      AND metric_id = metricId
      AND service_id = serviceId 
      AND timestamp =
        (SELECT max(timestamp) FROM statuschange_metric_serv_fqan
          WHERE nvl(fqan_id, -1) = nvl(fqanId, -1)
            AND metric_id = metricId
            AND service_id = serviceId 
            AND timestamp <= check_time);

  END IF;

END;

PROCEDURE calculateStatusChangeFqan(fqanId INT, serviceId INT, metricId INT, metricstatusId INT, check_time INT, voId INT,
                                       recalculationNeeded BOOLEAN, maxAgeOfTupleInHours INT) AS

  v_fqanId INT;
  v_profileId INT;
  done INT DEFAULT 0;
  CURSOR profileCurs IS select distinct profile_id, fqan_id from fqans_in_supported_services where service_id = serviceId and ((fqan_id = fqanId) or (fqanId is null));
                                       
BEGIN

  calculateStatusChangeMetFqan(metricId, serviceId, fqanId, voId, metricstatusId, check_time);
 
/*  select id into v_supportedVo from atp.vo where id = voId 
  and lower(voname) in (select name from supported_vos);
*/

/*  IF (v_supportedVo is not null) or (voId is NULL) THEN
*/  OPEN profileCurs;
  
    LOOP
  
      FETCH profileCurs INTO v_profileId, v_fqanId;
      EXIT WHEN profileCurs%NOTFOUND;
  
      IF checkIfCalculationNeeded(v_profileId, serviceId, metricId, voId) THEN
        IF (nvl(getServiceStatusFqan(v_fqanId, v_profileId, serviceId, check_time), -1) != metricstatusId) THEN

          calculateStatusChangeProfileFq(v_profileId, serviceId, v_fqanId, metricId, metricstatusId, check_time, voId,
                                     recalculationNeeded, maxAgeOfTupleInHours);
          BEGIN
            insert into tested_services_fqan (id, service_id, profile_id, fqan_id, last_insert_time)
            values(tested_services_fqan_seq.nextval, serviceId, v_profileId, v_fqanId, to_unixts(current_timestamp));
          
          EXCEPTION
            WHEN dup_val_on_index then
              null;    -- don't do anything, this record already exists
          END;

        END IF; 
      END IF;
        
    END LOOP;
   
    CLOSE profileCurs;
/*  END IF;*/   

END;


PROCEDURE calculateStatusChangeProfileFq(profileId INT, serviceId INT, fqanId INT, metricId INT, metricstatusId INT, 
                     check_time INT, voId INT, recalculationNeeded BOOLEAN, maxAgeOfTupleInHours INT) AS
                     
  old_metricstatus_id INTEGER; 
  old_timestamp INTEGER; 
  num_row INTEGER;
  curr_servicestatus_id INTEGER;                     

BEGIN

  num_row := 0;

--  IF (recalculationNeeded) THEN
--    call findCurrentStatusRecalculation(profileId, serviceId, metricstatusId, check_time, voId, maxAgeOfTupleInHours, curr_servicestatus_id, old_metricstatus_id, old_timestamp, num_row);
--  ELSE
  findCurrentStatusFqan(profileId, serviceId, fqanId, metricstatusId, check_time, voId, curr_servicestatus_id, old_metricstatus_id, old_timestamp, num_row); 
--  END IF;

  IF num_row > 0 THEN

    IF (metricstatusId > old_metricstatus_id) THEN
      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_serv_profile_fqan(id, fqan_id, service_id, profile_id, metricstatus_id, timestamp)
        VALUES(statuschange_serv_pr_fqan_seq.nextval, fqanId, serviceId, profileId, metricstatusId, check_time);
      ELSE
         UPDATE statuschange_serv_profile_fqan SET metricstatus_id = metricstatusId WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time and fqan_id = fqanId;
      END IF;

    ELSIF ((metricstatusId < old_metricstatus_id) AND (curr_servicestatus_id < old_metricstatus_id)) THEN

      IF old_timestamp != check_time THEN
        INSERT INTO statuschange_serv_profile_fqan(id, fqan_id, service_id, profile_id, metricstatus_id, timestamp)
        VALUES(statuschange_serv_pr_fqan_seq.nextval, fqanId, serviceId, profileId, metricstatusId, check_time);
      ELSE
         UPDATE statuschange_serv_profile_fqan SET metricstatus_id = curr_servicestatus_id WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time and fqan_id = fqanId;
      END IF;

    END IF;
  ELSE 
    INSERT INTO statuschange_serv_profile_fqan(id, fqan_id, service_id, profile_id, metricstatus_id, timestamp)
    VALUES(statuschange_serv_pr_fqan_seq.nextval, fqanId, serviceId, profileId, metricstatusId, check_time); 
  END IF;

END;



PROCEDURE findCurrentStatusFqan (profileId INT, serviceId INT, fqanId INT, metricstatusId INT, check_time INT,
  voId INT,
  curr_servicestatus_id OUT INT,
  old_metricstatus_id OUT INT,
  old_timestamp OUT INT,
  num_row OUT INT
  ) AS 

BEGIN
  
  select count(*) INTO num_row
  from statuschange_serv_profile_fqan 
  where service_id = serviceId and profile_id = profileId and fqan_id = fqanId
    and timestamp <= check_time;

  IF num_row > 0 THEN
    SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_serv_profile_fqan 
    WHERE service_id = serviceId AND profile_id = profileId AND fqan_id = fqanId
      AND timestamp =
        (SELECT max(timestamp) FROM statuschange_serv_profile_fqan
        WHERE service_id = serviceId AND profile_id = profileId 
          AND fqan_id = fqanId and timestamp <= check_time);

    IF (metricstatusId < old_metricstatus_id) THEN
        select max(metricstatus_id) INTO curr_servicestatus_id from metricdata_latest where metric_id in (
          SELECT m.metric_id FROM profile_metric_map m
            where m.profile_id = profileId 
              and (test_vo_id = voId or voId is null)
              and m.service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
          ) 
          and service_id = serviceId and (nvl(fqan_id, -1) = nvl(fqanId, -1) or fqan_id is NULL)
          and nvl(vo_id, -1) = nvl(voId, -1);
    END IF;
  
  END IF;

END;


PROCEDURE findCurrentStatusMetricService (metricId INT, serviceId INT, voId INT, metricstatusId INT, check_time INT,
  old_metricstatus_id OUT INT,
  old_timestamp OUT INT,
  num_row OUT INT) AS

BEGIN
  
  select count(*) INTO num_row
  from statuschange_metric_service 
  where nvl(vo_id, -1) = nvl(voId, -1)
    AND metric_id = metricId 
    AND service_id = serviceId
    AND timestamp <= check_time;

  IF num_row > 0 THEN

    SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
    FROM statuschange_metric_service 
    WHERE nvl(vo_id, -1) = nvl(voId, -1)
      AND metric_id = metricId
      AND service_id = serviceId 
      AND timestamp =
        (SELECT max(timestamp) FROM statuschange_metric_service
          WHERE nvl(vo_id, -1) = nvl(voId, -1)
            AND metric_id = metricId
            AND service_id = serviceId 
            AND timestamp <= check_time);

  END IF;

END;

PROCEDURE calculateStatusChangeMetFqan(metricId INT, serviceId INT, fqanId INT, voId INT, metricstatusId INT, check_time INT) AS

  num_row INTEGER;
  old_metricstatus_id INTEGER;
  old_timestamp INTEGER;

BEGIN

  num_row := 0;

  findCurrentStatusMetServFqan(metricId, serviceId, fqanId, voId, metricstatusId, check_time, old_metricstatus_id, old_timestamp, num_row); 

  IF num_row > 0 THEN
    IF ((old_timestamp != check_time) AND (metricstatusId != old_metricstatus_id)) THEN
      INSERT INTO statuschange_metric_serv_fqan(id, metric_id, service_id, fqan_id, vo_id, metricstatus_id, timestamp)
        VALUES(statuschange_metric_serv_f_seq.nextval, metricId, serviceId, fqanId, voId, metricstatusId, check_time); 
    ELSIF (metricstatusId > old_metricstatus_id) THEN
       UPDATE statuschange_metric_serv_fqan SET metricstatus_id = metricstatusId 
         WHERE service_id = serviceId AND metric_id = metricId and timestamp = check_time
           AND nvl(fqan_id, -1) = nvl(fqanId, -1);
    END IF;
  ELSE
    INSERT INTO statuschange_metric_serv_fqan(id, metric_id, service_id, fqan_id, vo_id, metricstatus_id, timestamp)
    VALUES(statuschange_metric_serv_f_seq.nextval, metricId, serviceId, fqanId, voId, metricstatusId, check_time); 
  END IF;

END;



PROCEDURE calculateStatusChangeMetric(metricId INT, serviceId INT, voId INT, metricstatusId INT, check_time INT) AS

  num_row INTEGER;
  old_metricstatus_id INTEGER;
  old_timestamp INTEGER;
    
BEGIN

  num_row := 0;
 
  findCurrentStatusMetricService(metricId, serviceId, voId, metricstatusId, check_time, old_metricstatus_id, old_timestamp, num_row); 

  IF num_row > 0 THEN
    IF ((old_timestamp != check_time) AND (metricstatusId != old_metricstatus_id)) THEN
      INSERT INTO statuschange_metric_service(id, metric_id, service_id, vo_id, metricstatus_id, timestamp)
      VALUES(statuschange_metric_s_seq.nextval, metricId, serviceId, voId, metricstatusId, check_time); 
    ELSIF (metricstatusId > old_metricstatus_id) THEN
       UPDATE statuschange_metric_service SET metricstatus_id = metricstatusId 
         WHERE service_id = serviceId AND metric_id = metricId and timestamp = check_time
           AND nvl(vo_id, -1) = nvl(voId, -1);
    END IF;
  ELSE
    INSERT INTO statuschange_metric_service(id, metric_id, service_id, vo_id, metricstatus_id, timestamp)
    VALUES(statuschange_metric_s_seq.nextval, metricId, serviceId, voId, metricstatusId, check_time); 
  END IF;

END;

PROCEDURE markRemovedMetrics AS

  v_metric_id INT;
  v_service_id INT;
  v_vo_id INT;
  v_fqan_id INT; 
  CURSOR removedMetricsCurs IS 
    select m_metric_id, service_id, vo_id, fqan_id from (
      select distinct p.metric_id as p_metric_id, m.service_id, m.vo_id, m.fqan_id, m.metric_id as m_metric_id, m.metricstatus_id from
             metricdata_latest m  left join metrics_in_supported_services p on m.metric_id = p.metric_id
               and m.service_id = p.service_id
               and nvl(m.vo_id, -1) = nvl(p.vo_id, -1)
               and nvl(m.fqan_id, -1) = nvl(p.fqan_id, -1)           
      ) a where a.p_metric_id is NULL and a.metricstatus_id != 6;

BEGIN

  OPEN removedMetricsCurs;

  LOOP
    FETCH removedMetricsCurs INTO v_metric_id, v_service_id, v_vo_id, v_fqan_id;
    EXIT WHEN removedMetricsCurs%NOTFOUND;
        
    insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, metric_id, service_id, vo_id, fqan_id, summarydata, check_time, insert_time)
      VALUES (MetricData_SpoolSeq.nextval, NULL, 6, NULL, v_metric_id, v_service_id, v_vo_id, v_fqan_id, NULL, to_unixts(current_timestamp),to_unixts(current_timestamp));

  END LOOP;

  CLOSE removedMetricsCurs;
  
END;
    
--WLAPKA

END DataLoader;
/
