CREATE OR REPLACE
PROCEDURE calcMetricFreq AS
  freqTable     freq_table := freq_table();
  past_roc   VARCHAR2(256) := '';
  past_flavour_id   NUMBER := 0;
  past_service_id   NUMBER := 0;
  past_metric_id    NUMBER := 0;
  past_check_time   NUMBER := 0;
  freq_filter       NUMBER := 0;
  now_time CONSTANT NUMBER := TO_UNIXTS(SYSDATE);

  CURSOR cur IS
  SELECT get_roc_name(service_id) AS roc, flavour_id, service_id, metric_id, check_time
    FROM metricdata
    JOIN service ON metricdata.service_id = service.id
    WHERE check_time > (now_time - 1209600) -- 2 weeks ago
      AND metricstatus_id IN (
          SELECT id FROM metricstatus
          WHERE description != 'MISSING' 
            AND description != 'REMOVED')
    ORDER BY roc, flavour_id, metric_id, service_id, check_time;

BEGIN
  DELETE FROM mon_frequency_statistics;

  FOR cur_metric IN cur
  LOOP
    IF (cur_metric.flavour_id <> past_flavour_id) OR (cur_metric.metric_id <> past_metric_id) THEN
      INSERT INTO mon_frequency_statistics 
      SELECT roc, flavour_id, metric_id, AVG(freq), STDDEV(freq) 
        FROM TABLE(freqTable)
        GROUP BY roc, flavour_id, metric_id;
      freqTable.DELETE;
    ELSE
      IF (cur_metric.service_id = cur_metric.service_id) THEN
      -- filter out all frequencies which are less than (supposed frequency / 2)
        SELECT MIN(frequency*60/2) INTO freq_filter
          FROM metrics_in_supported_services
          WHERE service_id = cur_metric.service_id
            AND metric_id = cur_metric.metric_id
            AND ROWNUM <= 1;
        IF ( (cur_metric.check_time - past_check_time) > freq_filter) THEN
          freqTable.EXTEND;
          freqTable(freqTable.last) := freq_rec(cur_metric.roc,
                                                cur_metric.flavour_id,
                                                cur_metric.metric_id,
                                                (cur_metric.check_time - past_check_time));
        END IF;
      END IF;
    END IF;
    past_roc        := cur_metric.roc;
    past_flavour_id := cur_metric.flavour_id;
    past_service_id := cur_metric.service_id;
    past_metric_id  := cur_metric.metric_id;
    past_check_time := cur_metric.check_time;
  END LOOP;    
  COMMIT;
END;
/



CREATE OR REPLACE
PROCEDURE getFreshMetrics(
	profileNameList IN VARCHAR2,
	NGINameList IN VARCHAR2,
	serviceFlavourNameList IN VARCHAR2,
	metricNameList IN VARCHAR2,
	fqanList IN VARCHAR2,
	devPercIn IN INTEGER,
	devTime IN INTEGER
) AS
  tmpStatusList      split_tbl;
  tmpProfileNameList split_tbl;
  tmpNGINameList     split_tbl;
  tmpFlavourNameList split_tbl;
  tmpMetricNameList  split_tbl;
  tmpFqanList        split_tbl;
  devPerc            INTEGER;
  check_any          NUMBER := 0;
  now_time  CONSTANT NUMBER := TO_UNIXTS(SYSDATE);
BEGIN

  devPerc := devPercIn/100 + 1;  
 
  SELECT id BULK COLLECT INTO tmpStatusList FROM metricstatus WHERE description != 'MISSING' AND description != 'REMOVED';
  
  -- split profileNameList
  SELECT COUNT(*) INTO check_any FROM TABLE(SPLIT(profileNameList)) WHERE column_value = 'ANY';
  IF (check_any) > 0 THEN
    SELECT name BULK COLLECT INTO tmpProfileNameList FROM profile;
  ELSE
    SELECT * BULK COLLECT INTO tmpProfileNameList FROM TABLE(SPLIT(profileNameList));
  END IF;
  
  -- split NGINameList
  SELECT COUNT(*) INTO check_any FROM TABLE(SPLIT(NGINameList)) WHERE column_value = 'ANY';
  IF (check_any) > 0 THEN
    SELECT DISTINCT get_roc_name(service_id) AS name BULK COLLECT INTO tmpNGINameList FROM metrics_in_supported_services;
  ELSE
    SELECT * BULK COLLECT INTO tmpNGINameList FROM TABLE(SPLIT(NGINameList));
  END IF;

  -- split serviceFlavourNameList
  SELECT COUNT(*) INTO check_any FROM TABLE(SPLIT(serviceFlavourNameList)) WHERE column_value = 'ANY';
  IF (check_any) > 0 THEN
    SELECT flavourname BULK COLLECT INTO tmpFlavourNameList FROM service_type_flavour;
  ELSE
    SELECT * BULK COLLECT INTO tmpFlavourNameList FROM TABLE(SPLIT(serviceFlavourNameList));
  END IF;

  -- split metricNameList
  SELECT COUNT(*) INTO check_any FROM TABLE(SPLIT(metricNameList)) WHERE column_value = 'ANY';
  IF (check_any) > 0 THEN
    SELECT name BULK COLLECT INTO tmpMetricNameList FROM metric;
  ELSE
    SELECT * BULK COLLECT INTO tmpMetricNameList FROM TABLE(SPLIT(metricNameList));
  END IF;  

  -- split fqanList
  SELECT COUNT(*) INTO check_any FROM TABLE(SPLIT(fqanList)) WHERE column_value = 'ANY';
  IF (check_any) > 0 THEN
    SELECT name BULK COLLECT INTO tmpFqanList FROM fqan;
    tmpFqanList.EXTEND; tmpFqanList(tmpFqanList.LAST) := 'NULL'; 
  ELSE
    SELECT * BULK COLLECT INTO tmpFqanList FROM TABLE(SPLIT(fqanList));
  END IF;

  INSERT INTO tmpJoinProfile
  SELECT name, metric_id, service_type_flavour_id 
    FROM profile 
    JOIN profile_metric_map 
      ON profile.id = profile_metric_map.profile_id
  WHERE name IN (SELECT * FROM TABLE(tmpProfileNameList));

  DELETE FROM tmpFilteredMetrics;

  INSERT INTO tmpFilteredMetrics
  SELECT get_roc_name(service.id) AS NGI,
	  service_type_flavour.flavourname AS flavour_name, 
	  metric.name AS metric_name, 
 	  NVL(fqan.name,'NULL') AS fqan_name,
 	  COUNT(*) AS counter
	  FROM metricdata_latest 
  JOIN metric ON metricdata_latest.metric_id = metric.id
  JOIN service ON metricdata_latest.service_id = service.id 
  JOIN service_type_flavour ON service.flavour_id = service_type_flavour.id
  LEFT JOIN fqan ON metricdata_latest.fqan_id = fqan.id
  WHERE metricstatus_id IN (SELECT * FROM TABLE(tmpStatusList))
    AND service_type_flavour.flavourname IN (SELECT * FROM TABLE(tmpFlavourNameList))
    AND metric.name IN (SELECT * FROM TABLE(tmpMetricNameList))
    AND metric.id IN (SELECT metric_id FROM tmpJoinProfile WHERE service_type_flavour_id = service.flavour_id)
    AND check_time > (  
             SELECT DISTINCT CASE
              WHEN  (frequency*60*devPerc < NVL( (freq_mean+2*freq_std),frequency*60*devPerc) AND
                     NVL( (freq_mean+2*freq_std),frequency*60*devPerc) < 2*frequency*60) THEN (now_time - (freq_mean+2*freq_std+devTime*60) )
              WHEN ( NVL( (freq_mean+2*freq_std),frequency*60*devPerc) <= frequency*60*devPerc) THEN (now_time - (frequency*devPerc+devTime)*60 )
              ELSE ( now_time - (2*frequency+devTime)*60 )
              END
              FROM mon_frequency_statistics
               RIGHT JOIN metrics_in_supported_services
                 ON mon_frequency_statistics.metric_id = metrics_in_supported_services.metric_id
                AND mon_frequency_statistics.roc = get_roc_name(metrics_in_supported_services.service_id)
               JOIN service atps
                 ON metrics_in_supported_services.service_id = atps.id 
              WHERE metrics_in_supported_services.metric_id = metricdata_latest.metric_id
                AND atps.flavour_id = service.flavour_id
                AND get_roc_name(metrics_in_supported_services.service_id) = get_roc_name(service.id)
                AND ROWNUM=1
          )
  GROUP BY get_roc_name(service.id), service_type_flavour.flavourname, metric.name, NVL(fqan.name,'NULL') 
  HAVING NVL(fqan.name,'NULL') IN (SELECT * FROM TABLE(tmpFqanList)); 

  DELETE FROM tmpMetricsCount;

  INSERT INTO tmpMetricsCount
  SELECT get_roc_name(service.id) AS NGI,
    service_type_flavour.flavourname AS flavour_name,
    metric.name AS metric_name,
    NVL(fqan.name,'NULL') AS fqan_name,
    COUNT(*) AS counter
    FROM metrics_in_supported_services
    JOIN service ON metrics_in_supported_services.service_id = service.id
    JOIN service_type_flavour ON service.flavour_id = service_type_flavour.id 
    JOIN metric ON metrics_in_supported_services.metric_id = metric.id
    LEFT JOIN fqan ON metrics_in_supported_services.fqan_id = fqan.id
    WHERE service_type_flavour.flavourname IN (SELECT * FROM TABLE(tmpFlavourNameList))
      AND metric.id IN (SELECT metric_id FROM tmpJoinProfile WHERE service_type_flavour_id = service.flavour_id)
      AND metric.name IN (SELECT * FROM TABLE(tmpMetricNameList))
  GROUP BY get_roc_name(service.id), service_type_flavour.flavourname, metric.name, NVL(fqan.name,'NULL')
  HAVING NVL(fqan.name,'NULL') IN (SELECT * FROM TABLE(tmpFqanList));
 
  DELETE FROM tmpFreshMetrics;

  INSERT INTO tmpFreshMetrics
   SELECT tmpMetricsCount.NGI AS NGI,
     tmpMetricsCount.flavour_name AS flavour_name,
     tmpMetricsCount.metric_name AS metric_name,
     tmpMetricsCount.fqan_name AS fqan_name,
     NVL(tmpFilteredMetrics.counter,0) AS received,
     tmpMetricsCount.counter AS total,
     NVL(CEIL((tmpFilteredMetrics.counter / tmpMetricsCount.counter)*100),0) AS percentage
   FROM tmpMetricsCount
     LEFT JOIN tmpFilteredMetrics ON tmpFilteredMetrics.NGI = tmpMetricsCount.NGI
           AND tmpFilteredMetrics.flavour_name = tmpMetricsCount.flavour_name
           AND tmpFilteredMetrics.metric_name = tmpMetricsCount.metric_name
           AND tmpFilteredMetrics.fqan_name = tmpMetricsCount.fqan_name
     WHERE tmpMetricsCount.NGI IN (SELECT * FROM TABLE(tmpNGINameList));

  COMMIT;
END;
/


