CREATE OR REPLACE PACKAGE DataLoader AS  -- specification
  PROCEDURE calculateStatusChange(
    serviceId in INTEGER, 
    metricId in INTEGER, 
    metricstatusId in INTEGER, 
    metricdataId in INTEGER, 
    check_time in INTEGER,
    voId in INTEGER);

  FUNCTION checkIfCalculationNeeded(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER)
    RETURN BOOLEAN;

  PROCEDURE calculateStatusChangeProfile(
    profileId in INTEGER,
    serviceId in INTEGER,
    metricId in INTEGER,
    metricstatusId in INTEGER,
    metricdataId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER);

  PROCEDURE loadmetricdata(in_serviceflavour in VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2, 
                   in_summarydata IN VARCHAR, in_detailsdata IN CLOB, in_voname IN VARCHAR2, 
                   in_hostname IN VARCHAR2, in_timestamp IN VARCHAR2, in_serviceURI in VARCHAR2);

  PROCEDURE insertMissingMetrics(maxAgeInHours INTEGER);

   
END DataLoader;
/

CREATE OR REPLACE PACKAGE BODY DataLoader AS 

  PROCEDURE calculateStatusChange(
    serviceId in INTEGER, 
    metricId in INTEGER, 
    metricstatusId in INTEGER,
    metricdataId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER) AS
  
    v_profileId INTEGER;
    CURSOR profileCurs is select id from profile;  
  
  BEGIN
  
    OPEN profileCurs;
  
    LOOP
  
      FETCH profileCurs INTO v_profileId;
      EXIT WHEN profileCurs%NOTFOUND;
  
      IF checkIfCalculationNeeded(v_profileId, serviceId, metricId, voId) THEN
          calculateStatusChangeProfile(v_profileId, serviceId, metricId, metricstatusId, metricdataId, check_time, voId);
          merge into tested_services t using dual on (service_id = serviceId and profile_id = v_profileId) 
            when not matched then           
              insert (id, service_id, profile_id, last_insert_time)
                values(TestedServicesSeq.nextval, serviceId, v_profileId, to_unixts(current_timestamp))
            when matched then
              update set last_insert_time = to_unixts(current_timestamp); 
      END IF;
            
    END LOOP;
   
  
    CLOSE profileCurs;
  
  END calculateStatusChange; 

  FUNCTION checkIfCalculationNeeded(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER)
  RETURN BOOLEAN 
  
  AS  
    v_counter INTEGER;
    
    CURSOR metricCurs IS SELECT count(1) FROM profile_METRIC_map 
    where profile_id = profileId and (test_vo_id = voId or voId is null) and metric_id = metricId and service_type_flavour_id =
    (select flavour_id from service where id = serviceId);
  
  BEGIN
  
    OPEN metricCurs;
    FETCH metricCurs INTO v_counter;
    CLOSE metricCurs;

    IF (v_counter > 0) THEN
      RETURN TRUE;
    ELSE 
      RETURN FALSE;
    END IF;

  END checkIfCalculationNeeded;


  PROCEDURE calculateStatusChangeProfile(
    profileId in INTEGER,
    serviceId in INTEGER,
    metricId in INTEGER,
    metricstatusId in INTEGER,
    metricdataId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER) AS
  
    old_metricstatus_id INTEGER; 
    old_timestamp INTEGER; 
    num_row INTEGER;
    curr_servicestatus_id INTEGER;

  BEGIN
  
    num_row := 0;
  
    select count(*) INTO num_row
    from statuschange_service_profile 
    where service_id = serviceId and profile_id = profileId;
  
    IF num_row > 0 THEN
      SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp
      FROM statuschange_service_profile 
      WHERE service_id = serviceId AND profile_id = profileId AND timestamp =
        (SELECT max(timestamp) FROM statuschange_service_profile
        WHERE service_id = serviceId AND profile_id = profileId and timestamp <= check_time);
  
      IF (metricstatusId < old_metricstatus_id) THEN
          select max(metricstatus_id) INTO curr_servicestatus_id from metricdata_latest_tmp where metric_id in (
            SELECT m.metric_id FROM profile_METRIC_map m
              where m.profile_id = profileId 
                and (test_vo_id = voId or voId is null)
                and m.service_type_flavour_id = (select flavour_id from service l where l.id = serviceId)
            ) 
            and service_id = serviceId;
      END IF;
  
  
      IF (metricstatusId > old_metricstatus_id) THEN
        IF old_timestamp != check_time THEN
          INSERT INTO statuschange_service_profile(id, service_id, profile_id, metricstatus_id, detail, timestamp)
          VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, 'Status changed by metricdata(1): ' || metricdataId, check_time);
        ELSE
           UPDATE statuschange_service_profile SET metricstatus_id = metricstatusId, detail = 'Status changed by metricdata(2): ' || metricdataId WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
        END IF;
  
      ELSIF ((metricstatusId < old_metricstatus_id) AND (curr_servicestatus_id < old_metricstatus_id)) THEN
  
        IF old_timestamp != check_time THEN
          INSERT INTO statuschange_service_profile(id, service_id, profile_id, metricstatus_id, detail, timestamp)
          VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, 'Status changed by metricdata(3): ' || metricdataId, check_time);
        ELSE
           UPDATE statuschange_service_profile SET metricstatus_id = curr_servicestatus_id, detail = 'Status changed by metricdata(4): ' || metricdataId WHERE service_id = serviceId AND profile_id = profileId and timestamp = check_time;
        END IF;
  
      END IF;
    ELSE 
      INSERT INTO statuschange_service_profile(id, service_id, profile_id, metricstatus_id, detail, timestamp)
      VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, 'Status changed by metricdata  : ' || metricdataId, check_time); 
    END IF;
  
  END;


  FUNCTION getVoIdForName (in_voname in VARCHAR2) RETURN INTEGER AS  
    v_vo_id INT;  
  BEGIN
    select id into v_vo_id from vo where lower(voname) = lower(in_voname);
    
    RETURN v_vo_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  FUNCTION getMetricIdForName (in_metricname in VARCHAR2) RETURN INTEGER AS  
    v_metric_id INT;  
  BEGIN
    select id into v_metric_id from metric where lower(name) = lower(in_metricname);
    
    RETURN v_metric_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  FUNCTION getMetricStatusIdForName (in_name in VARCHAR2) RETURN INTEGER AS  
    v_id INT;  
  BEGIN
    select id into v_id from metricstatus where lower(description) = lower(in_name);    
    RETURN v_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;
                               
  FUNCTION getServiceFlavourIdForName (in_name in VARCHAR2) RETURN INTEGER AS  
    v_id INT;  
  BEGIN
    select id into v_id from service_type_flavour where lower(flavourname) = lower(in_name);
    RETURN v_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;


  FUNCTION getServiceIdForHostAndFlavour (in_hostname in VARCHAR2, in_flavour in VARCHAR2) RETURN INTEGER AS  
    v_id INT;  
  BEGIN
    select id into v_id from service where lower(hostname) = lower(in_hostname) and flavour_id = in_flavour;
    RETURN v_id;
  EXCEPTION  
    when NO_DATA_FOUND
    then return NULL;    
  END;

  
  PROCEDURE loadmetricdata(in_serviceflavour in VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2, 
                   in_summarydata IN VARCHAR, in_detailsdata IN CLOB, in_voname IN VARCHAR2, 
                   in_hostname IN VARCHAR2, in_timestamp IN VARCHAR2, in_serviceURI in VARCHAR2) AS
  
    v_serviceflavour_name VARCHAR(100);
    v_vo_id INT;
    v_serviceflavour_id INT;
    v_metricstatus_id INT;
    v_metric_id INT;
    v_service_id INT;
    v_metricdetail_id INT;
    intFoundPos INT;
    strElement VARCHAR(100);
    CURSOR flavourCurs IS select decode(COLUMN_VALUE, 'sBDII', 'Site-BDII', 'SRMv2', 'SRMv1', COLUMN_VALUE) from table(split(in_serviceFlavour));
    
  BEGIN
      
    OPEN flavourCurs;
  
    LOOP 
    
      FETCH flavourCurs INTO v_serviceflavour_name;
      EXIT WHEN flavourCurs%NOTFOUND;
        v_serviceflavour_name := trim(v_serviceflavour_name);
        v_metric_id := getMetricIdForName(in_metricname);                                                    
        v_metricstatus_id := getMetricStatusIdForName(in_metricstatus);                                                            
		v_vo_id := getVoIdForName(in_voname);                                                                
        v_serviceflavour_id := getServiceFlavourIdForName(v_serviceflavour_name);
        v_service_id := getServiceIdForHostAndFlavour(in_hostname, v_serviceflavour_id);        
		
        IF v_service_id is not NULL and v_metric_id is not NULL THEN
          if in_detailsdata is not null then
            insert into metricdetails(id, detail)
            values (MetricDetailsSeq.nextval, in_detailsdata);
        
            select MetricDetailsSeq.currval into v_metricdetail_id from dual;        
          end if;
            
          DELETE FROM metricdata_latest WHERE (vo_id = v_vo_id or vo_id is NULL) and metric_id = v_metric_id and service_id = v_service_id;
      
          insert into metricdata_latest(id, metricstatus_id, metricdetail_id, metric_id, service_id, vo_id, summarydata, check_time, insert_time)
          VALUES (MetricData_LatestSeq.nextval, v_metricstatus_id, v_metricdetail_id, v_metric_id, v_service_id, v_vo_id, in_summarydata, w3ctime_to_uts(in_timestamp), to_unixts(current_timestamp));
       
        ELSIF (v_service_id is NULL and v_metric_id is NULL) THEN
          insert into metricdata_rejected(id, metricstatus, metric, serviceURI, hostname, service_flavour, vo, summarydata, check_time, insert_time, reason)
          VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_serviceURI, in_hostname, v_serviceflavour_name, in_voname, in_summarydata, w3ctime_to_uts(in_timestamp), to_unixts(current_timestamp), 'service_id and metric_id NULL');
        ELSIF v_service_id is NULL THEN
          insert into metricdata_rejected(id, metricstatus, metric, serviceURI, hostname, service_flavour, vo, summarydata, check_time, insert_time, reason)
          VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_serviceURI, in_hostname, v_serviceflavour_name, in_voname, in_summarydata, w3ctime_to_uts(in_timestamp), to_unixts(current_timestamp), 'service_id NULL');
        ELSIF v_metric_id is NULL THEN
          insert into metricdata_rejected(id, metricstatus, metric, serviceURI, hostname, service_flavour, vo, summarydata, check_time, insert_time, reason)
          VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_serviceURI, in_hostname, v_serviceflavour_name, in_voname, in_summarydata, w3ctime_to_uts(in_timestamp), to_unixts(current_timestamp), 'metric_id NULL');  
        END IF;  
    END LOOP;
  
    CLOSE flavourCurs;

  END;

  PROCEDURE insertMissingMetrics(maxAgeInHours INTEGER) AS

  v_id INTEGER;
  v_metric_id INTEGER;
  v_service_id INTEGER;
  v_vo_id INTEGER;
  
  CURSOR oldMetricsCurs IS select id, metric_id, service_id, vo_id from metricdata_latest
    where check_time <= (select to_unixts(current_timestamp - (1/24)*maxAgeInHours) from dual)
    and service_id in (select id from SERVICE where isdeleted = 'N');  

  BEGIN
  
    OPEN oldMetricsCurs;
  
    LOOP
  
      FETCH oldMetricsCurs INTO v_id, v_metric_id, v_service_id, v_vo_id;
      EXIT WHEN oldMetricsCurs%NOTFOUND;

      DELETE FROM metricdata_latest WHERE id = v_id;
      insert into metricdata_latest(id, metricstatus_id, metricdetail_id, metric_id, service_id, vo_id, summarydata, check_time, insert_time)
      VALUES (MetricData_LatestSeq.nextval, 5, NULL, v_metric_id, v_service_id, v_vo_id, NULL, to_unixts(current_timestamp), to_unixts(current_timestamp));
      
                
    END LOOP;
    
    CLOSE oldMetricsCurs;
    
    commit;
    
  END;

END DataLoader;
/


