create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H_DET"
AFTER INSERT ON METRICDATA_SPOOL
FOR EACH ROW

DECLARE
  current_hour NUMBER(2,0);
  this_flavour_id NUMBER;
  CURSOR profile_ids IS SELECT id FROM poem_sync_profile;
  has_profile BOOLEAN := false;
  prof_id NUMBER;

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  SELECT flavour_id INTO this_flavour_id FROM service WHERE id = :new.service_id;

  FOR this_profile IN profile_ids LOOP
    select p.id into prof_id from profile p, poem_sync_profile pp where pp.id = this_profile.id and p.name = pp.name;
    IF dataloader.checkifcalculationneeded(this_profile.id, :new.service_id, :new.metric_id, :new.vo_id, :new.check_time) THEN
      UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
        WHERE EXTRACT(hour FROM current_timestamp) = hour
          AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
          AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
          AND poem_sync_profile_id = this_profile.id
          and profile_id = prof_id
          AND flavour_id = this_flavour_id;

      IF SQL%rowcount = 0 THEN -- no update was actually made
        INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, poem_sync_profile_id,  flavour_id, number_of_records)
          VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), prof_id, this_profile.id, this_flavour_id, 1);
      END IF;

      has_profile := true;
    END IF;
  END LOOP;

  IF has_profile = false THEN
    UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
      WHERE EXTRACT(hour FROM current_timestamp) = hour
        AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
        AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
        AND profile_id is NULL
        and poem_sync_profile_id is null
        AND flavour_id = this_flavour_id;

    IF SQL%rowcount = 0 THEN -- no update was actually made
      INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, poem_sync_profile_id, flavour_id,  number_of_records)
        VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), NULL, null,  this_flavour_id, 1);
    END IF;
  END IF;

END;
/



create or replace
TRIGGER MIG2POEM_METRICDATA_TR 
BEFORE INSERT ON METRICDATA 
FOR EACH ROW
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATAFORREC_TR 
BEFORE INSERT ON METRICDATAFORRECALCULATION 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATALATEST_TR 
BEFORE INSERT ON METRICDATA_LATEST 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATARECREJ_TR 
BEFORE INSERT ON METRICDATAFORRECALC_REJECTED 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATASPOOL_TR 
BEFORE INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_METRICDATASPOOLREJ_TR 
BEFORE INSERT ON METRICDATA_SPOOL_REJECTED 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_STATCHANGESERVPROF_TR 
BEFORE INSERT ON STATUSCHANGE_SERVICE_PROFILE 
FOR EACH ROW 
  WHEN (new.poem_sync_profile_id is null)
    BEGIN
      select p.id into :new.poem_sync_profile_id
          from profile m, poem_sync_profile p
          where m.id=:new.profile_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_profile_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_STATCHANGEMETSERV_TR 
BEFORE INSERT ON STATUSCHANGE_METRIC_SERVICE 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



CREATE OR REPLACE TRIGGER MIG2POEM_METRICDATAREC_TR 
BEFORE INSERT ON METRICDATA_REC 
FOR EACH ROW 
  WHEN (new.poem_sync_metric_id is null)
    BEGIN
      select p.id into :new.poem_sync_metric_id
          from metric m, poem_sync_metric p
          where m.id=:new.metric_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_metric_id := null;
    END;
/



create or replace
TRIGGER MIG2POEM_MS_CURR_LOAD_H_D_TR 
BEFORE INSERT ON METRICSTORE_CURR_LOAD_H_DET 
FOR EACH ROW 
  WHEN (new.poem_sync_profile_id is null)
    BEGIN
      select p.id into :new.poem_sync_profile_id
          from profile m, poem_sync_profile p
          where m.id=:new.profile_id and m.name=p.name;
      exception
        when NO_DATA_FOUND
          then :new.poem_sync_profile_id := null;
    END;
/
