create or replace
trigger load_metricdata_fer
  after insert on metricdata for each row

DECLARE

  CURSOR metric_curs IS SELECT check_time FROM metricdata_latest
    WHERE nvl(fqan_id, -1) = nvl(:NEW.fqan_id, -1) AND nvl(vo_id, -1) = nvl(:NEW.vo_id, -1) AND metric_id = :NEW.metric_id 
      AND poem_sync_metric_id = :NEW.poem_sync_metric_id AND service_id = :NEW.service_id;

  v_check_time INTEGER;

BEGIN

  OPEN metric_curs;
  FETCH metric_curs INTO v_check_time;
  CLOSE metric_curs;

  IF ((v_check_time IS NOT NULL) AND (v_check_time <= :NEW.check_time)) THEN

    DELETE FROM metricdata_latest WHERE nvl(fqan_id, -1) = nvl(:NEW.fqan_id, -1) AND nvl(vo_id, -1) = nvl(:NEW.vo_id, -1)
      AND metric_id = :NEW.metric_id AND poem_sync_metric_id = :NEW.poem_sync_metric_id AND service_id = :NEW.service_id;
    INSERT INTO metricdata_latest(id, metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time, poem_sync_metric_id)
    VALUES (MetricData_LatestSeq.nextval, :NEW.metricstatus_id, :NEW.metricdetail_id, :NEW.metric_id, :NEW.service_id, :NEW.fqan_id, :NEW.vo_id, :NEW.summarydata, :NEW.gatheredat_id, :NEW.check_time, :NEW.insert_time, :NEW.poem_sync_metric_id);
  ELSIF (v_check_time IS NULL) THEN
    INSERT INTO metricdata_latest(id, metricstatus_id, metricdetail_id, metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time, poem_sync_metric_id)
    VALUES (MetricData_LatestSeq.nextval, :NEW.metricstatus_id, :NEW.metricdetail_id, :NEW.metric_id, :NEW.service_id, :NEW.fqan_id, :NEW.vo_id, :NEW.summarydata, :NEW.gatheredat_id, :NEW.check_time, :NEW.insert_time, :NEW.poem_sync_metric_id);
  END IF;


END;
/




DROP TRIGGER MIG2POEM_METRICDATA_TR;

DROP TRIGGER MIG2POEM_METRICDATAFORREC_TR;

DROP TRIGGER MIG2POEM_METRICDATALATEST_TR;

drop TRIGGER MIG2POEM_METRICDATARECREJ_TR;

drop TRIGGER MIG2POEM_METRICDATASPOOL_TR;

drop TRIGGER MIG2POEM_METRICDATASPOOLREJ_TR;

drop TRIGGER MIG2POEM_METRICDATAREC_TR;

drop TRIGGER MIG2POEM_STATCHANGESERVPROF_TR;

drop TRIGGER MIG2POEM_STATCHANGEMETSERV_TR;

drop TRIGGER MIG2POEM_MS_CURR_LOAD_H_D_TR;

------------------------------------------------------------
-- Begin: Bootstrapping
------------------------------------------------------------

CREATE OR REPLACE TRIGGER mrs_bootstrap_hist_ins
AFTER INSERT ON mrs_bootstrapper FOR EACH ROW
BEGIN
  insert into mrs_bootstrapper_history (service_id, metric_id, fqan_id, vo_id, timestamp, operation)
  values(:NEW.service_id, :NEW.metric_id, :NEW.fqan_id, :NEW.vo_id, :NEW.timestamp, 'insert');
END;
/

create or replace
TRIGGER mrs_bootstrap_hist_del
AFTER DELETE ON mrs_bootstrapper FOR EACH ROW
BEGIN
update  mrs_bootstrapper_history  set operation = 'delete', timestamp = current_timestamp
          where  service_id = :OLD.service_id and
          metric_id = :OLD.metric_id and
          nvl(fqan_id, -1) = nvl(:OLD.fqan_id, -1) and
          nvl(vo_id, -1) = nvl(:OLD.vo_id, -1) and
          operation like 'insert';
END;
/

create or replace
TRIGGER spool_bootstrapper_ins
AFTER INSERT ON mrs_bootstrapper FOR EACH ROW
DECLARE
   v_mddbMetric_id NUMBER := dataloader.getMddbMetricId(:NEW.metric_id) ;
   v_fqan_id NUMBER := dataloader.getMRSfqan_id(:NEW.fqan_id);  
   v_metricstatus_id NUMBER;
BEGIN
  select id into v_metricstatus_id from metricstatus where description = 'MISSING';
  
  IF v_mddbMetric_id is NULL THEN
      INSERT INTO Logger(id, proc_name, severity, message)
      values(Logger_seq.nextval, 'spool_bootstrapper_ins_TRIGGER',  'DEBUG', 'Metric ' || :NEW.metric_id  || ' skipped because it does not exist in MDDB' );
 ELSE 
   	 insert into metricdata_spool (id, service_id,  metric_id, poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
 		 values(MetricData_SpoolSeq.nextval, :NEW.service_id,  v_mddbMetric_id, :NEW.metric_id, v_metricstatus_id, v_fqan_id, :NEW.vo_id, 'MISSING', to_unixts(:NEW.timestamp), to_unixts(:NEW.timestamp));
  END IF;
END;
/

create or replace
TRIGGER spool_bootstrapper_del
AFTER DELETE ON mrs_bootstrapper FOR EACH ROW
DECLARE
  v_metricstatus_id NUMBER;
  v_mddbMetric_id NUMBER := dataloader.getMddbMetricId(:OLD.metric_id) ;
  v_fqan_id NUMBER := dataloader.getMRSfqan_id(:OLD.fqan_id);
BEGIN

  IF v_mddbMetric_id is NULL THEN
      INSERT INTO Logger(id, proc_name, severity, message)
      values(Logger_seq.nextval, 'spool_bootstrapper_ins_TRIGGER',  'DEBUG', 'Metric ' || :OLD.metric_id  || ' skipped because it does not exist in MDDB' );
 ELSE 
  select id into v_metricstatus_id from metricstatus where description = 'REMOVED';
   insert into metricdata_spool (id, service_id,  metric_id, poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
   values(MetricData_SpoolSeq.nextval, :OLD.service_id, v_mddbMetric_id,  :OLD.metric_id, v_metricstatus_id,  v_fqan_id, :OLD.vo_id, 'REMOVED', to_unixts(current_timestamp), to_unixts(current_timestamp));
  END IF;
END;
/

------------------------------------------------------------
-- End: Bootstrapping
------------------------------------------------------------



