USE mrs; 
  	
drop table supported_profiles;

DELIMITER //

DROP FUNCTION IF EXISTS getVoName;
CREATE FUNCTION getVoName (in_atp_vo_id INT) RETURNS VARCHAR(255)
  BEGIN
    DECLARE v_vo_name VARCHAR(255) default NULL;
    select voname into v_vo_name
        from vo where vo.id = in_atp_vo_id;
    RETURN v_vo_name;
END //

DELIMITER ;

create or replace view metric_history_in_poem_s_prof as
    SELECT
        pp.name profile_name,  
        pn.name namespace,
        getVoName(pp.atp_vo_id) defined_by_vo, 
        s.hostname, 
        st.serviceabbr service_type,
        t.flavourname service_flavour, 
        md.check_time metric_exec_time,
        m.name metric_name, 
        f.name fqan, 
        v.voname tested_by_vo,
        ms.description metric_status, 
        md.summarydata metric_output_summary,
        g.name gathered_at, 
        md.insert_time metric_result_insert_time
    from         
        service s, service_type st, service_type_flavour t,       
        poem_sync_metric m, metricstatus ms, gatheredat g,       
        poem_sync_profile pp, poem_sync_namespace pn,  
        poem_sync_p_m_instances ppm, poem_sync_metricinstance pm,
        vo v right outer join metricdata md on v.id = md.vo_id
        left outer join fqan f on md.fqan_id = f.id
    where  
        g.id = md.gatheredat_id
        and st.id = t.service_type_id
        and ms.id = md.metricstatus_id
        and s.id = md.service_id
        and m.id = md.poem_sync_metric_id
        and t.id = s.flavour_id
        and ppm.profile_id = pp.id
        and pm.atp_service_type_flavour_id = s.flavour_id
        and pm.metric_id = md.poem_sync_metric_id
        and (pm.atp_vo_id = md.vo_id or md.vo_id is null)
        and pm.id = ppm.metricinstance_id
        and pp.namespace_id = pn.id;
        
  CREATE TABLE IF NOT EXISTS metricdata_spool_rejected (
  id INT NOT NULL AUTO_INCREMENT,
  fqan_id INT,
  gatheredat_id INT,
  metricstatus_id INT NOT NULL,
  detailsdata text,
  service_id INT NOT NULL,
  vo_id INT,
  summarydata VARCHAR(256) NULL,
  check_time INT NOT NULL,
  insert_time INT NOT NULL,
  poem_sync_metric_id INT NOT NULL,
  PRIMARY KEY (id),
  CONSTRAINT fk_metricdata_spool_r_fqan_id
    FOREIGN KEY (fqan_id)
    REFERENCES fqan (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_gatheredat_id 
    FOREIGN KEY (gatheredat_id)
    REFERENCES gatheredat (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_nodes
    FOREIGN KEY (service_id)
    REFERENCES service (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_vos
    FOREIGN KEY (vo_id)
    REFERENCES vo (id)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_spr_metricstatus
    FOREIGN KEY (metricstatus_id)
    REFERENCES metricstatus (id )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_metricdata_sr_poem_metrics
    FOREIGN KEY (poem_sync_metric_id)
    REFERENCES poem_sync_metric(id)
) ENGINE = InnoDB ;
        
  CREATE INDEX metricdata_VoMetServ_idx ON metricdata(vo_id, poem_sync_metric_id, service_id);
  
  ALTER TABLE metricdata MODIFY service_id int NOT NULL ; 
  
  CREATE TABLE Logger (
  id INT NOT NULL AUTO_INCREMENT,
  inserttime  timestamp DEFAULT CURRENT_TIMESTAMP,
  proc_name VARCHAR(256),
  severity VARCHAR(50),
  message VARCHAR(1024),
  CONSTRAINT logger_pk PRIMARY KEY (id)
) ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS event_logger (
  id INT NOT NULL AUTO_INCREMENT,
  severity VARCHAR(45) NOT NULL,
  name VARCHAR(45) NOT NULL,
  message TEXT NOT NULL,
  inserttime  timestamp DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (id)
) ENGINE = InnoDB;

delimiter //

drop procedure if exists logger;
create PROCEDURE logger(in_procname VARCHAR(256), in_severity VARCHAR(50), in_message VARCHAR(1024)) 

BEGIN

  INSERT INTO Logger(proc_name, severity, message)
    values(in_procname, in_severity, in_message);

END //

delimiter ;






