USE mrs;

create or replace view metric_history_in_poem_s_prof as
   SELECT
       pp.name profile_name,
       pn.name namespace,
       getVoName(pp.atp_vo_id) defined_by_vo,
       s.hostname,
       st.serviceabbr service_type,
       t.flavourname service_flavour,
       md.check_time metric_exec_time,
       m.name metric_name,
       f.name fqan,
       v.voname tested_by_vo,
       ms.description metric_status,
       md.summarydata metric_output_summary,
       g.name gathered_at,
       md.insert_time metric_result_insert_time
   from
       service s, service_type st, service_type_flavour t,
       poem_sync_metric m, metricstatus ms,
       poem_sync_profile pp, poem_sync_namespace pn,
       poem_sync_p_m_instances ppm, poem_sync_metricinstance pm,
       vo v right outer join metricdata md on v.id = md.vo_id
       left outer join fqan f on md.fqan_id = f.id
       left outer join gatheredat g on g.id = md.gatheredat_id
   where
       st.id = t.service_type_id
       and ms.id = md.metricstatus_id
       and s.id = md.service_id
       and m.id = md.poem_sync_metric_id
       and t.id = s.flavour_id
       and ppm.profile_id = pp.id
       and pm.atp_service_type_flavour_id = s.flavour_id
       and pm.metric_id = md.poem_sync_metric_id
       and (pm.atp_vo_id = md.vo_id or md.vo_id is null)
       and pm.id = ppm.metricinstance_id
       and pp.namespace_id = pn.id;



alter table mrs_bootstrapper drop foreign key bootstrapper_fk3;
alter table mrs_bootstrapper change column fqan_id poem_sync_fqan_id INT;
alter table mrs_bootstrapper add fqan_id INT;


alter table mrs_bootstrapper_history drop foreign key bootstrapper_h_fk3;
alter table mrs_bootstrapper_history change column fqan_id poem_sync_fqan_id INT;
alter table mrs_bootstrapper_history add fqan_id INT;


delimiter //
create PROCEDURE migBootstrapper (num_records INT)
BEGIN
  DECLARE counter INT;
  DECLARE v_poem_sync_fqan_id INT;
  DECLARE v_fqan_name VARCHAR(256);
  DECLARE done INT DEFAULT 0;

 DECLARE tuple CURSOR FOR
    select distinct(m.poem_sync_fqan_id), p.fqan
        from mrs_bootstrapper m, poem_sync_fqan p
        where m.poem_sync_fqan_id is not null and m.poem_sync_fqan_id=p.id and m.fqan_id is null ;

 DECLARE tuple_history CURSOR FOR
    select distinct(m.poem_sync_fqan_id), p.fqan
        from mrs_bootstrapper_history m, poem_sync_fqan p
        where m.poem_sync_fqan_id is not null and m.poem_sync_fqan_id=p.id and m.fqan_id is null ;

  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

  set autocommit = 0;

 OPEN tuple;
  set counter = 0;

  REPEAT
    set done = 0;
    FETCH tuple INTO v_poem_sync_fqan_id, v_fqan_name;

    IF NOT done THEN
      UPDATE mrs_bootstrapper set fqan_id = getOrAddFqan(v_fqan_name)
        where poem_sync_fqan_id = v_poem_sync_fqan_id;
      set counter = counter + 1;
      IF counter = num_records THEN
        commit;
        set counter = 0;
      END IF;
      set done = 0;
    END IF;
  UNTIL done END REPEAT;
  
 CLOSE tuple;

 commit;

 OPEN tuple_history;
  set counter = 0;

  REPEAT
    set done = 0;
    FETCH tuple_history INTO v_poem_sync_fqan_id, v_fqan_name;

    IF NOT done THEN
      UPDATE mrs_bootstrapper_history set fqan_id = getOrAddFqan(v_fqan_name)
        where poem_sync_fqan_id = v_poem_sync_fqan_id;
      set counter = counter + 1;
      IF counter = num_records THEN
        commit;
        set counter = 0;
      END IF;
      set done = 0;
    END IF;
  UNTIL done END REPEAT;

CLOSE tuple_history;

commit;

END //

delimiter ;

call migBootstrapper(1000);


drop procedure if exists migBootstrapper;


alter table mrs_bootstrapper drop column poem_sync_fqan_id;
alter table mrs_bootstrapper add constraint  bootstrapper_fk3
  foreign key (fqan_id)
  references fqan (id);


alter table mrs_bootstrapper_history drop column poem_sync_fqan_id;
alter table mrs_bootstrapper_history add constraint  bootstrapper_h_fk3
  foreign key (fqan_id)
  references fqan (id);