DELIMITER //

USE `mrs`;

drop trigger if exists bootstrapper_ins;
create TRIGGER bootstrapper_ins
AFTER INSERT ON mrs_bootstrapper FOR EACH ROW

BEGIN

  insert into mrs_bootstrapper_history (service_id, metric_id, fqan_id, vo_id, timestamp, operation)
  values(NEW.service_id, NEW.metric_id, NEW.fqan_id, NEW.vo_id, NEW.timestamp, 'insert');
  call  logger('bootstrapper_ins', 'DEBUG', 'New entry in MRS bootstrapper (metric_id ' ||  NEW.metric_id || ' service_id ' || NEW.service_id || ' fqan_id ' || ifnull(NEW.fqan_id, -1) || ' vo_id '  || ifnull(NEW.vo_id, -1) || ')');
  

 insert into metricdata_spool (service_id,  poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
  values(NEW.service_id,  NEW.metric_id, (select id from metricstatus where description = 'MISSING'), NEW.fqan_id, NEW.vo_id, 'MISSING', unix_timestamp(NEW.timestamp), unix_timestamp(NEW.timestamp));
  
END //


drop trigger if exists bootstrapper_del;
create TRIGGER bootstrapper_del
AFTER DELETE ON mrs_bootstrapper FOR EACH ROW

BEGIN

  update  mrs_bootstrapper_history  set operation = 'delete', timestamp = current_timestamp
          where operation = 'insert' and
          service_id = OLD.service_id and
          metric_id = OLD.metric_id and
          ifnull(fqan_id, 'null') = ifnull(OLD.fqan_id, 'null') and
          ifnull(vo_id, 'null') = ifnull(OLD.vo_id, 'null');
  call logger('bootstrapper_del', 'DEBUG', 'Entry deleted in MRS bootstrapper (metric_id ' ||  OLD.metric_id || ' service_id ' || OLD.service_id || ' fqan_id ' || ifnull(OLD.fqan_id, -1) || ' vo_id '  || ifnull(OLD.vo_id, -1) || ')');


 insert into metricdata_spool (service_id,  poem_sync_metric_id, metricstatus_id, fqan_id, vo_id, summarydata, check_time, insert_time)
   values(OLD.service_id,  OLD.metric_id, (select id from metricstatus where description = 'REMOVED'),  OLD.fqan_id, OLD.vo_id, 'REMOVED', unix_timestamp(), unix_timestamp());
   
END //

DELIMITER ;
