create or replace
PACKAGE DataLoader AS  -- specification

  /*PROCEDURE addEntryToProfileHierarchy(
    in_profile in VARCHAR2,
    in_parent_profile in VARCHAR2
  );*/

  PROCEDURE calculateStatusChange(
    serviceId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN,
    maxAgeOfTupleInHours in INTEGER,
    poem_sync_metric_id in INTEGER,
    fqanName in VARCHAR2);


  FUNCTION checkIfMetricDefinedInProfile(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER, check_time INTEGER, fqanName VARCHAR2)
    RETURN BOOLEAN;
    
FUNCTION checkIfMetricDefinedInProfile(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER, check_time INTEGER)
    RETURN BOOLEAN;

  PROCEDURE calculateStatusChangeMetric(serviceId INT, voId INT, metricstatusId INT, check_time INT, poem_sync_metric_id INT);

  PROCEDURE calculateStatusChangeProfile(
    profileId in INTEGER,
    serviceId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN,
    maxAgeOfTupleInHours in INTEGER,
    poem_sync_metric_id in INTEGER);

  PROCEDURE findMetricsForStatusRecalc (in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER);

  PROCEDURE loadmetricdatafromspool(maxAgeInHours IN INT, ageInMinutes IN INT, startThreshold IN INT,
  				maxMinsLatestEntry IN INT);

  PROCEDURE loadmetricdatatospool (in_serviceflavour IN VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2,
                 in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2, in_fqan IN VARCHAR2,
                 in_hostname IN VARCHAR2, in_timestamp IN VARCHAR2, in_gatheredat IN VARCHAR2, metric_rejected_age_days IN INT);


  PROCEDURE loadmetricdatatospool_OSG (in_serviceuri IN VARCHAR2, in_servicetype IN VARCHAR2, in_metricname IN VARCHAR2,
  				 in_metricstatus IN VARCHAR2, in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2,
  				 in_timestamp IN VARCHAR2, in_gatheredat IN VARCHAR2, metric_rejected_age_days IN INT);

  PROCEDURE insertRowToMetricData(in_service_id IN INT, in_poem_metric_id IN INT, in_metricstatus_id IN INT, in_summarydata IN VARCHAR,
                   in_detailsdata IN CLOB, in_fqan_id IN INT, in_vo_id IN INT, in_gatheredat_id IN INT, in_check_time IN INT, in_insert_time IN INT,
                   in_recalculationNeeded IN BOOLEAN, in_maxAgeOfTupleInHours IN INT);

  PROCEDURE loadMetricDataWithRecalc(in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER);

  PROCEDURE insertMissingMetrics(maxAgeInHours INTEGER, ageInMinutes INTEGER);


  FUNCTION getServiceStatus(
    profileId IN INTEGER,
    serviceId IN INTEGER,
    checkTime IN INTEGER)
  RETURN INTEGER;

PROCEDURE purgeMetricstore (
    logger_kept IN INT,             
    mdata_rej_kept IN INT,          
    mdata_spool_rej_kept IN INT,    
    mdata_latest_kept IN INT,      
    mstore_clhd IN INT );

FUNCTION getMRSfqan_id (in_poem_sync_fqan_id in INTEGER) RETURN INTEGER;

END DataLoader;
/

create or replace
PACKAGE BODY DataLoader AS

  PROCEDURE logger(in_procname in VARCHAR2, in_severity IN VARCHAR2, in_message IN VARCHAR2) AS

  BEGIN

    INSERT INTO Logger(id, proc_name, severity, message)
      values(Logger_seq.nextval, in_procname, in_severity, in_message);


  END;

  FUNCTION getOrAddFqan(in_name VARCHAR2) RETURN INT
  AS

    v_id INT DEFAULT NULL;
  BEGIN

    v_id := NULL;

    IF in_name is not NULL THEN
      BEGIN
        select id into v_id from fqan where lower(name) = lower(in_name);

        EXCEPTION when NO_DATA_FOUND THEN
          insert into fqan(id, name)
            values(fqan_seq.nextval, in_name);

        select id into v_id from fqan where lower(name) = lower(in_name);

      END;
    END IF;

    RETURN v_id;

  END;

  FUNCTION getOrAddGatheredAt(in_gatheredat IN VARCHAR2) RETURN INT
  AS
    v_id INT DEFAULT NULL;
  BEGIN

    IF in_gatheredat is not NULL THEN
      BEGIN
        select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);

        EXCEPTION when NO_DATA_FOUND THEN
          insert into gatheredat(id, name)
             values(gatheredat_seq.nextval, in_gatheredat);

         select id into v_id from gatheredat where lower(name) = lower(in_gatheredat);

      END;
    END IF;

    RETURN v_id;
  END;

  FUNCTION getVoIdForName (in_voname in VARCHAR2) RETURN INTEGER AS
    v_vo_id INT;
  BEGIN
    select id into v_vo_id from vo where lower(voname) = lower(in_voname);

    RETURN v_vo_id;
  EXCEPTION
    when NO_DATA_FOUND
    then return NULL;
  END;

 FUNCTION getPoemSyncMetricIdForName (in_metricname in VARCHAR2) RETURN INTEGER AS
    v_metric_id INT;
  BEGIN
    select id into v_metric_id from poem_sync_metric where lower(name) = lower(in_metricname);

    RETURN v_metric_id;
  EXCEPTION
    when NO_DATA_FOUND
    then return NULL;
  END;

 FUNCTION getMetricStatusWeightForId (in_id in INTEGER) RETURN INTEGER AS
    v_weight INT;
  BEGIN
    select weight into v_weight from metricstatus where id = in_id;
    RETURN v_weight;
  END;

  FUNCTION getMetricStatusIdForName (in_name in VARCHAR2) RETURN INTEGER AS
    v_id INT;
  BEGIN
    select id into v_id from metricstatus where lower(description) = lower(in_name);
    RETURN v_id;
  EXCEPTION
    when NO_DATA_FOUND
    then return NULL;
  END;

  FUNCTION getServiceFlavourIdForName (in_name in VARCHAR2) RETURN INTEGER AS
    v_id INT;
  BEGIN
    select id into v_id from service_type_flavour where lower(flavourname) = lower(in_name);
    RETURN v_id;
  EXCEPTION
    when NO_DATA_FOUND
    then return NULL;
  END;

  FUNCTION getServiceIdForHostAndFlavour (in_hostname in VARCHAR2, in_flavour in VARCHAR2) RETURN INTEGER AS
    v_id INT;
  BEGIN
    select id into v_id from service where lower(hostname) = lower(in_hostname) and flavour_id = in_flavour;
    RETURN v_id;
  EXCEPTION
    when NO_DATA_FOUND
    then return NULL;
  END;

  /*PROCEDURE addEntryToProfileHierarchy(
    in_profile in VARCHAR2,
    in_parent_profile in VARCHAR2
  ) AS

    v_profile_id INTEGER;
    v_parent_profile_id INTEGER;
  BEGIN
    v_profile_id := getProfileIdForName(in_profile);
    v_parent_profile_id := getProfileIdForName(in_parent_profile);

    BEGIN
      insert into profile_hierarchy(id, profile_id, parent_profile_id, timestamp)
      values(profile_hierarchy_seq.nextval, v_profile_id, v_parent_profile_id,  to_unixts(current_timestamp));
    EXCEPTION
      WHEN dup_val_on_index then
        null;    -- don't do anything, this record already exists
    END;
  END;*/

  PROCEDURE calculateStatusChange(
    serviceId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN,
    maxAgeOfTupleInHours in INTEGER,
    poem_sync_metric_id in INTEGER,
    fqanName in VARCHAR2) AS

    v_profileId INTEGER;
    CURSOR profileCurs is select id from poem_sync_profile;

  BEGIN

    calculateStatusChangeMetric(serviceId, voId, metricstatusId, check_time, poem_sync_metric_id);

    OPEN profileCurs;

    LOOP

      FETCH profileCurs INTO v_profileId;
      EXIT WHEN profileCurs%NOTFOUND;
            
      IF checkIfMetricDefinedInProfile(v_profileId, serviceId, poem_sync_metric_id, voId, check_time, fqanName) THEN
        IF (nvl(getServiceStatus(v_profileId, serviceId, check_time), -1) != metricstatusId) THEN
          calculateStatusChangeProfile(v_profileId, serviceId, metricstatusId, check_time, voId,
                                       recalculationNeeded, maxAgeOfTupleInHours, poem_sync_metric_id);

        END IF;
      END IF;

    END LOOP;


    CLOSE profileCurs;

  END calculateStatusChange;

FUNCTION checkIfMetricDefinedInProfile(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER, check_time INTEGER, fqanName VARCHAR2)
  RETURN BOOLEAN AS
    v_counter integer;

    CURSOR metricCurs is select count(1) from poem_sync_p_m_instances a,
              poem_sync_metricinstance b where
                    a.profile_id = profileId and
                    a.metricinstance_id=b.id and
                    b.metric_id = metricId and
                    (nvl(b.atp_vo_id,-1) = nvl(voId,-1) or b.atp_vo_id is null) and
                    b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId) and
                   (b.fqan_id is null or b.fqan_id in (select id from poem_sync_fqan where fqan=fqanName));

  BEGIN

    OPEN metricCurs;
    FETCH metricCurs INTO v_counter;
    CLOSE metricCurs;

     IF (v_counter > 0) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;

  END checkIfMetricDefinedInProfile;
  
    FUNCTION checkIfMetricDefinedInProfile(profileId INTEGER, serviceId INTEGER, metricId INTEGER, voId INTEGER, check_time INTEGER)
  RETURN BOOLEAN AS
    v_counter integer;

    CURSOR metricCurs is select count(1) from poem_sync_p_m_instances a,
              poem_sync_metricinstance b where
                    a.profile_id = profileId and
                    a.metricinstance_id=b.id and
                    b.metric_id = metricId and
                    (nvl(b.atp_vo_id,-1) = nvl(voId,-1) or b.atp_vo_id is null) and
                    b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = serviceId);

  BEGIN

    OPEN metricCurs;
    FETCH metricCurs INTO v_counter;
    CLOSE metricCurs;

     IF (v_counter > 0) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;

  END checkIfMetricDefinedInProfile;

  FUNCTION findCurrentStatusRecalculation(in_profileId in INTEGER,
    in_serviceId in INTEGER,
    in_metricstatusId in INTEGER,
    in_check_time in INTEGER,
    in_voId in INTEGER,
    in_maxAgeOfTupleInHours in INTEGER,
    new_servicestatus_id out INTEGER,
    old_servicestatus_id out INTEGER,
    old_timestamp_st_change out INTEGER
  ) RETURN INTEGER AS


    num_row INTEGER;
    curr_weight INTEGER;
  BEGIN

    num_row := 0;

    select count(*) INTO num_row
    from statuschange_service_profile
    where service_id = in_serviceId and poem_sync_profile_id = in_profileId and timestamp <= in_check_time;

    IF num_row > 0 THEN
      SELECT metricstatus_id, timestamp INTO old_servicestatus_id, old_timestamp_st_change
      FROM statuschange_service_profile
      WHERE service_id = in_serviceId AND poem_sync_profile_id = IN_profileId AND timestamp =
        (SELECT max(timestamp) FROM statuschange_service_profile
        WHERE service_id = in_serviceId AND poem_sync_profile_id = in_profileId and timestamp <= in_check_time);

      IF (getMetricStatusWeightForId(in_metricstatusId) != getMetricStatusWeightForId(old_servicestatus_id)) THEN
        select max(s.weight) INTO curr_weight from metricdata m, metricstatus s where
          m.metricstatus_id = s.id
          and m.service_id = in_serviceId
          and (nvl(m.vo_id,-1) = nvl(in_voId,-1) or m.vo_id is null)
          and (m.poem_sync_metric_id, m.check_time) in (
            select poem_sync_metric_id, max(check_time)  from metricdata where poem_sync_metric_id in (
              select b.metric_id from poem_sync_p_m_instances a, poem_sync_metricinstance b
            	where a.profile_id=in_profileId
            	and a.metricinstance_id=b.id
            	and (nvl(b.atp_vo_id,-1) =nvl( in_voId,-1) or b.atp_vo_id is null)
            	and b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = in_serviceId)
              )
            and (vo_id is null or vo_id in (
              select b.atp_vo_id from poem_sync_p_m_instances a, poem_sync_metricinstance b
            	where a.profile_id=in_profileId
            	and a.metricinstance_id=b.id
            	and (nvl(b.atp_vo_id,-1) = nvl(in_voId,-1) or b.atp_vo_id is null)
            	and b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = in_serviceId)
              ))
            and service_id = in_serviceId
            and check_time >= in_check_time - (in_maxAgeOfTupleInHours+1) * 3600
            and check_time <= in_check_time
            group by poem_sync_metric_id
          );
        select id into new_servicestatus_id from metricstatus where weight = curr_weight;
      END IF;

      return num_row;
    ELSE
      return 0;
    END IF;

  END;

  FUNCTION findCurrentStatus (in_profileId in INTEGER,
    in_serviceId in INTEGER,
    metricstatusId in INTEGER,
    in_check_time in INTEGER,
    in_voId in INTEGER,
    new_servicestatus_id out INTEGER,
    old_servicestatus_id out INTEGER,
    old_timestamp_st_change out INTEGER
    ) RETURN INTEGER AS

    num_row INTEGER;
    curr_weight INTEGER;
  BEGIN
    num_row := 0;

    select count(*) INTO num_row
    from statuschange_service_profile
    where service_id = in_serviceId and poem_sync_profile_id = in_profileId;

    IF num_row > 0 THEN
      SELECT metricstatus_id, timestamp INTO old_servicestatus_id, old_timestamp_st_change
      FROM statuschange_service_profile
      WHERE service_id = in_serviceId AND poem_sync_profile_id = in_profileId AND timestamp =
        (SELECT max(timestamp) FROM statuschange_service_profile
        WHERE service_id = in_serviceId AND poem_sync_profile_id = in_profileId and timestamp <= in_check_time);

      IF (getMetricStatusWeightForId(metricstatusId) != getMetricStatusWeightForId(old_servicestatus_id)) THEN
        select max(s.weight) INTO curr_weight from metricdata_latest m, metricstatus s where
          m.metricstatus_id = s.id
          and m.service_id = in_serviceId
          and (m.vo_id = in_voId or in_voId is null or m.vo_id is null)
          and (m.poem_sync_metric_id, m.check_time) in (
            select poem_sync_metric_id, max(check_time)  from metricdata_latest where poem_sync_metric_id in (
              select b.metric_id from poem_sync_p_m_instances a, poem_sync_metricinstance b
                where a.profile_id=in_profileId
                and a.metricinstance_id=b.id
                and (b.atp_vo_id = in_voId or in_voId is null)
                and b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = in_serviceId)
              )
            and (vo_id is null or vo_id in (
              select b.atp_vo_id from poem_sync_p_m_instances a, poem_sync_metricinstance b
                where a.profile_id=in_profileId
                and a.metricinstance_id=b.id
                and (b.atp_vo_id = in_voId or in_voId is null)
                and b.atp_service_type_flavour_id = (select flavour_id from service l where l.id = in_serviceId)
              ))
            and service_id = in_serviceId
            group by poem_sync_metric_id
          );

         select id into new_servicestatus_id from metricstatus where weight = curr_weight;
      END IF;

      return num_row;
    ELSE
      return 0;
    END IF;
  END;


  PROCEDURE calculateStatusChangeProfile(
    profileId in INTEGER,
    serviceId in INTEGER,
    metricstatusId in INTEGER,
    check_time in INTEGER,
    voId in INTEGER,
    recalculationNeeded in BOOLEAN,
    maxAgeOfTupleInHours in INTEGER,
    poem_sync_metric_id in INTEGER) AS

    old_servicestatus_id INTEGER;
    old_timestamp_st_change INTEGER;
    num_row INTEGER;
    new_servicestatus_id INTEGER;
    wasStatusChange BOOLEAN;
  BEGIN
    wasStatusChange := true;

    IF (recalculationNeeded) THEN
      num_row := findCurrentStatusRecalculation(profileId, serviceId, metricstatusId, check_time, voId, maxAgeOfTupleInHours, new_servicestatus_id, old_servicestatus_id, old_timestamp_st_change);
    ELSE
      num_row := findCurrentStatus(profileId, serviceId, metricstatusId, check_time, voId, new_servicestatus_id, old_servicestatus_id, old_timestamp_st_change);
    END IF;

    IF num_row > 0 THEN

      IF (getMetricStatusWeightForId(new_servicestatus_id) > getMetricStatusWeightForId(old_servicestatus_id)) THEN
        IF old_timestamp_st_change != check_time THEN
          INSERT INTO statuschange_service_profile(id, service_id, poem_sync_profile_id, metricstatus_id, detail, timestamp, insert_time)
          VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, new_servicestatus_id, NULL, check_time, to_unixts(current_timestamp));
        ELSE
           UPDATE statuschange_service_profile SET metricstatus_id = new_servicestatus_id, insert_time = to_unixts(current_timestamp) WHERE service_id = serviceId AND poem_sync_profile_id = profileId and timestamp = check_time;
        END IF;

      ELSIF ((getMetricStatusWeightForId(metricstatusId) < getMetricStatusWeightForId(old_servicestatus_id)) AND
             (getMetricStatusWeightForId(new_servicestatus_id) < getMetricStatusWeightForId(old_servicestatus_id))) THEN

        IF old_timestamp_st_change != check_time THEN
          INSERT INTO statuschange_service_profile(id, service_id, poem_sync_profile_id, metricstatus_id, detail, timestamp, insert_time)
          VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, new_servicestatus_id, NULL, check_time, to_unixts(current_timestamp));
        ELSE
           UPDATE statuschange_service_profile SET metricstatus_id = new_servicestatus_id, insert_time = to_unixts(current_timestamp) WHERE service_id = serviceId AND poem_sync_profile_id = profileId and timestamp = check_time;
        END IF;

      ELSE
        wasStatusChange := false;
      END IF;
    ELSE
      INSERT INTO statuschange_service_profile(id, service_id, poem_sync_profile_id, metricstatus_id, detail, timestamp, insert_time)
      VALUES(StatusChange_ServiceProfileSeq.nextval, serviceId, profileId, metricstatusId, NULL, check_time, to_unixts(current_timestamp));
    END IF;

    IF (wasStatusChange AND recalculationNeeded) THEN
      delete from statuschange_service_profile
      where service_id = serviceId and poem_sync_profile_id = profileId and timestamp > check_time;

    END IF;

  END;

  PROCEDURE findMetricsForStatusRecalc (in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER) AS

  BEGIN

    insert into MetricDataForRecalculation(id, metricdata_spool_id, gatheredat_id, metricstatus_id, detailsdata,
                            service_id, fqan_id, vo_id, summarydata, check_time, insert_time, poem_sync_metric_id)
      select METRICDATAFORRECALCULATIONSEQ.nextval, id, gatheredat_id, metricstatus_id, detailsdata,
                            service_id, fqan_id, vo_id, summarydata, check_time, insert_time, poem_sync_metric_id from metricdata_spool
        where check_time < in_minCheckTime;

    delete from metricdata_spool where id in (select metricdata_spool_id from MetricDataForRecalculation);

    insert into MetricDataForRecalculation(id, metricdata_spool_id, gatheredat_id, metricstatus_id, detailsdata,
                            service_id, fqan_id, vo_id, summarydata, check_time, insert_time, poem_sync_metric_id)
      select  METRICDATAFORRECALCULATIONSEQ.nextval, id, gatheredat_id, metricstatus_id, detailsdata,
                            service_id, fqan_id, vo_id, summarydata, check_time, insert_time, poem_sync_metric_id
        from metricdata_spool
          where service_id in (
            select distinct service_id from MetricDataForRecalculation
          );

    delete from metricdata_spool where id in (select MetricData_Spool_id from MetricDataForRecalculation);

    commit;

  END;

  PROCEDURE loadMetricDataWithRecalc (in_maxCheckTime in INTEGER, in_minCheckTime in INTEGER) AS

    CURSOR metricResultsCurs IS select id, gatheredat_id, metricstatus_id,
                            service_id, fqan_id, vo_id, summarydata, check_time, insert_time, poem_sync_metric_id from MetricDataForRecalculation
                            order by check_time;

    metricResultRec metricResultsCurs%ROWTYPE;
    v_detailsdata CLOB;
    v_iLoop INTEGER;

  BEGIN

    v_iLoop := 0;
    findMetricsForStatusRecalc(in_maxCheckTime, in_minCheckTime);

    OPEN metricResultsCurs;

    LOOP

      FETCH metricResultsCurs INTO metricResultRec;
      EXIT WHEN metricResultsCurs%NOTFOUND;

      BEGIN


        logger('loadMetricDataWithRecalc', 'DEBUG', 'Before insertRowToMetricData');

        select detailsdata into v_detailsdata from MetricDataForRecalculation where id = metricResultRec.id;

        insertRowToMetricData(metricResultRec.service_id, metricResultRec.poem_sync_metric_id, metricResultRec.metricstatus_id, metricResultRec.summarydata,
                   v_detailsdata, metricResultRec.fqan_id, metricResultRec.vo_id, metricResultRec.gatheredat_id, metricResultRec.check_time,
                   metricResultRec.insert_time, true, 24);

        logger('loadMetricDataWithRecalc', 'DEBUG', 'After insertRowToMetricData');

        exception
          when DUP_VAL_ON_INDEX then
            dbms_output.put_line('Row already exists: ' || metricResultRec.id);
            logger('loadMetricDataWithRecalc', 'ERROR', 'Row already exists: ' || metricResultRec.id);
          when NO_DATA_FOUND then
            insert into MetricDataForRecalc_rejected select * from MetricDataForRecalculation where id = metricResultRec.id;
            logger('loadMetricDataWithRecalc', 'ERROR', 'NO_DATA_FOUND');

      END;

      delete from MetricDataForRecalculation where id = metricResultRec.id;

      v_iLoop := v_iLoop + 1;


      IF v_iLoop = 1000 THEN
        commit;
        logger('loadMetricDataWithRecalc', 'DEBUG', 'After commit of ' || v_iloop || ' records.');
        v_iLoop := 0;
      END IF;

    END LOOP;

    commit;
    logger('loadMetricDataWithRecalc', 'DEBUG', 'After commit of ' || v_iloop || ' records.');

  END loadMetricDataWithRecalc;

  PROCEDURE loadmetricdatatospool_OSG (in_serviceuri IN VARCHAR2, in_servicetype IN VARCHAR2, in_metricname IN VARCHAR2,
  				 in_metricstatus IN VARCHAR2, in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2,
  				 in_timestamp IN VARCHAR2, in_gatheredat IN VARCHAR2, metric_rejected_age_days IN INT) AS

    v_gatheredat_id INT;
    v_metricstatus_id INT;
    v_poem_metric_id INT;
    v_fqan_id INT;
    v_vo_id INT;
    v_service_id INT;
    v_metricdetail_id INT;
    v_check_time INT;
    v_insert_time INT;
    v_metric_age_days INT;
    v_servicetype VARCHAR2(256);
    v_serviceflavour_id INT;

  BEGIN

      if lower(in_servicetype) = 'ce' then
        v_servicetype := 'OSG-CE';
      elsif lower(in_servicetype) = 'gridftp' then
        v_servicetype := 'OSG-GridFtp';
      elsif lower(in_servicetype) = 'srmv2' then
        v_servicetype := 'OSG-SRMv2';
      end if;


      v_poem_metric_id := getPoemSyncMetricIdForName(in_metricname);
      v_serviceflavour_id := getServiceFlavourIdForName(v_servicetype);
      v_service_id := get_service_id(in_serviceuri, v_serviceflavour_id);

      v_metricstatus_id := getMetricStatusIdForName(in_metricstatus);
      v_vo_id := getVoIdForName(in_voname);


      v_gatheredat_id := getOrAddGatheredAt(in_gatheredat);
      v_check_time := w3ctime_to_uts(in_timestamp);
      v_insert_time := to_unixts(current_timestamp);
      v_metric_age_days := floor((v_insert_time - v_check_time) / 86400);

      IF v_service_id is not NULL and v_poem_metric_id is not NULL AND v_metric_age_days < metric_rejected_age_days THEN
        insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, service_id, vo_id, fqan_id, summarydata, check_time, insert_time, poem_sync_metric_id)
          VALUES (MetricData_SpoolSeq.nextval, v_gatheredat_id, v_metricstatus_id, in_detailsdata, v_service_id, v_vo_id, NULL, in_summarydata, v_check_time, v_insert_time, v_poem_metric_id);
      ELSIF (v_metric_age_days >= metric_rejected_age_days) THEN
        insert into metricdata_rejected_osg(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedOsgSeq.nextval, in_metricstatus, in_metricname, in_serviceuri, v_servicetype, in_voname, NULL, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'metric too old');
      ELSIF (v_service_id is NULL and v_poem_metric_id is NULL) THEN
        insert into metricdata_rejected_osg(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedOsgSeq.nextval, in_metricstatus, in_metricname, in_serviceuri, v_servicetype, in_voname, NULL, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id and poem_sync_metric_id NULL');
      ELSIF v_service_id is NULL THEN
        insert into metricdata_rejected_osg(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedOsgSeq.nextval, in_metricstatus, in_metricname, in_serviceuri, v_servicetype, in_voname, NULL, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id NULL');
     ELSIF v_poem_metric_id is NULL THEN
        insert into metricdata_rejected_osg(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedOsgSeq.nextval, in_metricstatus, in_metricname, in_serviceuri, v_servicetype, in_voname, NULL, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'poem_sync_metric_id NULL');
      END IF;

  END;

  PROCEDURE loadmetricdatatospool (in_serviceflavour IN VARCHAR2, in_metricname IN VARCHAR2, in_metricstatus IN VARCHAR2,
                 in_summarydata IN VARCHAR2, in_detailsdata in CLOB, in_voname IN VARCHAR2, in_fqan IN VARCHAR2,
                 in_hostname IN VARCHAR2, in_timestamp IN VARCHAR2, in_gatheredat IN VARCHAR2, metric_rejected_age_days IN INT) AS

  v_gatheredat_id INT;
  v_serviceflavour_name VARCHAR2(100);
  v_serviceflavour_id INT;
  v_metricstatus_id INT;
  v_poem_metric_id INT;
  v_fqan_id INT;
  v_vo_id INT;
  v_service_id INT;
  v_metricdetail_id INT;
  intFoundPos INT;
  strElement VARCHAR2(100);
  v_check_time INT;
  v_insert_time INT;
  v_metric_age_days INT;
  /*Temporary hack - See: https://savannah.cern.ch/bugs/?62833*/
  CURSOR flavourCurs IS select decode(COLUMN_VALUE, 'sBDII', 'Site-BDII' /*, 'SRMv2', 'SRMv1'*/, COLUMN_VALUE) from table(split(in_serviceflavour));

  BEGIN
    OPEN flavourCurs;

    LOOP

      FETCH flavourCurs INTO v_serviceflavour_name;
      EXIT WHEN flavourCurs%NOTFOUND;

      v_serviceflavour_name := trim(v_serviceflavour_name);
      v_poem_metric_id := getPoemSyncMetricIdForName(in_metricname);
      v_metricstatus_id := getMetricStatusIdForName(in_metricstatus);
      v_vo_id := getVoIdForName(in_voname);
      v_fqan_id := getOrAddFqan(in_fqan);
      v_serviceflavour_id := getServiceFlavourIdForName(v_serviceflavour_name);
      v_service_id := getServiceIdForHostAndFlavour(in_hostname, v_serviceflavour_id);
      v_gatheredat_id := getOrAddGatheredAt(in_gatheredat);
      v_check_time := w3ctime_to_uts(in_timestamp);
      v_insert_time := to_unixts(current_timestamp);
      v_metric_age_days := floor((v_insert_time - v_check_time) / 86400);

      IF v_service_id is not NULL and v_poem_metric_id is not NULL AND v_metric_age_days < metric_rejected_age_days THEN
        insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, service_id,
                                       vo_id, fqan_id, summarydata, check_time, insert_time, poem_sync_metric_id)
          VALUES (MetricData_SpoolSeq.nextval, v_gatheredat_id, v_metricstatus_id, in_detailsdata, v_service_id, v_vo_id, v_fqan_id, in_summarydata,
                  v_check_time, v_insert_time, v_poem_metric_id);
      ELSIF (v_metric_age_days >= metric_rejected_age_days) THEN
        INSERT INTO metricdata_rejected  (id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'metric too old');
      ELSIF (v_service_id is NULL and v_poem_metric_id is NULL) THEN
        insert into metricdata_rejected(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id and poem_sync_metric_id NULL');
      ELSIF v_service_id is NULL THEN
        insert into metricdata_rejected(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'service_id NULL');
      ELSIF v_poem_metric_id is NULL THEN
        insert into metricdata_rejected(id, metricstatus, metric, service, service_flavour, vo, fqan, summarydata, gatheredat, check_time, insert_time, reason)
        VALUES (MetricData_RejectedSeq.nextval, in_metricstatus, in_metricname, in_hostname, in_serviceflavour, in_voname, in_fqan, in_summarydata, in_gatheredat, v_check_time, v_insert_time, 'poem_sync_metric_id NULL');
      END IF;

    END LOOP;

  END;


  PROCEDURE loadmetricdatafromspool(maxAgeInHours IN INT, ageInMinutes IN INT, startThreshold IN INT, 
  maxMinsLatestEntry IN INT) AS

    v_maxCheckTime INT;
    v_minCheckTime INT;
    v_detailsdata CLOB;
    v_rowCounter INT;
    v_iLoop INT;
    v_timeLatestEntry INT;
    v_maxTimeLatestEntry INT;
    v_rowsInSpool INT;

	CURSOR metricResultsCurs IS select /*+ index_rs_asc(A METRICDATA_S_CHECK_TIME_IDX) */ 
	id, gatheredat_id, metricstatus_id, poem_sync_metric_id, service_id,
	fqan_id, vo_id, summarydata, check_time, insert_time
	from metricdata_spool A
	    where check_time < v_maxCheckTime and check_time >= v_minCheckTime
	    order by check_time;
	metricResultRec metricResultsCurs%ROWTYPE;

    CURSOR metricInsertsCurs IS SELECT number_of_records FROM metricstore_curr_load_m
      ORDER BY timestamp DESC;
    metricInsertsRec metricInsertsCurs%ROWTYPE;

  BEGIN

    logger('loadmetricdata', 'DEBUG', 'Begin: loadmetricdata');
    -- IBUCUR
    OPEN metricInsertsCurs;

    FETCH metricInsertsCurs INTO metricInsertsRec; -- this gives the number of metrics inserted in the
      -- current minute in METRICDATA_SPOOL
    FETCH metricInsertsCurs INTO metricInsertsRec; -- this gives the number of metrics inserted in the
      -- previous minute in METRICDATA_SPOOL... this is what we need

    IF metricInsertsCurs%NOTFOUND THEN
 		logger('loadmetricdata', 'WARNING',  'There is no data in METRICSTORE_CURR_LOAD_M to condition the start of this procedure. The procedure has been executed anyway.');
    ELSIF metricInsertsRec.number_of_records > startThreshold THEN
      CLOSE metricInsertsCurs;
      logger('loadmetricdata', 'WARNING',   'The number of records inserted into METRICDATA_SPOOL is over the defined threshold: ' || startThreshold ||  '. The execution of this procedure has been halted until the next call from job SAM_MS_LOADDATA.');
      RETURN; -- if the number of records created per minute is over the threshold
        -- the procedure does not execute... it waits 3 minutes until SAM_MS_LOADDATA
        -- calls it again
    END IF;

    CLOSE metricInsertsCurs;
    -- / IBUCUR
    
    
    v_rowsInSpool := 0;
    select count(*) into v_rowsInSpool from metricdata_spool;
    if v_rowsInSpool = 0 then /*There is no data in metricdata_spool. This may mean consumers are down*/
    	logger('loadmetricdata', 'WARNING', 'The number of records inserted into METRICDATA_SPOOL is ZERO. The execution of this procedurem has been halted until the next call from job SAM_MS_LOADDATA.');
	    RETURN; -- the procedure does not execute... it waits 3 minutes until SAM_MS_LOADDATA calls it again
    end if;
    
    v_maxTimeLatestEntry := to_unixts(sys_extract_utc(current_timestamp)) - maxMinsLatestEntry*60;
    select max(check_time) into v_timeLatestEntry from metricdata_spool;
    if v_timeLatestEntry < v_maxTimeLatestEntry then 
	    /*Data in metricdata_spool is older than maxMinsLatestEntry minutes.
	    This may mean we receive data that was stuck on the brokers*/
    	logger('loadmetricdata', 'WARNING', 'Data in METRICDATA_SPOOL is older than  ' || maxMinsLatestEntry || 'minutes. The execution of this procedure has been halted until the next call from job SAM_MS_LOADDATA.');
	    RETURN; -- the procedure does not execute... it waits 3 minutes until SAM_MS_LOADDATA calls it again
    end if;


	BEGIN

      select max(check_time) into v_minCheckTime from DataLoader_LastCheckTime;

      exception
        when NO_DATA_FOUND
          then v_minCheckTime := 0;
	END;

    v_maxCheckTime := to_unixts(sysdate - ageInMinutes/(24*60));

    logger('loadmetricdata', 'DEBUG', 'Begin: findMetricsForStatusRecalc');
    findMetricsForStatusRecalc(v_maxCheckTime, v_minCheckTime);
    logger('loadmetricdata', 'DEBUG', 'Begin: findMetricsForStatusRecalc');

    logger('loadmetricdata', 'DEBUG', 'Begin: load of data from metricdata_spool');
    OPEN metricResultsCurs;

    v_iLoop := 0;
    v_rowCounter := 0;
    LOOP

      FETCH metricResultsCurs INTO metricResultRec;
      EXIT WHEN metricResultsCurs%NOTFOUND;

      BEGIN
         select detailsdata into v_detailsdata from metricdata_spool where id = metricResultRec.id;

         v_rowCounter := v_rowCounter + 1;
         insertRowToMetricData(metricResultRec.service_id, metricResultRec.poem_sync_metric_id, metricResultRec.metricstatus_id, metricResultRec.summarydata,
                   v_detailsdata, metricResultRec.fqan_id, metricResultRec.vo_id, metricResultRec.gatheredat_id,
                   metricResultRec.check_time, metricResultRec.insert_time, false, maxAgeInHours);
         v_iLoop := v_iLoop + 1;

        exception
          when DUP_VAL_ON_INDEX then
            dbms_output.put_line('Row already exists: ' || metricResultRec.id);
             logger('loadMetricDataWithRecalc', 'ERROR', 'Row already exists: ' || metricResultRec.id);
          when NO_DATA_FOUND then
            insert into metricdata_spool_rejected select * from metricdata_spool where id = metricResultRec.id;
             logger('loadMetricDataWithRecalc', 'ERROR', 'NO_DATA_FOUND');
      END;


      delete from metricdata_spool where id = metricResultRec.id;

      v_iLoop := v_iLoop + 1;

      IF v_iLoop >= 1000 THEN
        commit;
        logger('loadmetricdata', 'DEBUG', 'After commit of ' || v_iloop || ' records.');
        v_iLoop := 0;
      END IF;

    END LOOP;

    commit;
    logger('loadmetricdata', 'DEBUG', 'After commit of ' || v_iloop || ' records.');

    CLOSE metricResultsCurs;
    logger('loadmetricdata', 'DEBUG', 'End: load of data from metricdata_spool. ' || v_rowCounter || ' records loaded.');


    logger('loadmetricdata', 'DEBUG', 'Begin: loadMetricDataWithRecalc');
    loadMetricDataWithRecalc (v_maxCheckTime, v_minCheckTime);
    logger('loadmetricdata', 'DEBUG', 'End: loadMetricDataWithRecalc');

    delete from DataLoader_LastCheckTime;
    insert into DataLoader_LastCheckTime(check_time) values(v_maxCheckTime);
    commit;

    logger('loadmetricdata', 'DEBUG', 'Begin: insertMissingMetrics');
    insertMissingMetrics(maxAgeInHours, ageInMinutes);
    logger('loadmetricdata', 'DEBUG', 'End: insertMissingMetrics');

    logger('loadmetricdata', 'DEBUG', 'End: loadmetricdata');
  END;

  PROCEDURE insertRowToMetricData(in_service_id IN INT, in_poem_metric_id IN INT, in_metricstatus_id IN INT, in_summarydata IN VARCHAR,
                   in_detailsdata IN CLOB, in_fqan_id IN INT, in_vo_id IN INT, in_gatheredat_id IN INT, in_check_time IN INT, in_insert_time IN INT,
                   in_recalculationNeeded IN BOOLEAN, in_maxAgeOfTupleInHours IN INT) AS


    v_metricdetail_id INT;
    fqanName VARCHAR(256);
    v_status_id INT;
    v_num_row INT;
    v_poem_fqan_id INT;

  BEGIN
  
    v_status_id := 0;
    v_num_row := 0;
      

    if in_detailsdata is not null then
      insert into metricdetails(id, detail, timestamp)
      values (MetricDetailsSeq.nextval, in_detailsdata, in_insert_time);

      select MetricDetailsSeq.currval into v_metricdetail_id from dual;
    end if;

    fqanName := '';
    if in_fqan_id is not null then
      select name into fqanName from fqan where id = in_fqan_id;
    end if;
    
    select id into v_status_id from metricstatus where description = 'REMOVED';
    
    select count(*) into v_num_row
      from mrs_bootstrapper
      where metric_id = in_poem_metric_id
      and service_id = in_service_id
      and (nvl(vo_id, -1) = nvl(in_vo_id, -1) or vo_id is null)
      and (nvl(fqan_id, -1) = nvl(in_fqan_id, -1) or fqan_id is null);
      
      if v_num_row > 0 or in_metricstatus_id = v_status_id then
          insert into metricdata(id, metricstatus_id, metricdetail_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
          VALUES (MetricDataSeq.nextval, in_metricstatus_id, v_metricdetail_id, in_poem_metric_id, in_service_id, in_fqan_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time);

         calculateStatusChange(in_service_id, in_metricstatus_id, in_check_time, in_vo_id, in_recalculationNeeded, in_maxAgeOfTupleInHours, in_poem_metric_id, fqanName);
      else
           insert into metricdata_spool_rejected(id, metricstatus_id, poem_sync_metric_id, service_id, fqan_id, vo_id, summarydata, gatheredat_id, check_time, insert_time)
          VALUES (MetricDataSeq.nextval, in_metricstatus_id, in_poem_metric_id, in_service_id, in_fqan_id, in_vo_id, in_summarydata, in_gatheredat_id, in_check_time, in_insert_time);
     		 logger('insertRowToMetricData', 'WARNING', 'Metric not in MRS bootstrapper --> REJECTED (metric_id ' || in_poem_metric_id || ' service_id ' || in_service_id || ' fqan_id ' || nvl(in_fqan_id, -1) || ' vo_id '  || nvl(in_vo_id, -1) || ')');
      end if;

  END;


 PROCEDURE insertMissingMetrics(maxAgeInHours INTEGER, ageInMinutes INTEGER) AS

 v_poem_metric_id INTEGER;
 v_service_id INTEGER;
 v_vo_id INTEGER;
 v_fqan_id INTEGER;
 v_fqan_name VARCHAR2(256);

 CURSOR oldMetricsCurs IS select m.service_id, m.vo_id, m.fqan_id, m.poem_sync_metric_id from metricdata_latest m
   where
        m.check_time <= (select to_unixts(current_timestamp - (1/24)*maxAgeInHours - (1/1440)*ageInMinutes) from dual)
    and m.service_id in (select id from service where isdeleted = 'N')
    and m.metricstatus_id not in (select id from metricstatus where description = 'REMOVED');

 CURSOR newMissingMetricsCurs is
    select service_id, vo_id, fqan_id, metric_id from
     (select distinct p.service_id, p.vo_id, p.fqan_id, p.metric_id, m.poem_sync_metric_id from
        mrs_bootstrapper p left join metricdata_latest m on m.poem_sync_metric_id = p.metric_id
          and m.service_id = p.service_id
          and (nvl(m.vo_id, -1) = nvl(p.vo_id, -1) or p.vo_id is null)
           and (nvl(p.fqan_id, -1) = nvl (m.fqan_id, -1) or p.fqan_id is null)
     ) a, service s where a.service_id = s.id and a.poem_sync_metric_id is null and s.isdeleted = 'N';


 BEGIN

   OPEN oldMetricsCurs;

   LOOP

     FETCH oldMetricsCurs INTO v_service_id, v_vo_id, v_fqan_id, v_poem_metric_id;
     EXIT WHEN oldMetricsCurs%NOTFOUND;

     insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, service_id, vo_id, fqan_id, summarydata, check_time, insert_time, poem_sync_metric_id)
       VALUES (MetricData_SpoolSeq.nextval, NULL, (select id from metricstatus where description = 'MISSING'), NULL, v_service_id, v_vo_id, v_fqan_id, NULL, to_unixts(current_timestamp - (1/1440)*ageInMinutes), to_unixts(current_timestamp), v_poem_metric_id);

     logger('oldMissingMetricsCurs', 'DEBUG', 'Inserting metric ' || v_poem_metric_id || ' for service ' || v_service_id || ' and vo ' || nvl(v_vo_id, 0) || ' and fqan ' || v_fqan_id);



   END LOOP;

   CLOSE oldMetricsCurs;

   OPEN newMissingMetricsCurs;

   LOOP

     FETCH newMissingMetricsCurs INTO v_service_id, v_vo_id, v_fqan_id, v_poem_metric_id;
     EXIT WHEN newMissingMetricsCurs%NOTFOUND;
     
      insert into metricdata_spool(id, gatheredat_id, metricstatus_id, detailsdata, service_id, vo_id, fqan_id, summarydata, check_time, insert_time, poem_sync_metric_id)
     VALUES (MetricData_SpoolSeq.nextval ,NULL, (select id from metricstatus where description = 'MISSING'), NULL, v_service_id, v_vo_id, v_fqan_id, NULL, to_unixts(current_timestamp - (1/1440)*ageInMinutes), to_unixts(current_timestamp), v_poem_metric_id);

   END LOOP;

   CLOSE newMissingMetricsCurs;

   commit;

 END;

  FUNCTION getServiceStatus(
    profileId IN INTEGER,
    serviceId IN INTEGER,
    checkTime IN INTEGER) RETURN INTEGER AS

    statusId INTEGER;
  BEGIN

    select metricstatus_id into statusId from STATUSCHANGE_SERVICE_PROFILE where timestamp =
      (select max(timestamp) from STATUSCHANGE_SERVICE_PROFILE
      where service_id = serviceId and poem_sync_profile_id = profileId
      and timestamp <= checkTime)
      and service_id = serviceId and poem_sync_profile_id = profileId;

    RETURN statusId;

  exception

    when NO_DATA_FOUND

    then return NULL;

  END;


 PROCEDURE findCurrentStatusMetricService (serviceId INT, voId INT, metricstatusId INT, check_time INT, poem_sync_metricId INT,
  old_metricstatus_id OUT INT,
  old_timestamp_st_change OUT INT,
  num_row OUT INT) AS
    
BEGIN

  select count(*) INTO num_row
  from statuschange_metric_service
  where  (nvl(vo_id, -1) = nvl(voId, -1) or vo_id is null)
    AND service_id = serviceId
    AND timestamp <= check_time
    AND poem_sync_metric_id = poem_sync_metricId;


  IF num_row > 0 THEN


  SELECT max(timestamp) into old_timestamp_st_change FROM statuschange_metric_service
          WHERE (nvl(vo_id, -1) = nvl(voId, -1) or vo_id is null)
            AND poem_sync_metric_id = poem_sync_metricId
            AND service_id = serviceId
            AND timestamp <= check_time;
            
  SELECT max(metricstatus_id) INTO old_metricstatus_id
    FROM statuschange_metric_service
    WHERE (nvl(vo_id, -1) = nvl(voId, -1) or vo_id is null)
      AND service_id = serviceId
      AND poem_sync_metric_id = poem_sync_metricId
      AND timestamp = old_timestamp_st_change;

    /*SELECT metricstatus_id, timestamp INTO old_metricstatus_id, old_timestamp_st_change
    FROM statuschange_metric_service
    WHERE nvl(vo_id, -1) = nvl(voId, -1)
      AND service_id = serviceId
      AND poem_sync_metric_id = poem_sync_metric_id
      AND timestamp =
        (SELECT max(timestamp) FROM statuschange_metric_service
          WHERE nvl(vo_id, -1) = nvl(voId, -1)
            AND poem_sync_metric_id = poem_sync_metric_id
            AND service_id = serviceId
            AND timestamp <= check_time);*/ 
 
  END IF;

END;

PROCEDURE calculateStatusChangeMetric(serviceId INT, voId INT, metricstatusId INT, check_time INT, poem_sync_metric_id INT) AS

  num_row INTEGER;
  old_metricstatus_id INTEGER;
  old_timestamp_st_change INTEGER;

BEGIN

  num_row := 0;

  findCurrentStatusMetricService(serviceId, voId, metricstatusId, check_time, poem_sync_metric_id, old_metricstatus_id, old_timestamp_st_change, num_row);

  IF num_row > 0 THEN
    IF ((old_timestamp_st_change != check_time) AND (metricstatusId != old_metricstatus_id)) THEN
      INSERT INTO statuschange_metric_service(id, service_id, vo_id, metricstatus_id, timestamp, insert_time, poem_sync_metric_id)
      VALUES(statuschange_metric_s_seq.nextval, serviceId, voId, metricstatusId, check_time, to_unixts(current_timestamp), poem_sync_metric_id);
    ELSIF (getMetricStatusWeightForId(metricstatusId) > getMetricStatusWeightForId(old_metricstatus_id)) THEN
       UPDATE statuschange_metric_service SET metricstatus_id = metricstatusId, insert_time = to_unixts(current_timestamp)
         WHERE service_id = serviceId and timestamp = check_time
           AND nvl(vo_id, -1) = nvl(voId, -1) AND poem_sync_metric_id = poem_sync_metric_id;
    END IF;
  ELSE
    INSERT INTO statuschange_metric_service(id, service_id, vo_id, metricstatus_id, timestamp, insert_time, poem_sync_metric_id)
    VALUES(statuschange_metric_s_seq.nextval, serviceId, voId, metricstatusId, check_time, to_unixts(current_timestamp), poem_sync_metric_id);
  END IF;

END;


PROCEDURE purgeMetricstore (        -- all values are given in days
    logger_kept IN INT,             -- logger
    mdata_rej_kept IN INT,          -- metricdata_rejected, metricdata_rejected_osg
    mdata_spool_rej_kept IN INT,    -- metricdata_spool_rejected
    mdata_latest_kept IN INT,       -- metricdata_latest
    mstore_clhd IN INT )            -- metricstore_curr_load_h_det
    IS
    cur_ts INT := to_unixts(sys_extract_utc(current_timestamp));
    day_ts INT := 86400;            -- one day has 86400 seconds
    batch_size INT := 10000;        -- batch size for commits
   
    BEGIN
        logger('purgeMetricstore', 'DEBUG', 'DB cleanup');

        LOOP
            DELETE FROM logger
                WHERE inserttime < (sysdate - logger_kept)
                    AND ROWNUM <= batch_size;
            EXIT WHEN (SQL%ROWCOUNT=0);
            COMMIT;
        END LOOP;

        LOOP         
            DELETE FROM metricdata_rejected
                WHERE check_time < (cur_ts - day_ts * mdata_rej_kept)
                    AND ROWNUM <= batch_size;
            EXIT WHEN (SQL%ROWCOUNT=0);
            COMMIT;
        END LOOP;        

        LOOP
            DELETE FROM metricdata_rejected_osg
                WHERE check_time < (cur_ts - day_ts * mdata_rej_kept)
                    AND ROWNUM <= batch_size;
            EXIT WHEN (SQL%ROWCOUNT=0);
            COMMIT;
        END LOOP;

        LOOP
            DELETE FROM metricdata_spool_rejected
                WHERE check_time < (cur_ts - day_ts * mdata_spool_rej_kept)
                    AND ROWNUM <= batch_size;
            EXIT WHEN (SQL%ROWCOUNT=0);
            COMMIT;
        END LOOP;

        LOOP
            DELETE FROM metricdata_latest
                WHERE check_time < (cur_ts - day_ts * mdata_latest_kept) 
                    AND ROWNUM <= batch_size;
            EXIT WHEN (SQL%ROWCOUNT=0);
            COMMIT;
        END LOOP;

        LOOP
            DELETE FROM metricstore_curr_load_h_det
                WHERE to_unixts(timestamp) < (cur_ts - day_ts * mstore_clhd)
                        AND ROWNUM <= batch_size;
            EXIT WHEN (SQL%ROWCOUNT=0);
            COMMIT;
        END LOOP;        

    END;

------------------------------------------------------------
-- Begin: Bootstrapping
------------------------------------------------------------

FUNCTION getMRSfqan_id (in_poem_sync_fqan_id in INTEGER) RETURN INTEGER AS
    v_fqan_id INT;
  BEGIN
    IF in_poem_sync_fqan_id is NULL THEN
      BEGIN
        return null;
      END;
    END IF;
    select m.id into v_fqan_id from fqan m, poem_sync_fqan p
    where lower(m.name) = lower(p.fqan)
      and p.id = in_poem_sync_fqan_id;

    RETURN v_fqan_id;
  EXCEPTION
    when NO_DATA_FOUND
    then return NULL;
  END;


------------------------------------------------------------
-- End: Bootstrapping
------------------------------------------------------------

END DataLoader;
/
