create or replace view metric_history_in_poem_s_prof as
  select profile_name, namespace,  vo.voname as defined_by_vo, hostname, service_type, service_flavour, metric_exec_time,
    metric_name, fqan, tested_by_vo, metric_status, metric_output_summary, gathered_at, metric_result_insert_time
  from (
    select pp.name as profile_name, pp.atp_vo_id, s.hostname, st.serviceabbr as service_type, t.flavourname as service_flavour, 
          l.check_time as metric_exec_time, m.name as metric_name, f.name as fqan, v.voname as tested_by_vo, ms.description as metric_status, 
          l.summarydata as metric_output_summary, g.name as gathered_at, l.insert_time as metric_result_insert_time, pn.name as namespace 
          from metricdata l, vo v, fqan f, 
          service s, service_type st, service_type_flavour t, poem_sync_metric m, metricstatus ms, gatheredat g,  
          poem_sync_profile pp, poem_sync_namespace pn, poem_sync_p_m_instances ppm, poem_sync_metricinstance pm
    where 
      (g.id(+) = l.gatheredat_id)
      and st.id = t.service_type_id
      and ms.id = l.metricstatus_id
      and m.id = l.poem_sync_metric_id
      and t.id = s.flavour_id
      and s.id = l.service_id
      and (f.id(+) = l.fqan_id)  
      and (v.id(+) = l.vo_id)
      and ppm.profile_id = pp.id
      and pm.atp_service_type_flavour_id = s.flavour_id
      and pm.metric_id = l.poem_sync_metric_id
      and (pm.atp_vo_id = l.vo_id or l.vo_id is null)
      and pm.id = ppm.metricinstance_id
      and pp.namespace_id = pn.id
  ) a, vo
  where a.atp_vo_id = vo.id;

alter table mrs_bootstrapper drop constraint bootstrapper_fk3;
alter table mrs_bootstrapper rename column fqan_id  to poem_sync_fqan_id;
alter table mrs_bootstrapper add fqan_id INT;


alter table mrs_bootstrapper_history drop constraint bootstrapper_h_fk3;
alter table mrs_bootstrapper_history rename column fqan_id  to poem_sync_fqan_id;
alter table mrs_bootstrapper_history add fqan_id INT;


 create FUNCTION addFqan(in_name VARCHAR2) RETURN INT
  AS

    v_id INT DEFAULT NULL;
  BEGIN

    v_id := NULL;

    IF in_name is not NULL THEN
      BEGIN
        select id into v_id from fqan where lower(name) = lower(in_name);

        EXCEPTION when NO_DATA_FOUND THEN
          insert into fqan(id, name)
            values(fqan_seq.nextval, in_name);

        select id into v_id from fqan where lower(name) = lower(in_name);

      END;
    END IF;

    RETURN v_id;

  END;
  /


create or replace PROCEDURE migBootstrapper (num_records integer) as
  counter integer;
  v_poem_sync_fqan_id integer;
  v_fqan_name VARCHAR(256);
  
  BEGIN
  counter := 0;
  
  BEGIN   
    FOR tuple IN
      (select distinct(m.poem_sync_fqan_id), p.fqan
        from mrs_bootstrapper m, poem_sync_fqan p
        where m.poem_sync_fqan_id is not null and m.poem_sync_fqan_id=p.id and m.fqan_id is null)
      LOOP
        BEGIN
          UPDATE mrs_bootstrapper set fqan_id = addFqan(tuple.fqan)
            where poem_sync_fqan_id = tuple.poem_sync_fqan_id;
          counter := counter + 1;
          IF counter=num_records THEN
            commit;
            counter := 0;
          END IF;
        END;
      END LOOP;
      commit;    
      counter := 0; 
    END;
  
    BEGIN   
    FOR tuple IN
      ( select distinct(m.poem_sync_fqan_id), p.fqan
        from mrs_bootstrapper_history m, poem_sync_fqan p
        where m.poem_sync_fqan_id is not null and m.poem_sync_fqan_id=p.id and m.fqan_id is null )
      LOOP
        BEGIN
          UPDATE mrs_bootstrapper_history set fqan_id = addFqan(tuple.fqan)
            where poem_sync_fqan_id = tuple.poem_sync_fqan_id;
          counter := counter + 1;
          IF counter=num_records THEN
            commit;
            counter := 0;
          END IF;
        END;
      END LOOP;
      commit;     
  END;

END;
/


call migBootstrapper(1000);


drop procedure migBootstrapper;
drop function addFqan;


alter table mrs_bootstrapper drop column poem_sync_fqan_id;
alter table mrs_bootstrapper add constraint  bootstrapper_fk3
  foreign key (fqan_id)
  references fqan (id);


alter table mrs_bootstrapper_history drop column poem_sync_fqan_id;
alter table mrs_bootstrapper_history add constraint  bootstrapper_h_fk3
  foreign key (fqan_id)
  references fqan (id);


declare 
  v_exists integer;

begin
-- mrs_bootstrapper_s_id
select count(*) into v_exists from user_indexes where lower(index_name)='mrs_bootstrapper_s_id';
if v_exists = 0 then
    execute immediate 'create index mrs_bootstrapper_s_id on mrs_bootstrapper (service_id)';
end if;
-- mrs_bootstrapper_m_id
select count(*) into v_exists from user_indexes where lower(index_name)='mrs_bootstrapper_m_id';
if v_exists = 0 then
    execute immediate 'create index mrs_bootstrapper_m_id on mrs_bootstrapper (metric_id)';
end if;
-- mrs_bootstrapper_f_id
select count(*) into v_exists from user_indexes where lower(index_name)='mrs_bootstrapper_f_id';
if v_exists = 0 then
    execute immediate 'create index mrs_bootstrapper_f_id on mrs_bootstrapper (fqan_id)';
end if;
-- mrs_bootstrapper_v_id
select count(*) into v_exists from user_indexes where lower(index_name)='mrs_bootstrapper_v_id';
if v_exists = 0 then
    execute immediate 'create index mrs_bootstrapper_v_id on mrs_bootstrapper (vo_id)';
end if;
-- mrs_bootstrapper_t_id
select count(*) into v_exists from user_indexes where lower(index_name)='mrs_bootstrapper_t_id';
if v_exists = 0 then
    execute immediate 'create index mrs_bootstrapper_t_id on mrs_bootstrapper (timestamp)';
end if;

-- statuschange_m_s_s_id
select count(*) into v_exists from user_indexes where lower(index_name)='statuschange_m_s_s_id';
if v_exists = 0 then
    execute immediate 'create index statuschange_m_s_s_id on statuschange_metric_service (service_id)';
end if;
-- statuschange_m_s_v_id
select count(*) into v_exists from user_indexes where lower(index_name)='statuschange_m_s_v_id';
if v_exists = 0 then
    execute immediate 'create index statuschange_m_s_v_id on statuschange_metric_service (vo_id)';
end if;
-- statuschange_m_s_t_id
select count(*) into v_exists from user_indexes where lower(index_name)='statuschange_m_s_t_id';
if v_exists = 0 then
    execute immediate 'create index statuschange_m_s_t_id on statuschange_metric_service (timestamp)';
end if;
-- statuschange_m_s_ms_id
select count(*) into v_exists from user_indexes where lower(index_name)='statuschange_m_s_ms_id';
if v_exists = 0 then
    execute immediate 'create index statuschange_m_s_ms_id on statuschange_metric_service (metricstatus_id)';
end if;
end;
/


