create or replace view metric_history_in_poem_s_prof as
 select profile_name, namespace,  vo.voname as defined_by_vo, hostname, service_flavour, metric_exec_time,
    metric_name, fqan, tested_by_vo, metric_status, metric_output_summary, gathered_at, metric_result_insert_time
  from (                                                                        
    select pp.name as profile_name, pp.atp_vo_id, s.hostname, t.flavourname as service_flavour, 
          l.check_time as metric_exec_time, m.name as metric_name, f.name as fqan, v.voname as tested_by_vo, ms.description as metric_status, 
          l.summarydata as metric_output_summary, g.name as gathered_at, l.insert_time as metric_result_insert_time, pn.name as namespace 
          from metricdata l, vo v, fqan f,                                      
          service s, service_type_flavour t, poem_sync_metric m, metricstatus ms, gatheredat g,  
          poem_sync_profile pp, poem_sync_namespace pn, poem_sync_p_m_instances ppm, poem_sync_metricinstance pm
    where                                                                       
      (g.id(+) = l.gatheredat_id)                                               
      and ms.id = l.metricstatus_id                                             
      and m.id = l.poem_sync_metric_id                                          
      and t.id = s.flavour_id                                                   
      and s.id = l.service_id                                                   
      and (f.id(+) = l.fqan_id)                                                 
      and (v.id(+) = l.vo_id)                                                   
      and ppm.profile_id = pp.id                                                
      and pm.atp_service_type_flavour_id = s.flavour_id                         
      and pm.metric_id = l.poem_sync_metric_id                                  
      and (pm.atp_vo_id = l.vo_id or l.vo_id is null)                           
      and pm.id = ppm.metricinstance_id                                         
      and pp.namespace_id = pn.id                                               
  ) a, vo                                                                       
  where a.atp_vo_id = vo.id; 

/* disable triggers responsible for the stchg_serv_prof_maxts table */
ALTER TRIGGER stchg_serv_prof_maxts_dtr DISABLE;
ALTER TRIGGER stchg_serv_prof_maxts_tr  DISABLE;
ALTER TRIGGER stchg_serv_prof_maxts_utr DISABLE;

DROP VIEW statuschange_service_profile_l; 
CREATE MATERIALIZED VIEW statuschange_service_profile_l 
  (id, service_id, poem_sync_profile_id, 
  metricstatus_id, detail, timestamp) 
  REFRESH COMPLETE 
  START WITH SYSDATE 
  NEXT SYSDATE + 5/1440 
  AS 
  SELECT t.id, t.service_id, t.poem_sync_profile_id, t.metricstatus_id, t.detail, t.timestamp 
  FROM statuschange_service_profile t 
  JOIN ( 
    SELECT service_id, poem_sync_profile_id, MAX(timestamp) AS timestamp 
    FROM statuschange_service_profile 
    GROUP BY service_id, poem_sync_profile_id 
    ) g 
  ON t.service_id = g.service_id 
  AND t.poem_sync_profile_id = g.poem_sync_profile_id 
  AND t.timestamp = g.timestamp; 

DROP PROCEDURE metricstore_performance_tuning;
