create or replace view fqans_in_regions as
  select distinct get_roc_name(service_id) as roc_name, fqan_id from fqans_in_supported_services;

ALTER TABLE metricstatus 
  ADD WEIGHT INT; 

update metricstatus set weight = 1 where description = 'OK';
update metricstatus set weight = 2 where description = 'WARNING';
update metricstatus set weight = 3 where description = 'UNKNOWN';
update metricstatus set weight = 4 where description = 'MISSING';
update metricstatus set weight = 5 where description = 'CRITICAL';
update metricstatus set weight = 6 where description = 'REMOVED';

ALTER TABLE metricstatus
  modify WEIGHT NOT NULL;
  
ALTER TABLE metricstatus ADD CONSTRAINT
  uk_metricstatus_weight UNIQUE(weight);
  
create or replace view metric_history_in_profile as
SELECT a.name profile_name, d.voname defined_by_vo, l.hostname, f.serviceabbr service_type,
  e.flavourname service_flavour, h.check_time metric_exec_time,
  c.name metric_name, j.name fqan, g.voname tested_by_vo,
  i.description metric_status, h.summarydata metric_output_summary,
  k.name gathered_at, h.insert_time metric_result_insert_time
FROM profile a, profile_metric_map b, metric c, vo d, service_type_flavour e,
  service_type f, vo g, metricdata h, metricstatus i, fqan j, gatheredat k, service l
WHERE b.profile_id=a.id AND b.metric_id=c.id AND a.defining_vo_id=d.id
  AND b.metric_id=h.metric_id AND h.metricstatus_id=i.id AND h.service_id=l.id
  AND l.flavour_id=e.id AND b.service_type_flavour_id=l.flavour_id
  AND e.service_type_id=f.id AND b.test_vo_id=g.id AND h.vo_id=g.id
  AND h.fqan_id=j.id AND h.gatheredat_id=k.id;

drop table statuschangeRecalculated;
drop sequence STATUSCHANGERECALCULATEDSEQ;  
/*Missing hierarchy*/

CREATE TABLE PROFILE_HIERARCHY (	 
  ID INT NOT NULL, 
  PROFILE_ID INT NOT NULL,
  PARENT_PROFILE_ID INT NOT NULL,
  TIMESTAMP INT NOT NULL,
  PRIMARY KEY (ID), 
  CONSTRAINT UK_PROFILE_HIERARCHY UNIQUE (PROFILE_ID, PARENT_PROFILE_ID), 
  CONSTRAINT FK_PROFILE_HIERARCHY_PROFILE FOREIGN KEY (PROFILE_ID)
    REFERENCES PROFILE (ID), 
  CONSTRAINT FK_PROFILE_HIERARCHY_PARENT FOREIGN KEY (PARENT_PROFILE_ID)
	REFERENCES PROFILE (ID)
);

CREATE SEQUENCE PROFILE_HIERARCHY_SEQ;   

insert into profile_hierarchy(id, profile_id, parent_profile_id, timestamp)
values(PROFILE_HIERARCHY_SEQ.nextval, (select id from profile where name = 'ALICE_CRITICAL'), (select id from profile where name = 'ALICE'),
to_unixts(current_timestamp));

insert into profile_hierarchy(id, profile_id, parent_profile_id, timestamp)
values(PROFILE_HIERARCHY_SEQ.nextval, (select id from profile where name = 'ROC_CRITICAL'), (select id from profile where name = 'ROC'),
to_unixts(current_timestamp));

insert into profile_hierarchy(id, profile_id, parent_profile_id, timestamp)
values(PROFILE_HIERARCHY_SEQ.nextval, (select id from profile where name = 'ROC_OPERATORS'), (select id from profile where name = 'ROC'),
to_unixts(current_timestamp));

commit;
