DELIMITER //

USE `mrs`;

DROP TRIGGER IF EXISTS metricdata_spool_insert;

CREATE TRIGGER metricdata_spool_insert
AFTER INSERT ON metricdata_spool
FOR EACH ROW 
  
trig: BEGIN
	DECLARE current_hour NUMERIC DEFAULT EXTRACT(hour FROM utc_timestamp);
	DECLARE current_minute NUMERIC DEFAULT EXTRACT(minute FROM utc_timestamp);
	DECLARE this_flavour_id INT;
	DECLARE this_profile_id INT;
	DECLARE no_more_profiles BOOLEAN DEFAULT FALSE;
	DECLARE has_profile BOOLEAN DEFAULT FALSE;
	DECLARE profile_ids CURSOR FOR SELECT id FROM profile;
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET no_more_profiles = TRUE;

    IF NEW.metricstatus_id = 5 OR NEW.metricstatus_id = 6 THEN
   		LEAVE trig;
    END IF;
    
    -- updating metricstore_curr_load_m
    DELETE FROM metricstore_curr_load_m
      WHERE DATEDIFF(utc_timestamp, timestamp) > 0;
	
	 INSERT INTO metricstore_curr_load_m (id, timestamp, hour, minute, number_of_records)
      VALUES (NULL, utc_timestamp, EXTRACT(hour FROM utc_timestamp), EXTRACT(minute FROM utc_timestamp), 1)
      ON DUPLICATE KEY UPDATE number_of_records = number_of_records + 1;
      
      
    -- updating metricstore_curr_load_h
    DELETE FROM metricstore_curr_load_h
   	  WHERE DATEDIFF(utc_timestamp, timestamp) > 30;
   	  
   	INSERT INTO metricstore_curr_load_h (id, timestamp, rec_date, hour, number_of_records)
   	  VALUES (NULL, utc_timestamp, utc_date, EXTRACT(hour FROM utc_timestamp), 1)
   	  ON DUPLICATE KEY UPDATE number_of_records = number_of_records + 1;
   	  
   	  
   	-- updating metricstore_curr_load_h_det
    DELETE FROM metricstore_curr_load_h_det
   	  WHERE DATEDIFF(utc_timestamp, timestamp) > 30;
   	  
   	SELECT flavour_id INTO this_flavour_id FROM service WHERE id = NEW.service_id; 

	SET has_profile = FALSE;
	SET no_more_profiles = FALSE;
	
	OPEN profile_ids;
		
	profile_loop : WHILE(no_more_profiles = FALSE) DO
	
	  FETCH profile_ids INTO this_profile_id;
	  IF no_more_profiles = TRUE THEN
	    LEAVE profile_loop;
	  END IF;
	  
	  IF checkIfCalculationNeeded(this_profile_id, NEW.service_id, NEW.metric_id, NEW.vo_id, NEW.check_time) THEN
	  	SET has_profile = TRUE;
	  	
	  	UPDATE metricstore_curr_load_h_det
	  	  SET number_of_records = number_of_records + 1
	  	  WHERE ifnull(ngi_name, 'null') = ifnull(get_roc_name(NEW.service_id), 'null')
	  	    AND profile_id = this_profile_id
	  	    AND flavour_id = this_flavour_id
	  	    AND EXTRACT(hour from utc_timestamp) = hour
	  	    AND rec_date = utc_date;
	    -- if update unsuccesful (row does not exist), create new row
	  	IF ROW_COUNT() = 0 THEN
	  	  INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, flavour_id, timestamp, hour, rec_date, number_of_records)
	  	    VALUES (NULL, get_roc_name(NEW.service_id), this_profile_id, this_flavour_id, utc_timestamp, EXTRACT(hour from utc_timestamp), utc_date, 1);
	  	END IF;
	  END IF;
	  
	END WHILE profile_loop;

	CLOSE profile_ids;
	

	IF has_profile = FALSE THEN
	  -- insert on duplicate key does not work here because ngi_name can be null
	  UPDATE metricstore_curr_load_h_det
	  	SET number_of_records = number_of_records + 1
	  	WHERE ifnull(ngi_name, 'null') = ifnull(get_roc_name(NEW.service_id), 'null')
	  	  AND profile_id is NULL
	  	  AND flavour_id = this_flavour_id
	  	  AND EXTRACT(hour from utc_timestamp) = hour
	  	  AND rec_date = utc_date;
	  -- if update unsuccesful (row does not exist), create new row
	  IF ROW_COUNT() = 0 THEN
	    INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, flavour_id, timestamp, hour, rec_date, number_of_records)
	      VALUES (NULL, get_roc_name(NEW.service_id), NULL, this_flavour_id, utc_timestamp, EXTRACT(hour from utc_timestamp), utc_date, 1);
	  END IF;
	END IF;
END //

DELIMITER ;


