CREATE TABLE Logger (
  id INT NOT NULL,
  inserttime timestamp default sysdate NOT NULL,
  proc_name VARCHAR2(256),
  severity VARCHAR2(50),
  message VARCHAR2(1024)
);

CREATE SEQUENCE logger_seq
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;

CREATE TABLE "METRICSTORE_CURR_LOAD_H"
(
    "ID" NUMBER 
    	NOT NULL,
    "REC_DATE" VARCHAR2(20 BYTE)
    	DEFAULT to_char(sys_extract_utc(CURRENT_TIMESTAMP), 'DD-MON-YYYY')
    	NOT NULL,
    "HOUR" NUMBER(2,0) 
    	DEFAULT EXTRACT(HOUR FROM CURRENT_TIMESTAMP) 
    	NOT NULL,
    "TIMESTAMP" TIMESTAMP (6) 
    	DEFAULT SYS_EXTRACT_UTC(CURRENT_TIMESTAMP) 
    	NOT NULL,
    "NUMBER_OF_RECORDS" NUMBER 
    	NOT NULL,
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_CHK1" 
    	CHECK (HOUR >= 0 AND HOUR <= 23
  			AND NUMBER_OF_RECORDS >= 0),
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_PK" 
    	PRIMARY KEY ("ID"),
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_UK1" 
    	UNIQUE ("REC_DATE", "HOUR")
);

CREATE SEQUENCE "METRICSTORE_CURR_LOAD_H_SEQ"
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;
  
  
CREATE TABLE "METRICSTORE_CURR_LOAD_M"
(
    "ID" NUMBER 
    	NOT NULL,
    "TIMESTAMP" TIMESTAMP (6) 
    	DEFAULT sys_extract_utc(CURRENT_TIMESTAMP) 
    	NOT NULL,
    "HOUR" NUMBER(2,0) 
    	DEFAULT EXTRACT(HOUR FROM CURRENT_TIMESTAMP) 
    	NOT NULL,
    "MINUTE" NUMBER(2,0) 
    	DEFAULT EXTRACT(MINUTE FROM CURRENT_TIMESTAMP) 
    	NOT NULL,
    "NUMBER_OF_RECORDS" NUMBER 
    	NOT NULL,
    CONSTRAINT "METRICSTORE_CURR_LOAD_M_PK" 
    	PRIMARY KEY ("ID"),
    CONSTRAINT "METRICSTORE_CURR_LOAD_M_UK1" 
    UNIQUE ("HOUR", "MINUTE"),
    CONSTRAINT "METRICSTORE_CURR_LOAD_M_CHK1" 
    	CHECK (HOUR >= 0 AND HOUR <= 23 
    		AND MINUTE >= 0 AND MINUTE <= 59 
    		AND NUMBER_OF_RECORDS >= 0)
);

CREATE SEQUENCE "METRICSTORE_CURR_LOAD_M_SEQ"
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;


CREATE TABLE "METRICSTORE_CURR_LOAD_H_DET"
  (
    "ID" NUMBER 
    	NOT NULL,
    "REC_DATE" VARCHAR2(20 BYTE)
    	DEFAULT to_char(sys_extract_utc(CURRENT_TIMESTAMP), 'DD-MON-YYYY')
    	NOT NULL,
    "HOUR" NUMBER(2,0) 
    	DEFAULT EXTRACT(HOUR FROM CURRENT_TIMESTAMP) 
    	NOT NULL,
    "TIMESTAMP" TIMESTAMP (6) 
    	DEFAULT SYS_EXTRACT_UTC(CURRENT_TIMESTAMP) 
    	NOT NULL,
    "NGI_NAME" VARCHAR2(20 BYTE),
    "PROFILE_ID" NUMBER,
    "FLAVOUR_ID" NUMBER 
    	NOT NULL,
    "NUMBER_OF_RECORDS" NUMBER,
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_D_CHK1" 
    	CHECK (HOUR >= 0 AND HOUR <= 23
  			AND NUMBER_OF_RECORDS >= 0),
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_D_PK" 
    	PRIMARY KEY ("ID"),
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_D_UK1" 
    	UNIQUE ("REC_DATE", "HOUR", "NGI_NAME", "PROFILE_ID", "FLAVOUR_ID"),
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_D_FK1" 
    	FOREIGN KEY ("PROFILE_ID") 
    	REFERENCES "PROFILE" ("ID"),
    CONSTRAINT "METRICSTORE_CURR_LOAD_H_D_FK2" 
    	FOREIGN KEY ("FLAVOUR_ID") 
    	REFERENCES "SERVICE_TYPE_FLAVOUR" ("ID")
);

CREATE SEQUENCE "METRICSTORE_CURR_LOAD_H_D_SEQ"
  START WITH 1
  INCREMENT BY 1
  NOCACHE
  NOCYCLE;
  
