create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_M" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 
  
DECLARE
  current_hour NUMBER := EXTRACT(hour FROM current_timestamp);
  current_minute NUMBER := EXTRACT(minute FROM current_timestamp);
  
BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  UPDATE metricstore_curr_load_m SET number_of_records = number_of_records + 1
    WHERE current_hour = hour
      AND current_minute = minute
      AND EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) = 0;
      
  IF SQL%rowcount = 0 THEN -- no update was actually made
    DELETE FROM metricstore_curr_load_m
      WHERE current_hour = hour AND current_minute = minute;
    INSERT INTO metricstore_curr_load_m (id, number_of_records) VALUES (metricstore_curr_load_m_seq.nextval, 1);
  END IF;

END;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;

  UPDATE metricstore_curr_load_h SET number_of_records = number_of_records + 1
    WHERE EXTRACT(hour FROM current_timestamp) = hour 
      AND EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) = 0;
    
  IF SQL%rowcount = 0 THEN -- no update was actually made
    DELETE FROM metricstore_curr_load_h
      WHERE EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) > 30;
    INSERT INTO metricstore_curr_load_h (id, number_of_records) VALUES (metricstore_curr_load_h_seq.nextval, 1);
  END IF;

END;
/


create or replace
TRIGGER "METRICDATA_SPOOL_INSERT_H_DET" 
AFTER INSERT ON METRICDATA_SPOOL 
FOR EACH ROW 

DECLARE
  current_hour NUMBER(2,0);
  this_flavour_id NUMBER;
  CURSOR profile_ids IS SELECT id FROM profile;
  has_profile BOOLEAN := false;

BEGIN
  IF :new.metricstatus_id = 5 OR :new.metricstatus_id = 6 THEN
    RETURN; --metrics with status MISSING or REMOVED, they do not come from messaging
  END IF;
  
  SELECT flavour_id INTO this_flavour_id FROM service WHERE id = :new.service_id; 

  FOR this_profile IN profile_ids LOOP
    IF dataloader.checkifcalculationneeded(this_profile.id, :new.service_id, :new.metric_id, :new.vo_id, :new.check_time) THEN
      UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
        WHERE EXTRACT(hour FROM current_timestamp) = hour 
          AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
          AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
          AND profile_id = this_profile.id
          AND flavour_id = this_flavour_id;
        
      IF SQL%rowcount = 0 THEN -- no update was actually made
        DELETE FROM metricstore_curr_load_h_det
          WHERE EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) > 30;
        INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, flavour_id, number_of_records) 
          VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), this_profile.id, this_flavour_id, 1);
      END IF;
      
      has_profile := true;
    END IF;
  END LOOP;
  
  IF has_profile = false THEN
    UPDATE metricstore_curr_load_h_det SET number_of_records = number_of_records + 1
      WHERE EXTRACT(hour FROM current_timestamp) = hour 
        AND to_char(sys_extract_utc(current_timestamp), 'DD-MON-YYYY') = rec_date
        AND nvl(ngi_name, 'null') = nvl(get_roc_name(:new.service_id), 'null') -- ngi_name can be null
        AND profile_id is NULL
        AND flavour_id = this_flavour_id;
    
    IF SQL%rowcount = 0 THEN -- no update was actually made
      DELETE FROM metricstore_curr_load_h_det
        WHERE EXTRACT(day FROM (sys_extract_utc(current_timestamp) - timestamp)) > 30;
      INSERT INTO metricstore_curr_load_h_det (id, ngi_name, profile_id, flavour_id,  number_of_records) 
        VALUES (metricstore_curr_load_h_d_seq.nextval, get_roc_name(:new.service_id), NULL, this_flavour_id, 1);
    END IF;
  END IF;

END;
/

